/*
 * $CHeader: cps.h 0.1 1995/12/20 13:07:30 $
 * Copyright 1993, CONVEX Computer Corporation.
 *
 * This document is copyrighted.  All rights are reserved.  This document
 * may not, in whole or part, be copied, duplicated, reproduced, translated,
 * electronically stored or reduced to machine readable form without prior
 * written consent from CONVEX Computer Corporation.
 *
 * This file defines all of the data structures, data objects and routines
 * that are exported to the user of cps libraries.
 */

#ifndef _CPS_H_DEF
#define _CPS_H_DEF

#include <stdarg.h>

#ifndef NULL
		#undef NULL
#define NULL	(0)
#endif

/*
 * define the semaphore data types
 */
#ifndef _MEM_SEMA_T_DEF
#   define _MEM_SEMA_T_DEF
    typedef struct __mem_sema *mem_sema_t;
#endif

#ifndef _CACHE_SEMA_T_DEF
#   define _CACHE_SEMA_T_DEF
    typedef struct __cache_sema *cache_sema_t;
#endif

#ifndef _BARRIER_T_DEF
#   define _BARRIER_T_DEF
    typedef struct __barrier *barrier_t;
    typedef struct __barrier8 *barrier8_t;
#endif

#ifndef _MUTEX_T_DEF
#   define _MUTEX_T_DEF
    typedef struct __mutex *cps_mutex_t;
    typedef struct __mutex8 *cps_mutex8_t;
#endif

/*
 * define the cps data types
 */
#ifndef _STID_T_DEF
#   define _STID_T_DEF
    typedef int stid_t;
#endif

#ifndef _KTID_T_DEF
#   define _KTID_T_DEF
    typedef int ktid_t;
#endif

typedef struct {
	int	node;		/* node to place the allocated threads on */
	int	min;		/* minimum number of threads to allocate */
	int	max;		/* maximum number of threads to allocate */
	int	threadscope;	/* thread scope attributes */
} spawn_sym_t;

typedef struct {
	int	wait_attr;	/* idle wait attribute */
	int	wait_time;	/* wait time in milliseconds */
} cps_wait_attr_t;

#if (defined(__STDC_EXT__) || defined(__CONVEX_PCC)) && !defined(__cplusplus)
typedef struct {
	long long node;		/* node to place the allocated threads on */
	long long min;		/* minimum number of threads to allocate */
	long long max;		/* maximum number of threads to allocate */
	long long threadscope;	/* thread scope attributes */
} spawn_sym8_t;

#ifndef _CPS_BUILD
typedef struct {
	long long	wait_attr;	/* idle wait attribute */
	long long	wait_time;	/* wait time in milliseconds */
} cps_wait_attr8_t;
#endif
#endif

/*
 * node value can be from 0 to N-1, where N is the number of nodes used by
 * the program or one of the following:
 */
#define CPS_SAME_NODE		-1
#define CPS_ANY_NODE		-2
#define CPS_DIFFERENT_NODE	-3
#define CPS_SMART_NODE		-4

/*
 * threadscope is made up of the following bit masks (note that 
 * CPS_THREAD_PARALLEL and CPS_NODE_PARALLEL or mutually exclusive):
 */
#define CPS_THREAD_PARALLEL	1
#define CPS_NODE_PARALLEL	2
#define CPS_OVER_SUBSCRIBE	4
#define CPS_IGNORE_STACKSCOPE	8
#ifdef  _HLO_
#define CPS_HLO                 16
#endif

/*
 * cps wait attribute commands
 */
#define CPS_GETWAIT	0	/* get wait attributes */
#define CPS_SETWAIT	1	/* set wait attributes */
#define CPS_SETWAITI	2	/* set wait attributes immediately */

/*
 * cps wait attribute flags
 */
#define CPS_SUSPEND	1	/* suspend thread in idle loop */
#define CPS_SPINWAIT	2	/* spin thread in idle loop */

/*
 * cps wait maximum wait time in milliseconds.
 */
#define	CPS_WAIT_MAX	700000

/*
 * exported CPS functions
 */
#ifdef __cplusplus
extern "C" {
#endif
#if defined(__stdc__) || defined(__STDC__) || defined(__cplusplus)
    extern stid_t cps_stid(void);
    extern ktid_t cps_ktid(void);
    extern int    cps_nsthreads(void);
    extern int    cps_plevel(void);
    extern int    cps_node_id(void);
    extern int    cps_node_cpus(void);
    extern int    cps_node_nthreads(void);
    extern int    cps_complex_cpus(void);
    extern int    cps_complex_nthreads(void);
    extern int    cps_complex_nodes(void);
    extern int    cps_is_parallel(void);
    extern int	  cps_topology(int *, const int *);

    extern int    cps_ppcall(spawn_sym_t *, void (*)(void *), void *);
    extern int    cps_ppcalln(spawn_sym_t *, void (*)(), const int *, ...);
    extern int    cps_ppcallv(spawn_sym_t *, void (*)(), const int *, va_list );
    extern int    cps_thread_create(const int *, void (*)(void *), void *);
    extern int    cps_thread_createn(const int *, void (*)(), const int *, ... );
    extern int    cps_thread_exit(void);
    extern int    cps_thread_wait(const int *);
    extern int    cps_thread_stack(const int *, const int *);
    extern int	  cps_wait_attr(const int *, cps_wait_attr_t *);
    extern int	  cps_thread_register_lock(mem_sema_t *);
    extern int	  cps_set_threads(const int *);

#   if defined(__STDC_EXT__) && !defined(__cplusplus) && !defined(_CPS_BUILD)
	extern long long cps_stid_8(void);
	extern long long cps_ktid_8(void);
	extern long long cps_nsthreads_8(void);
	extern long long cps_plevel_8(void);
	extern long long cps_node_id_8(void);
	extern long long cps_node_cpus_8(void);
	extern long long cps_node_nthreads_8(void);
	extern long long cps_complex_cpus_8(void);
	extern long long cps_complex_nthreads_8(void);
	extern long long cps_complex_nodes_8(void);
	extern long long cps_is_parallel_8(void);
	extern long long cps_topology_8(long long *, const long long *);

	extern long long cps_ppcall_8(spawn_sym8_t *, void (*)(void *), void *);
	extern long long cps_ppcalln_8(spawn_sym8_t *, void (*)(), const long long *, ...);
	extern long long cps_thread_create_8(const long long *, void (*)(void *), void *);
	extern long long cps_thread_exit_8(void);
	extern long long cps_thread_wait_8(const long long *);
	extern long long cps_thread_stack_8(const long long *,const long long *);
    	extern long long cps_wait_attr_8(const long long *, cps_wait_attr8_t *);
#   endif
#else
    extern stid_t cps_stid();
    extern ktid_t cps_ktid();
    extern int    cps_nsthreads();
    extern int    cps_plevel();
    extern int    cps_node_id();
    extern int    cps_node_cpus();
    extern int    cps_node_nthreads();
    extern int    cps_complex_cpus();
    extern int    cps_complex_nthreads();
    extern int    cps_complex_nodes();
    extern int    cps_is_parallel();
    extern int	  cps_topology();

    extern int    cps_ppcall();
    extern int    cps_ppcalln();
    extern int    cps_ppcallv();
    extern int    cps_thread_create();
    extern int    cps_thread_createn();
    extern int    cps_thread_exit();
    extern int    cps_thread_wait();
    extern int    cps_thread_stack();
    extern int	  cps_wait_attr();
    extern int	  cps_thread_register_lock();
    extern int	  cps_set_threads();

#   ifdef __CONVEX_PCC
	extern long long cps_stid_8();
	extern long long cps_ktid_8();
	extern long long cps_nsthreads_8();
	extern long long cps_plevel_8();
	extern long long cps_node_id_8();
	extern long long cps_node_cpus_8();
	extern long long cps_node_nthreads_8();
	extern long long cps_complex_cpus_8();
	extern long long cps_complex_nthreads_8();
	extern long long cps_complex_nodes_8();
	extern long long cps_is_parallel_8();
	extern long long cps_topology_8();

	extern long long cps_ppcall_8();
	extern long long cps_ppcalln_8();
	extern long long cps_thread_create_8();
	extern long long cps_thread_exit_8();
	extern long long cps_thread_wait_8();
	extern long long cps_thread_stack_8();
    	extern long long cps_wait_attr_8();
#   endif
#endif
#ifdef __cplusplus
}
#endif


/*
 * values for the cps_thread_stack cmd argument:
 */
#define CPS_GET_STACK_SIZE	0
#define CPS_SET_STACK_SIZE	1
#define CPS_GET_STACK_SCOPE	2
#define CPS_SET_STACK_SCOPE	3

/*
 * values for the cps_thread_stack value argument, when setting/getting the
 * stackscope are:
 */
#define CPS_FAR_STACK		0
#define CPS_NEAR_STACK		1
#define CPS_NODE_STACK		2

/*
 * valid values for cps_plevel are
 */
#define CPS_PL_NONE		0	/* no parallel threads */
#define CPS_PL_PARALLEL		1	/* asymmetric threads are active */
#define CPS_PL_NODE		2	/* node way parallel */
#define CPS_PL_NTHREAD		4	/* thread way parallel on a node */
#define CPS_PL_THREAD		8	/* thread way parallel across nodes */
#define CPS_PL_ASYMMETRIC	16	/* asymmretric thread or child of one */

/*
 * defined the low level synchronization routines
 */

#ifdef __cplusplus
extern "C" {
#endif
#if defined(__stdc__) || defined(__STDC__) || defined(__cplusplus)
    extern int	c_fetch32(cache_sema_t *);
    extern int	c_fetch_and_clear32(cache_sema_t *);
    extern int	c_fetch_and_inc32(cache_sema_t *);
    extern int	c_fetch_and_dec32(cache_sema_t *);
    extern int	c_fetch_and_set32(cache_sema_t *, const int *);
    extern int	c_fetch_and_add32(cache_sema_t *, const int *);
    extern int	c_init32(cache_sema_t *, const int *);
    extern int	c_free32(cache_sema_t *);
    extern int 	c_lock(cache_sema_t *);
    extern int	c_unlock(cache_sema_t *);
    extern int	c_cond_lock(cache_sema_t *);
    extern int	m_fetch32(mem_sema_t *);
    extern int	m_fetch_and_clear32(mem_sema_t *);
    extern int	m_fetch_and_inc32(mem_sema_t *);
    extern int	m_fetch_and_dec32(mem_sema_t *);
    extern int	m_init32(mem_sema_t *, const int *);
    extern int	m_free32(mem_sema_t *);
    extern int	m_lock(mem_sema_t *);
    extern int	m_unlock(mem_sema_t *);
    extern int	m_cond_lock(mem_sema_t *);
#   if defined(__STDC_EXT__) && !defined(__cplusplus) && !defined(_CPS_BUILD)
	extern long long c_fetch32_8(cache_sema_t *);
	extern long long c_fetch_and_clear32_8(cache_sema_t *);
	extern long long c_fetch_and_inc32_8(cache_sema_t *);
	extern long long c_fetch_and_dec32_8(cache_sema_t *);
	extern long long c_fetch_and_set32_8(cache_sema_t *, const long long *);
	extern long long c_fetch_and_add32_8(cache_sema_t *, const long long *);
	extern long long c_init32_8(cache_sema_t *, const long long *);
	extern long long c_free32_8(cache_sema_t *);
	extern long long c_lock_8(cache_sema_t *);
	extern long long c_unlock_8(cache_sema_t *);
	extern long long c_cond_lock_8(cache_sema_t *);
	extern long long m_fetch32_8(mem_sema_t *);
	extern long long m_fetch_and_clear32_8(mem_sema_t *);
	extern long long m_fetch_and_inc32_8(mem_sema_t *);
	extern long long m_fetch_and_dec32_8(mem_sema_t *);
	extern long long m_init32_8(mem_sema_t *, const long long *);
	extern long long m_free32_8(mem_sema_t *);
	extern long long m_lock_8(mem_sema_t *);
	extern long long m_unlock_8(mem_sema_t *);
	extern long long m_cond_lock_8(mem_sema_t *);
#   endif
#else
    extern int	c_fetch32();
    extern int	c_fetch_and_clear32();
    extern int	c_fetch_and_inc32();
    extern int	c_fetch_and_dec32();
    extern int	c_fetch_and_set32();
    extern int	c_fetch_and_add32();
    extern int 	c_init32();
    extern int	c_free32();
    extern int	c_lock();
    extern int	c_unlock();
    extern int	c_cond_lock();
    extern int	m_fetch32();
    extern int	m_fetch_and_clear32();
    extern int	m_fetch_and_inc32();
    extern int	m_fetch_and_dec32();
    extern int	m_init32();
    extern int	m_free32();
    extern int	m_lock();
    extern int	m_unlock();
    extern int	m_cond_lock();
#   ifdef __CONVEX_PCC
	extern long long c_fetch32_8();
	extern long long c_fetch_and_clear32_8();
	extern long long c_fetch_and_inc32_8();
	extern long long c_fetch_and_dec32_8();
	extern long long c_fetch_and_set32_8();
	extern long long c_fetch_and_add32_8();
	extern long long c_init32_8();
	extern long long c_free32_8();
	extern long long c_lock_8();
	extern long long c_unlock_8();
	extern long long c_cond_lock_8();
	extern long long m_fetch32_8();
	extern long long m_fetch_and_clear32_8();
	extern long long m_fetch_and_inc32_8();
	extern long long m_fetch_and_dec32_8();
	extern long long m_init32_8();
	extern long long m_free32_8();
	extern long long m_lock_8();
	extern long long m_unlock_8();
	extern long long m_cond_lock_8();
#   endif
#endif

/*
 * high level synchronization functions
 */
#if defined(__stdc__) || defined(__STDC__) || defined(__cplusplus)
    extern int	cps_mutex_alloc(cps_mutex_t *);
    extern int	cps_limited_spin_mutex_alloc(cps_mutex_t *, const int *);
    extern int	cps_mutex_free(cps_mutex_t *);
    extern int	cps_mutex_lock(cps_mutex_t *);
    extern int	cps_mutex_trylock(cps_mutex_t *);
    extern int	cps_mutex_unlock(cps_mutex_t *);
    extern int	cps_barrier_alloc(barrier_t *);
    extern int	cps_limited_spin_barrier_alloc(barrier_t *, const int *);
    extern int	cps_barrier_free(barrier_t *);
    extern int	cps_barrier(barrier_t *, const int *);
#   if defined(__STDC_EXT__) && !defined(__cplusplus) && !defined(_CPS_BUILD)
	extern long long cps_mutex_alloc_8(cps_mutex8_t *);
	extern long long cps_limited_spin_mutex_alloc_8(cps_mutex8_t *, const long long *);
	extern long long cps_mutex_free_8(cps_mutex8_t *);
	extern long long cps_mutex_lock_8(cps_mutex8_t *);
	extern long long cps_mutex_trylock_8(cps_mutex8_t *);
	extern long long cps_mutex_unlock_8(cps_mutex8_t *);
	extern long long cps_barrier_alloc_8(barrier8_t *);
	extern long long cps_barrier_free_8(barrier8_t *);
	extern long long cps_barrier_8(barrier8_t *, const long long *);
#   endif
#else
    extern int	cps_mutex_alloc();
    extern int	cps_limited_spin_mutex_alloc();
    extern int	cps_mutex_free();
    extern int	cps_mutex_lock();
    extern int	cps_mutex_trylock();
    extern int	cps_mutex_unlock();
    extern int	cps_barrier_alloc();
    extern int	cps_limited_spin_barrier_alloc();
    extern int	cps_barrier_free();
    extern int	cps_barrier();
#   ifdef __CONVEX_PCC
	extern long long cps_mutex_alloc_8();
	extern long long cps_limited_spin_mutex_alloc_8();
	extern long long cps_mutex_free_8();
	extern long long cps_mutex_lock_8();
	extern long long cps_mutex_trylock_8();
	extern long long cps_mutex_unlock_8();
	extern long long cps_barrier_alloc_8();
	extern long long cps_limited_spin_barrier_alloc_8();
	extern long long cps_barrier_free_8();
	extern long long cps_barrier_8();
#   endif
#endif
#ifdef __cplusplus
}
#endif

#endif
