#ifndef _MATH_INCLUDED
#define _MATH_INCLUDED

#ifndef _SYS_STDSYMS_INCLUDED
#  include <sys/stdsyms.h>
#endif /* _SYS_STDSYMS_INCLUDED */

#if !defined(__IA64__)
#  define _PA_RISC
#endif

#ifdef __cplusplus
extern "C" {
#endif


/********* STANDARD C INTERFACE ********************************/

#ifndef HUGE_VAL
#define HUGE_VAL   _DINFINITY
#endif

#if defined(_PA_RISC) && !defined(__cplusplus)
#  pragma NO_SIDE_EFFECTS fabs
#  ifdef __BUILTIN_MATH
#    pragma NO_SIDE_EFFECTS acos, asin, atan, atan2, cos, sin, tan
#    pragma NO_SIDE_EFFECTS cosh, sinh, tanh
#    pragma NO_SIDE_EFFECTS exp, ldexp, log, log10, pow
#    pragma NO_SIDE_EFFECTS sqrt, ceil, floor, fmod
#  endif
#endif /* #if for pragmas */

#ifdef _PROTOTYPES 
   extern double acos(double);
   extern double asin(double);
   extern double atan(double);
   extern double atan2(double, double);
   extern double cos(double);
   extern double sin(double);
   extern double tan(double);
   extern double cosh(double);
   extern double sinh(double);
   extern double tanh(double);
   extern double exp(double);
   extern double frexp(double, int *);
   extern double ldexp(double, int);
   extern double log(double);
   extern double log10(double);
   extern double modf(double, double *);
   extern double pow(double, double);
   extern double sqrt(double);
   extern double ceil(double);
   extern double fabs(double);
   extern double floor(double);
   extern double fmod(double, double);
#else /* not _PROTOTYPES */
   extern double acos();
   extern double asin();
   extern double atan();
   extern double atan2();
   extern double cos();
   extern double sin();
   extern double tan();
   extern double cosh();
   extern double sinh();
   extern double tanh();
   extern double exp();
   extern double frexp();
   extern double ldexp();
   extern double log();
   extern double log10();
   extern double modf();
   extern double pow();
   extern double sqrt();
   extern double ceil();
   extern double fabs();
   extern double floor();
   extern double fmod();
#endif /* _PROTOTYPES */


/********* XOPEN EXTENSIONS ***********************************/ 

#ifdef _INCLUDE_XOPEN_SOURCE

#  define M_E           2.7182818284590452354
#  define M_LOG2E       1.4426950408889634074
#  define M_LOG10E      0.43429448190325182765
#  define M_LN2         0.69314718055994530942
#  define M_LN10        2.30258509299404568402
#  define M_PI          3.14159265358979323846
#  define M_PI_2        1.57079632679489661923
#  define M_PI_4        0.78539816339744830962
#  define M_1_PI        0.31830988618379067154
#  define M_2_PI        0.63661977236758134308
#  define M_2_SQRTPI    1.12837916709551257390
#  define M_SQRT2       1.41421356237309504880
#  define M_SQRT1_2     0.70710678118654752440
#  define MAXFLOAT      ((float)3.40282346638528860e+38)

   extern int signgam;

#  if defined(_PA_RISC) && !defined(__cplusplus)
#    pragma NO_SIDE_EFFECTS isnan
#    ifdef __BUILTIN_MATH
#      pragma NO_SIDE_EFFECTS acosh, asinh, atanh, rint, scalb, logb, cbrt
#      pragma NO_SIDE_EFFECTS nextafter, remainder, expm1, ilogb, log1p
#    endif
#  endif /* #if for pragma */

#  ifdef _PROTOTYPES
     extern double erf(double);
     extern double erfc(double);
     extern double gamma(double);                             /* obsolescent */
     extern double hypot(double, double);
     extern int    isnan(double);
     extern double j0(double);
     extern double j1(double);
     extern double jn(int, double);
     extern double lgamma(double);
     extern double y0(double);
     extern double y1(double);
     extern double yn(int, double);
#  else /* not _PROTOTYPES */
     extern double erf();
     extern double erfc();
     extern double gamma();                                   /* obsolescent */
     extern double hypot();
     extern int    isnan();
     extern double j0();
     extern double j1();
     extern double jn();
     extern double lgamma();
     extern double y0();
     extern double y1();
     extern double yn();
#  endif /* _PROTOTYPES */

#endif /* _INCLUDE_XOPEN_SOURCE */

#ifdef _INCLUDE_XOPEN_SOURCE_EXTENDED

#  ifdef _PROTOTYPES
     extern double acosh(double);
     extern double asinh(double);
     extern double atanh(double);
     extern double cbrt(double);
     extern double expm1(double);
     extern int    ilogb(double);
     extern double log1p(double);
     extern double logb(double);
     extern double nextafter(double, double);
     extern double remainder(double, double);
     extern double rint(double);
     extern double scalb(double,double);
#  else /* not _PROTOTYPES */
     extern double acosh();
     extern double asinh();
     extern double atanh();
     extern double cbrt();
     extern double expm1();
     extern int    ilogb();
     extern double log1p();
     extern double logb();
     extern double nextafter();
     extern double remainder();
     extern double rint();
     extern double scalb();
#  endif /* _PROTOTYPES */

#endif /* _INCLUDE_XOPEN_SOURCE_EXTENDED */


/******** HPUX EXTENSIONS *************************************/ 

#ifdef _INCLUDE_HPUX_SOURCE

#  ifdef __STDC__
     typedef float float_t;
     typedef double double_t;
#  endif

#  define HUGE_VALF     _SINFINITY
#  define INFINITY      _SINFINITY
#  define NAN           _SQNAN

#  define isnan(x)		_ISNAN(x)
#  define isinf(x)		_ISINF(x)
#  define signbit(x)		_SIGNBIT(x)
#  define isfinite(x)		_ISFINITE(x)
#  define isnormal(x)		_ISNORMAL(x)
#  define fpclassify(x)		_FPCLASSIFY(x)
#  define isunordered(x,y)	_ISUNORDERED(x,y)
#  define isgreater(x,y)	_ISGREATER(x,y)
#  define isgreaterequal(x,y)	_ISGREATEREQUAL(x,y)
#  define isless(x,y)		_ISLESS(x,y)
#  define islessequal(x,y)	_ISLESSEQUAL(x,y)
#  define islessgreater(x,y)	_ISLESSGREATER(x,y)

#  define FP_NORMAL     0
#  define FP_ZERO       1
#  define FP_INFINITE   2
#  define FP_SUBNORMAL  3
#  define FP_NAN        4

#  define DECIMAL_DIG   17

#  if defined(_PA_RISC) && !defined(__cplusplus)
#    pragma NO_SIDE_EFFECTS copysign, copysignf, fabsf
#    pragma NO_SIDE_EFFECTS _Fpclassify, _Fpclassifyf, _Fpclassifyfd
#    pragma NO_SIDE_EFFECTS _Isunordered, _Isunorderedf
#    pragma NO_SIDE_EFFECTS _Signbit, _Signbitf, _Isnanf, _Isinf, _Isinff
#    pragma NO_SIDE_EFFECTS _Isfinite, _Isfinitef, _Isnormal, _Isnormalf, _Isnormalfd
#    ifdef __BUILTIN_MATH
#      pragma NO_SIDE_EFFECTS acosd, asind, atand, atan2d, cosd, sind, tand, log2
#      pragma NO_SIDE_EFFECTS acosf, asinf, atanf, atan2f, cosf, sinf, tanf
#      pragma NO_SIDE_EFFECTS sinhf, coshf, tanhf, acosdf, asindf, atandf, atan2df
#      pragma NO_SIDE_EFFECTS cosdf, sindf, tandf, expf, logf, log10f, log2f
#      pragma NO_SIDE_EFFECTS powf, sqrtf, fmodf, cbrtf
#    endif
#  endif /* #if for pragmas */

#  ifdef _PROTOTYPES
     extern double copysign(double, double);
     extern double log2(double);
     extern double lgamma_r(double, int *);
     extern double exp2(double);
     extern double fdim(double, double);
     extern double fmax(double, double);
     extern double fmin(double, double);
     extern double nan(const char *);
     extern double scalbn(double, int);
     extern double nearbyint(double);
     extern double round(double);
     extern double trunc(double);
     extern double remquo(double, double, int *);
     extern long   lrint(double);
     extern long   lround(double);
#    if defined(__STDC_EXT__) || defined(_INCLUDE_LONGLONG)
       extern long long llrint(double);
       extern long long llround(double);
#    endif
     extern double cosd(double);
     extern double sind(double);
     extern double tand(double);
#    ifndef _PA_RISC
       extern double    exp10(double);
#    endif
     extern double acosd(double);
     extern double asind(double);
     extern double atand(double);
     extern double atan2d(double, double);
     extern float acosf(float);
     extern float asinf(float);
     extern float atanf(float);
     extern float atan2f(float, float);
     extern float cosf(float);
     extern float sinf(float);
     extern float tanf(float);
     extern float sinhf(float);     
     extern float coshf(float);
     extern float tanhf(float);
     extern float expf(float);
     extern float logf(float);
     extern float log10f(float);
     extern float powf(float, float);
     extern float sqrtf(float);
     extern float fabsf(float);
     extern float fmodf(float, float);
     extern float cbrtf(float);
     extern float copysignf(float, float);
     extern float nextafterf(float, float);
     extern float log2f(float);
     extern float cosdf(float);
     extern float sindf(float);
     extern float tandf(float);
#    ifndef _PA_RISC
       extern float exp2f(float);
       extern float exp10f(float);
       extern float expm1f(float);
       extern float log1pf(float);
#    endif
     extern float acosdf(float);
     extern float asindf(float);
     extern float atandf(float);
     extern float atan2df(float, float);
#  else /* not _PROTOTYPES */
     extern double copysign();
     extern double log2();
     extern double lgamma_r();
     extern double exp2();
     extern double fdim();
     extern double fmax();
     extern double fmin();
     extern double nan();
     extern double scalbn();
     extern double nearbyint();
     extern double round();
     extern double trunc();
     extern double remquo();
     extern long   lrint();
     extern long   lround();
     extern double cosd();
     extern double sind();
     extern double tand();
#    ifndef _PA_RISC
       extern double    exp10();
#    endif
     extern double acosd();
     extern double asind();
     extern double atand();
     extern double atan2d();
#  endif /* _PROTOTYPES */

#endif /* _INCLUDE_HPUX_SOURCE */


/******** INTERNAL INTERFACE used in macro definitions ********/

#ifdef _PROTOTYPES 
   extern const double _DINFINITY;
#else
   extern double _DINFINITY;
#endif

#ifdef _PA_RISC
#  define fabsf(x) ((float)fabs((double)(float)(x)))
#endif

#ifdef _INCLUDE_HPUX_SOURCE

#  define _IS64(x) (sizeof(x)==sizeof(double))
#  define _IS32(x) (sizeof(x)==sizeof(float))
#  ifdef _PROTOTYPES
     extern const float _SINFINITY;
     extern const float _SQNAN;
#    ifdef _PA_RISC
#      define _ISNAN(x)          (_IS32(x)?_Isnanf(x):(isnan)(x))
#      define _ISINF(x)          (_IS32(x)?_Isinff(x):_Isinf(x))
#      define _SIGNBIT(x)        (_IS32(x)?_Signbitf(x):_Signbit(x))
#      define _ISFINITE(x)	 (_IS32(x)?_Isfinitef(x):_Isfinite(x))
#      define _ISNORMAL(x)       (_IS32(x)?_Isnormalf(x):_Isnormal(x))
#      define _FPCLASSIFY(x)     (_IS32(x)?_Fpclassifyf(x)>>1:_Fpclassify(x)>>1)
#      define _ISUNORDERED(x,y)  (_IS32(x)&&_IS32(y)?_Isunorderedf(x,y):_Isunordered(x,y))
       extern int _Signbit(double);
       extern int _Signbitf(float);
       extern int _Isnanf(float);
       extern int _Isfinite(double);
       extern int _Isfinitef(float);
       extern int _Isinf(double);
       extern int _Isinff(float);
       extern int _Isnormal(double);
       extern int _Isnormalf(float);
       extern int _Isunordered(double, double);
       extern int _Isunorderedf(float, float);
       extern int _Fpclassify(double);
       extern int _Fpclassifyf(float);
#    else 
#      include "math_ia64_internal.h"
#      define _FPCLASSIFY(x)     (_IS32(x)?_Fpclassf(x):_Fpclass(x))
       extern int _Fpclass(double);
       extern int _Fpclassf(float);
#    endif
#  else /* NO PROTOTYPES */
     extern float _SINFINITY;
     extern float _SQNAN;
#    define _ISNAN(x)		((isnan)((double)(x)))
#    define _ISINF(x)		(_Isinf((double)(x)))
#    define _SIGNBIT(x)		(_Signbit((double)(x)))
#    define _ISFINITE(x)	(_Isfinite((double)(x)))
#    define _ISNORMAL(x)	(_IS32(x)?_Isnormalfd((double)(x)):_Isnormal((double)(x)))
#    define _ISUNORDERED(x,y) 	(_Isunordered((double)(x),(double)(y)))
#    ifdef _PA_RISC
#      define _FPCLASSIFY(x) (_IS32(x)?_Fpclassifyfd((double)(x))>>1:_Fpclassify((double)(x))>>1)
       extern int _Fpclassify();
       extern int _Fpclassifyfd();
#    else
#      define _FPCLASSIFY(x) (_IS32(x)?_Fpclassfd((double)(x)):_Fpclass((double)(x)))
       extern int _Fpclass();
       extern int _Fpclassfd();
#    endif
     extern int _Signbit();
     extern int _Isfinite();
     extern int _Isinf();
     extern int _Isnormal();
     extern int _Isnormalfd();
     extern int _Isunordered();
#  endif

#  define _ISGREATER(x,y)      (!(_ISUNORDERED(x,y)||(x)<=(y)))
#  define _ISGREATEREQUAL(x,y) (!(_ISUNORDERED(x,y)||(x)<(y)))
#  define _ISLESS(x,y)         (!(_ISUNORDERED(x,y)||(x)>=(y)))
#  define _ISLESSEQUAL(x,y)    (!(_ISUNORDERED(x,y)||(x)>(y)))
#  define _ISLESSGREATER(x,y)  (!(_ISUNORDERED(x,y)||(x)==(y)))

#endif /* _INCLUDE_HPUX_SOURCE */

#ifdef __cplusplus
}
#endif


/******** PRE STANDARD C++ ****************************/

#if defined(__cplusplus) && (__cplusplus < 199707)
   
   
#  ifndef _STDLIB_INCLUDED
     extern "C" int abs(int);
#  endif /* _STDLIB_INCLUDED */
   
#endif


#endif /* _MATH_INCLUDED */
