/* $Source: /usr/local/kcs/sys.ROSE_800/sio/RCS/dma_A.h,v $
 * $Revision: 1.12.106.1 $     $Author: root $
 * $State: Exp $      $Locker: CRT $
 * $Date: 97/07/07 18:20:21 $
 */

#ifndef _DMA_A_INCLUDED /* allows multiple inclusion */
#define _DMA_A_INCLUDED

#ifdef _KERNEL_BUILD
#include "../h/spinlock.h"
#else  /* ! _KERNEL_BUILD */
#include <sys/spinlock.h>
#endif /* _KERNEL_BUILD */

#ifndef NULL
		#undef NULL
#define NULL 0L
#endif

/* dma_A_SRS_status return values */
#define DMA_A_OK		0
#define DMA_A_FE_MODE		1
#define DMA_A_POWER_FAIL	2

/* hw_class types */
#define DMA_A_WORD_HW           0
#define DMA_A_BYTE_HW           1
#define DMA_A_SHZM_HW           2
#define DMA_A_LOQUIX_HW         3 /*
				   * Unless otherwise noted all "SHZM"
				   * constants and macros below are valid
				   * for the Lo-quix DMA chip as well.
				   */

/* Typedef's for Type-A DMA */

/* Supervisor Register Set (SRS) */
typedef volatile struct {
    uint32_t type_I_0;     	/* I  I   implementation dependent   */
    uint32_t io_eim;       	/* W  OI  external interrupt message */
    union {
        uint32_t data;     	/* R  A   io_dc_data	*/
        uint32_t addr;     	/* W  A   io_dc_addr 	*/
    } io_dc;
    union {
        uint32_t data;		/* R  OI  io_ii_data    */
        uint32_t clear;		/* W  OI  io_ii_clear   */
    } io_ii;
    uint32_t type_U_0[4];       /* U  U   undefined 	*/
    uint32_t type_R_0;  	/* RW R   reserved  	*/
    uint32_t io_spa;
    uint32_t type_R_1[2];     	/* RW R   reserved  	*/
    uint32_t io_command;	/* W  A   command   	*/
    uint32_t io_status;		/* R  A   status    	*/
    uint32_t io_control;	/* RW OI  control   	*/
    union {
        uint32_t in;		/* R  OI  io_data_in	*/
        uint32_t out;		/* W  OI  io_data_out	*/
    } io_data;
} dma_A_srs_type;


/* Auxiliary Register Set (ARS) */
typedef volatile struct {
    uint32_t io_err_sadd;     	/* R  OI  log slave address	*/
    uint32_t io_err_info;       /* R  OI  log detailed info	*/
    uint32_t io_err_madd;       /* R  OI  log master address	*/
    uint32_t type_U_0;  	/* U  U   undefined 	*/
    uint32_t io_dma_link;  	/* RW OI  chain link 	*/
    uint32_t io_dma_command;  	/* R  OI  link command 	*/
    uint32_t io_dma_address;  	/* R  OI  link address 	*/
    uint32_t io_dma_count;  	/* R  OI  link count	*/
    uint32_t type_U_1[8];  	/* U  U   undefined 	*/
} dma_A_ars_type;


/* Undefined Register Set (URS) */
typedef volatile struct {
    uint32_t reg[16];
} dma_A_urs_type;


/* Implementation dependent Register Set (IRS) */
typedef volatile struct {
    uint32_t reg[16];
} dma_A_irs_type;


/* A DMA Register Set (ADMARS) */
typedef volatile struct {
    uint32_t type_I_0;    	/* I  I   implementation dependent   */
    uint32_t io_eim;      	/* W  OI  external interrupt message */
    uint32_t type_I_1;    	/* I  I   implementation dependent   */
    union {
	uint32_t data;	    	/* R  OI  io_ii_data		     */
	uint32_t clear;	    	/* W  OI  io_ii_clear	             */
    } io_ii;
    uint32_t io_dma_link;      	/* RW OI  chain link 		     */
    uint32_t io_dma_command;   	/* R  OI  link command 		     */
    uint32_t io_dma_address;   	/* R  OI  link address 		     */
    uint32_t io_dma_count;     	/* R  OI  link count		     */
    uint32_t type_I_2[4];    	/* I  I   implementation dependent   */
    uint32_t io_command;	/* W  A   command   		     */
    uint32_t io_status;	    	/* R  A   status    		     */
    uint32_t type_I_3;    	/* I  I   implementation dependent   */
    union {
	uint32_t in;	    	/* R  OI  io_data_in		     */
	uint32_t out;	    	/* W  OI  io_data_out		     */
    } io_data;
} dma_A_rs_type;


/* Type dependent Register Set (TRS) */
typedef struct {
    union {                     /* Either IRS or ADMARS */
        dma_A_irs_type irs;
        dma_A_rs_type  admars;
    } u;
} dma_A_trs_type;


typedef struct {
    dma_A_srs_type  srs;
    dma_A_ars_type  ars;
    dma_A_trs_type  trs[10];
    dma_A_urs_type  urs[20];
} dma_A_lower_page_type;


typedef struct {
    dma_A_trs_type  trs[32];
} dma_A_upper_page_type;


typedef struct {
    dma_A_lower_page_type	lower_page;
    dma_A_upper_page_type	upper_page;
} dma_A_module_type;


/* WARNING:  IN dma_A_create_chain(), the algorithm for making       */
/* chains expects that the first word of a pquad is the io_dma_link. */
/* If this ever changes, then the code/algorithm must be changed.    */
/* There is a warning in the code too, as an added caution.	     */
/* The first 16 bytes of the pquad are used by the H/W, the next 16  */
/* bytes are used by the S/W for accounting purposes		     */

/* Physical Quad */
typedef	struct dma_A_pquad {
    volatile uint32_t		io_dma_link;
    volatile uint32_t		io_dma_command;
    volatile uint32_t		io_dma_address;
    volatile uint32_t 		io_dma_count;
    struct dma_A_pquad *host_dma_link;		/* host link address	*/
    struct dma_A_pquad *bq_link;		/* buflet quad link  	*/
    space_t		data_sid;		/* data buffer sid 	*/
    anyptr		data_vba;		/* data buffer vaddr	*/
} dma_A_pquad_type;


/* Chain Header */
typedef struct dma_A_chain {
    union {
	struct dma_A_chain *link;
        struct {
            caddr_t	io_chain_head;	/* IOVA of first pquad on chain */
            dma_A_pquad_type *chain_tail;
            dma_A_pquad_type *bq_head;
            dma_A_pquad_type *bq_tail;
            dma_A_pquad_type *host_chain_head;
            space_t	data_sid;
            anyptr	data_vba;
            uint32_t	count;
        } hdr;
    } u;
    uint32_t	hw_class;  /* Added to handle exception cases */
    char	spu;	   /* Spu from which this header was allocated */
} dma_A_chain_type;

/* Buflet Type */
typedef struct dma_A_buflet {
    union {
	struct dma_A_buflet *link;
	u_char mem[MAX_CACHE_LINE];
    } u;
} dma_A_buflet_type;


/* Miscellaneous defines */

#define BIT_MASK(bit_num)    (1L << (31-bit_num))

#define GET_SRS_IO_STATUS(hpa) \
    (((dma_A_module_type *) (hpa))->lower_page.srs.io_status)

#define MIN_IO_LINE		16

/* MAX_CACHE_LINE is defined in 'sio/llio.h' to 64 bytes */

#define CHAIN_SHIFT_ALIGN        2   /* Align on 4 byte boundary */
#define QUAD_SHIFT_ALIGN         4   /* Align on 16 byte boundary */
#define BUFLET_SHIFT_ALIGN       6   /* Align on 64 byte boundary */

/* Type-A DMA io_commands */

#define CMD_STOP         	0
#define CMD_FLUSH        	1
#define CMD_CHAIN        	2
#define CMD_CLEAR        	3
#define CMD_RESET        	5
#define CMD_INPUT		8
#define CMD_OUTPUT		9

#define COMMAND_IE              BIT_MASK(24)

#define CMD_STOP_I         	(CMD_STOP  | COMMAND_IE)
#define CMD_FLUSH_I        	(CMD_FLUSH | COMMAND_IE)
#define CMD_CHAIN_I        	(CMD_CHAIN | COMMAND_IE)
#define CMD_CLEAR_I        	(CMD_CLEAR | COMMAND_IE)
#define CMD_RESET_I        	(CMD_RESET | COMMAND_IE)

/* Type-A DMA implementation dependent io_commands */
#define SHZM_CMD_FIFO_IN	0x78
#define SHZM_CMD_FIFO_OUT	0x79
#define SHZM_CMD_XFR3		0x7c
#define SHZM_CMD_SCLEAR		0x7d
#define SHZM_CMD_SCLEAR_I       (SHZM_CMD_SCLEAR | COMMAND_IE)


/* Type-A DMA io_ii data/clear */
#define II_BIT      		BIT_MASK(26)
#define IIC_BIT      		BIT_MASK(26)


/* Type-A DMA io_status */

#define SECONDARY_LOSS  	BIT_MASK(15)
#define ESTAT          	       (BIT_MASK(16) | \
			        BIT_MASK(17) | \
				BIT_MASK(18) | \
				BIT_MASK(19) | \
				BIT_MASK(20) | \
				BIT_MASK(21))
#define SOFT_ERROR      	BIT_MASK(22)
#define HARD_ERROR      	BIT_MASK(23)
#define FATAL_ERROR     	BIT_MASK(24)
#define READY          		BIT_MASK(25)
#define POWER_WARNING   	BIT_MASK(29)
#define POWER_LOSS      	BIT_MASK(30)
#define END_OF_CHAIN        	BIT_MASK(31)

/* Type-A DMA implementation dependent io_status */
#define SHZM_PBUS_INT           BIT_MASK(2)
#define SHZM_XFR_RDY		BIT_MASK(3)
#define SHZM_LENGTH_CONFLICT 	SOFT_ERROR


/* Type-A DMA io_dma_command */

#define CCMD_OUT		4   /* DMA output (memory to pbus) */
#define CCMD_IN			0   /* DMA input  (pbus to memory) */


/* Shazam implementation features */

#define SHZM_DMA_CONTROL_REG        lower_page.trs[7].u.irs.reg[12]
#define SHZM_DMA_PAUSE              1
#define SHZM_DMA_NORMAL             0

#define SHZM_SET_IIENB_REG          lower_page.trs[7].u.irs.reg[0]
#define SHZM_CLEAR_IIENB_REG        lower_page.trs[7].u.irs.reg[1]

#define SHZM_TEST_REG               lower_page.trs[7].u.irs.reg[14]
#define SHZM_TEST_DMA_PAUSE	    BIT_MASK(25)

#define SHZM_DIAG_REG               lower_page.trs[7].u.irs.reg[15]
#define SHZM_DIAG_NINTIN   	    BIT_MASK(0)
#define SHZM_DIAG_DMA_ENB   	    BIT_MASK(4)
#define SHZM_DIAG_CHAIN_CMD   	    BIT_MASK(10)
#define SHZM_DIAG_RESET_CMD   	    BIT_MASK(11)
#define SHZM_DIAG_FLUSH_CMD   	    BIT_MASK(12)
#define SHZM_DIAG_IE                BIT_MASK(13)
#define SHZM_DIAG_STOP_CMD   	    BIT_MASK(14)
#define SHZM_DIAG_IIENB             BIT_MASK(19)

/* Lo-quix implementation features */

#define LOQUIX_DMA_STOP		    BIT_MASK(5)		/* use instead of SHZM_TEST_DMA_PAUSE */
#define LOQUIX_DIAG_PEND_EIM	    BIT_MASK(3)		/* use instead of bit 7 of SHZM_TEST_REG */

#define LOQUIX_CONF_CONTROL_REG     lower_page.trs[7].u.irs.reg[13]
#define LOQUIX_DMAENB16		    BIT_MASK(30)
#define LOQUIX_DIRENB16		    BIT_MASK(31)

#define LOQUIX_INT_CONTROL_REG     lower_page.trs[7].u.irs.reg[14]	/* replaces SHZM_TEST_REG */
#define LOQUIX_DIE		    BIT_MASK(30)
#define LOQUIX_EIE		    BIT_MASK(31)


/* The busywait is for a SHAZAM/BC bug */
#define dma_A_SRS_reset(hpa) \
    ((dma_A_module_type *) (hpa))->lower_page.srs.io_command = CMD_RESET; \
    busywait(100L); 							  \
    dma_A_wait_for_fe_or_rdy(hpa)

#define dma_A_SRS_status(hpa) \
    ((GET_SRS_IO_STATUS(hpa) & FATAL_ERROR) ? \
     ((io_em_fe(hpa)) ? DMA_A_POWER_FAIL : DMA_A_FE_MODE) : DMA_A_OK)

#define dma_A_SRS_set_eim(hpa, eim) \
    ((dma_A_module_type *) (hpa))->lower_page.srs.io_eim = eim

#define dma_A_SRS_set_spa(hpa, spa) \
    ((dma_A_module_type *) (hpa))->lower_page.srs.io_spa = spa

#define dma_A_SRS_clear_ii(hpa) \
    ((dma_A_module_type *) (hpa))->lower_page.srs.io_ii.clear = 0

#define dma_A_SHZM_pause(hpa) \
    ((dma_A_module_type *) (hpa))->SHZM_DMA_CONTROL_REG = SHZM_DMA_PAUSE

#define dma_A_SHZM_unpause(hpa) \
    ((dma_A_module_type *) (hpa))->SHZM_DMA_CONTROL_REG = SHZM_DMA_NORMAL

#define dma_A_SHZM_chain_int(hpa) \
    ( (((GET_SRS_IO_STATUS(hpa) & (READY | FATAL_ERROR)) == READY) &&         \
       ( ((dma_A_module_type *) (hpa))->SHZM_DIAG_REG & SHZM_DIAG_CHAIN_CMD)) \
       ? TRUE : FALSE)

#define dma_A_SHZM_pbus_int(hpa) \
    (((GET_SRS_IO_STATUS(hpa) & (SHZM_PBUS_INT | FATAL_ERROR))==SHZM_PBUS_INT) \
      ? TRUE : FALSE)

/* Presumes the chip (Shazam or Lo-quix) is in the ready state (ry==1) */
#define dma_A_SHZM_set_ie(hpa) \
    ((dma_A_module_type *) (hpa))->lower_page.srs.io_command = CMD_CLEAR_I

#define dma_A_SHZM_set_iienb(hpa) \
    ((dma_A_module_type *) (hpa))->SHZM_SET_IIENB_REG = 1

#define dma_A_SHZM_clr_iienb(hpa) \
    ((dma_A_module_type *) (hpa))->SHZM_CLEAR_IIENB_REG = 1

/* Following two macros presume other bits of the conf_control reg are not used */
#define dma_A_LOQUIX_set_width_8(hpa) \
    ((dma_A_module_type *) (hpa))->LOQUIX_CONF_CONTROL_REG = 0

#define dma_A_LOQUIX_set_width_16(hpa) \
    ((dma_A_module_type *) (hpa))->LOQUIX_CONF_CONTROL_REG = (LOQUIX_DMAENB16 | LOQUIX_DIRENB16)

/* macro  void               dma_A_SRS_reset();    		*/
/* macro  uint32_t           dma_A_SRS_status();           	*/
/* macro  void               dma_A_SRS_set_eim();	  	*/
/* macro  void               dma_A_SRS_set_spa();		*/
/* macro  void  	     dma_A_SHZM_pause();           	*/
/* macro  void  	     dma_A_SHZM_unpause();         	*/
/* macro  uint32_t 	     dma_A_SHZM_chain_int();       	*/
/* macro  uint32_t 	     dma_A_SHZM_pbus_int();        	*/
/* macro  void		     dma_A_SHZM_set_ie(hpa);		*/
/* macro  void  	     dma_A_SHZM_set_iienb();       	*/
/* macro  void  	     dma_A_SHZM_clr_iienb();		*/

/* 
 * There will be a maximum of 2 buflets used for any DMA.  One to handle
 * requests which do not start on a cache-line boundry and one for requests
 * which do not end on a cache-line boundry. 
 */

/* 
 * This is the main structure for the DMA services generic routines.  The
 * routines will do alignment checking and buflet management, but leave the
 * handling of operations specific to a particular model to a function 
 * specified in (*dma_func)().  If (*dma_func)() needs additional information
 * local_parms can be used to point to a structure containing this information.
 * (*dma_func)() must take three parameters:
 *
 * dma     - a pointer to the structure below
 * address - a REAL memory address of this element of the DMA chain
 * count   - the number of bytes to be transferred at this address
 *
 * The function must return an integer which will be returned to the 
 * caller of io_dma_setup().  A non-zero value will terminate processing
 * of the DMA chain.
 *
 * Synopsis:
 *
 * int
 * my_dma_func(struct dma_serv *dma, caddr_t address, uint32_t count)
 * {
 *     int          return_value;
 *     my_param_t  *my_params = (my_param_t *)dma->local_parms;
 *
 *     /* routine to handle DMA structures for this DMA model 
 *     return(return_value);
 * }
 */

/* Buflet book-keeping structure */
typedef struct dma_buflet {
    u_char             mem[MAX_CACHE_LINE];
    struct dma_buflet *next;
    space_t            b_sid;
    anyptr             b_addr;   /* user's address for the data */
    uint32_t           b_count;  /* bytes of data in buflet     */
} dma_buflet_t;

typedef struct dma_serv {
    data_ptr_type       b_ptr;     /* original data buffer                  */
    addr_class_type     b_class;   /* data class: VIRTUAL_BUFFER etc.       */
    uint32_t            b_count;   /* bytes to transfer                     */
    uint32_t            b_dir;     /* direction of transfer: READ_DATA etc. */
    uint32_t            segment_size; /* size of DMA segment                */
    uint32_t            seg_residue;  /* data residue in segment (VBLOCKS)  */
    dma_buflet_t       *buflet_list;  /* buflet information                 */
    int               (*dma_func) __((struct dma_serv *dma,
					caddr_t addr, uint32_t count));
    void               *local_parms;  /* specific DMA model parameters      */
} dma_serv_t;

/*
** Allocate 16 byte aligned pquads
** Replacment for get_in_pquad and get_out_pquad.
*/
#define SIO_DMA_ALLOC_PQUAD( pquad_p, port_num, iova, dma_cmd)           \
    pquad_p = (dma_A_pquad_type *) sio_dma_alloc( (port_num), & (iova)); \
    pquad_p->io_dma_command = (dma_cmd)

/* dma A header free list structure */
typedef struct {
      lock_t           *lock;
      dma_A_chain_type *chain_list;
#ifdef __LP64__
      uint32_t          pad[12];   /* align it to 64 bytes */
#else  /* !__LP64__ */
      uint32_t          pad[14];   /* align it to 64 bytes */
#endif /* !__LP64__ */
} dma_A_pool_t;

/* Type-A DMA routine declarations */

#ifdef _KERNEL
extern dma_A_chain_type *dma_A_create_chain __((
	port_num_type port_num, uint32_t hw_class, data_ptr_type *b_ptr,
	addr_class_type b_class, uint32_t b_dir, uint32_t b_count));
extern void dma_A_free_chain __((
	port_num_type port_num, dma_A_chain_type *chain_p));
extern uint32_t dma_A_SRS_end_chain __((
	port_num_type port_num, dma_A_module_type *hpa,
	dma_A_chain_type *chain_p));
extern void dma_A_SRS_start_chain __((
	dma_A_module_type *hpa, dma_A_chain_type *chain_p));
extern void dma_A_SRS_start_chain_nint __((
	dma_A_module_type *hpa, dma_A_chain_type *chain_p));
extern void dma_A_wait_for_fe_or_rdy __((
	dma_A_module_type *hpa));
extern int io_dma_setup __((
	port_num_type port_num, dma_serv_t *dma));
extern void io_dma_cleanup __((
	port_num_type port_num, dma_serv_t *dma));
#endif /* _KERNEL */

#endif /* not _DMA_A_INCLUDED */
