/*
 * @(#) hpib1.h: $Revision: 1.16.106.1 $ $Date: 97/07/07 18:22:24 $
 */

/************************************************************************/
/*                                                                      */
/*	          This is the header file for hpib1.c 			*/
/*                                                                      */
/*   		(c) Copyright 1988, Hewlett-Packard Company             */
/*                                                                      */
/************************************************************************/

/* When DEBUG is defined, MUCHO error checking is enabled.  This allows */
/* the driver to handle improper messages and other errors robustly.    */
/* Appropriate error messages will be printed when operating errors are */
/* detected.  DEBUG should be defined until the code is ready to ship.  */
/*
#define DEBUG
*/

/* Whenever a TRACE is defined, some kind of printed output statements  */
/* will be generated to the console.  Normally NO tracing should be set */
/*
#define DAP_TRACE
#define QSTAT_TRACE
#define ABORT_TRACE
#define LOCK_GRANT_TRACE
#define CONFIG_TRACE
#define DA_TRACE       0
#define BUS_TRACE      0
#define INT_TRACE      0
#define CS80_TRACE     0
#define RDWR_TRACE     0
#define DIAG_TRACE     0
#define PORT_TRACE     0
*/

/* These are some of the various options available in the code */
/*
#define DAP_CRC_ON
#define DAP_RW_BURST_ON
#define CS80_CRC_ON
#define GENERIC_CRC_ON
#define HP27110A_COMPAT

#define CHECK_HPMC
#define BUTLER_DEBUG
#define BUSY_SELFTEST
#define IN_FIFO_CHECK
#define POLL_FOR_PPR_IN_GET_BUS
*/

/* Enable LED to display DMA activity */
/*
 * Turn off this flag, since LED that displays this info
 * is supposed to only be used for self-test.
#define LED_ACTIVITY 
 */

#define IS_CS80(id)      (((id) & 0x0f00) == 0x0200)

#define SETUNIT0         0x20
#define DESCRIBE_CMD     0x35
#define DESCRIBE_LEN     37
#define DESCRIBE_TO      2
#define DESCRIBE_RETRY   3
#define DEVICE_NUM       6
#define CS80_STATUS_LEN  20

/*
 * Probe status buffer needs 2 more bytes: 
 *  ST_DSR_CMD_1 starts 1 byte into the status buffer,
 *  ST_DSR_EXEC_3 needs 1 byte after.
 */
#define QSTAT_BYTES	 2

typedef struct cs80_2cmd {
    unsigned char     cmd1;
    unsigned char     cmd2;
    unsigned char     filler[62];
} cs80_2cmd_t;

/************************************************************************/
/*  This is where the HPIB DAM defines which messages it can understand */
/************************************************************************/

/* Using contents of "../sio/llio.h" here.   Should be include after llio.h */

typedef struct {
	llio_std_header_type		msg_header;
	union {
		creation_info_type 		creation_info;
		do_bind_req_type 		do_bind_req;
		do_bind_reply_type 		do_bind_reply;
		bind_req_type 			bind_req; 
		bind_reply_type 		bind_reply;
		probe_req_type          	probe_req;    
		probe_reply_type        	probe_reply;
		unbind_req_type 		unbind_req; 
		unbind_reply_type 		unbind_reply;
		do_unbind_req_type 		do_unbind_req;
		do_unbind_reply_type 		do_unbind_reply;
		die_req_type 		  	die_req;
		die_reply_type 			die_reply;
		dam_req_type 			dam_req;
		dam_reply_type			dam_reply;
		dam_event_type			dam_event;
		int_direct_type		        dam_int_direct;
		diag_req_type 			diag_req;
		diag_reply_type 		diag_reply;
		diag_event_type			diag_event;
		reset_req_type 			reset_req;
		reset_reply_type 		reset_reply;
		lock_req_type			lock_req;
		lock_reply_type 		lock_reply;
		unlock_req_type	 		unlock_req;
		unlock_reply_type 		unlock_reply;
	} u;
} hpib_message_type;



/* Make sure that a few miscellaneous but useful things are defined.	*/
#ifndef TRUE
#define	TRUE     1
#define	FALSE    0
#endif
#ifndef NULL
		#undef NULL
#define	NULL 	0
#endif


#undef  CONTINUE       /* defined in ../sio/iocam.h */
#define BREAK      0
#define CONTINUE   1

#define PPOLL_DISABLE   0
#define PPOLL_ENABLE    1

#define	NUM_MEDUSA_REGS		8


/*
 *               Medusa bit position numbering (16 bit word)
 *      -------------------------------------------------------------
 *      |  0 |  1 | . . . . |  8 |  9 | 10 | 11 | 12 | 13 | 14 | 15 |
 *      -------------------------------------------------------------
 */

/* The Medusa HPIB interface chip uses a ten bit wide data bus.  The	*/
/* chip appears to the host processor as a bank of eight 10-bit wide	*/
/* registers.  If the I/O path cannot support ten bit wide transfers	*/
/* Medusa is capable of operating in an eight bit wide mode.  In this	*/
/* mode whenever any register other than register 3 is read by the	*/
/* host, Medusa places the high order bits of the register into bits	*/
/* 8 and 9 of register 3.  Conversely, if any register other than	*/
/* register 3 is written in 8-bit mode then bits 8 and 9 of register 3	*/
/* are taken as the source of the high order bits for the register	*/
/* being written.							*/

/* Medusa register 0 - INTERRUPT CONDITIONS				*/
/* Medusa register 1 - INTERRUPT MASK					*/
/*	All of the following bit masks are common to both registers	*/
/*	except for INTR_PENDING and INTR_ENABLE.			*/
/*	INTR_PENDING is specific to Medusa register 0.			*/
/*	INTR_ENABLE is specific to Medusa register 1.			*/
#define	INTR_PENDING		0x0200
#define	INTR_ENABLE		0x0200

#define	PARITY_ERROR		0x0100
#define	STATUS_CHG		0x0080
#define	HDSHK_ABORT		0x0040
#define	PPOLL_RESP		0x0020
#define	SERVICE_REQ		0x0010
#define	OUT_FIFO_AVAIL		0x0008
#define	IN_FIFO_AVAIL		0x0004
#define	OUT_FIFO_IDLE		0x0002
#define	DEV_CLEAR		0x0001


/* Medusa register 2 (write) - OUTBOUND FIFO				*/
#define	DATA_MASK		0x0000
#define	CMD_MASK		0x0100
#define	EOI_MASK		0x0200
#define	LF_COUNTED_BTE		0x0000
#define	COUNTED_BTE		0x0200
#define	UNCOUNTED_BTE		0x0300

#define MAX_MEDUSA_COUNT	0x00ff

/* Medusa register 2 (read) - INBOUND FIFO				*/
#define	IN_TYPE_MASK		0x0300

#define	DATA_BYTE		0x0000
#define	SECONDARY_ADDR		0x0100
#define	SUBGROUP_END		0x0200
#define	RECORD_END		0x0300

#define	TALK_FLAG		0x0020
#define	SECONDARY_ADDR_MASK	0x001F

/* Medusa register 3 - STATUS						*/
#define	RX_HIGH_BITS		0x00C0
#define	REMOTE			0x0020
#define	CONTROLLER		0x0010
#define	SYS_CONTROLLER		0x0008
#define	TALK_MODE		0x0004
#define	LISTEN_MODE		0x0002
#define	OUTBOUND_FREEZE		0x0001

/* Medusa register 4 - CONTROL						*/
#define	POLL_HOLDOFF		0x0200
#define	HNDSHK_DELAY		0x0100
#define	EIGHT_BIT_MODE		0x0080
#define	PARITY_FREEZE		0x0040
#define	REMOTE_ENABLE		0x0020
#define	INTERFACE_CLEAR		0x0010
#define	PPOLL_RESP_ON		0x0008
#define	REQUEST_SERVICE		0x0004
#define	OUT_FIFO_DMA		0x0002
#define	INIT_OUT_FIFO		0x0001

/* Medusa register 5 - HPIB ADDRESS					*/
#define	CRC_ENABLE		0x0200
#define	EVEN_PARITY		0x0100
#define	ONLINE			0x0080
#define	TALK_ALWAYS		0x0040
#define	LISTEN_ALWAYS		0x0020
#define	HPIB_ADDR_MASK		0x001F

/* Medusa register 6 - PARALLEL POLL MASK / FIRST ID BYTE		*/
#define	PPOLL_MASK		0x00FF
#define	FIRST_ID		0x00FF

/* Medusa register 7 - PARALLEL POLL SENSE / SECOND ID BYTE		*/
#define	PPOLL_SENSE		0x00FF
#define	SECOND_ID		0x00FF





/* HPIB Bus Status Register Masks */
#define BUS_STATUS_EOI		0x0080
#define BUS_STATUS_REN		0x0040
#define BUS_STATUS_SRQ		0x0020
#define BUS_STATUS_ATN		0x0010
#define BUS_STATUS_IFC		0x0008
#define BUS_STATUS_DAV		0x0004
#define BUS_STATUS_NDAC		0x0002
#define BUS_STATUS_NRFD		0x0001


/* System I/O Control Register Masks */
#define SYS_IO_CTRL_FAULT_LED                   0x0001
#define SYS_IO_CTRL_NOT_IFC_CLR                 0x0002
#define SYS_IO_CTRL_NOT_GET_CLR                 0x0004
#define SYS_IO_CTRL_EOI_ENABLE                  0x0010
#define SYS_IO_CTRL_HPIB_FAST                   0x0040
#define SYS_IO_CTRL_HPIB_SCTRL           	0x0080

/* System I/O Status Register */
#define SYS_IO_STAT_FAULT_LED                   0x0001
#define SYS_IO_STAT_IFC_INTR                    0x0002
#define SYS_IO_STAT_GET_INTR                    0x0004
#define SYS_IO_STAT_MEDUSA_INTR                 0x0008
#define SYS_IO_STAT_DMA_RD_D0                   0x0010
#define SYS_IO_STAT_DMA_RD_D1                   0x0020
#define SYS_IO_STAT_HPIB_FAST                   0x0040
#define SYS_IO_STAT_HPIB_SCTRL           	0x0080


#define END_ON_EOI              0
#define DA_INT_DISABLE_MASK  	0xDF00
#define DIAG_INT_DISABLE_MASK   0xBE


#define DIP_HPIB1_FAST_MODE        0x00000040
#define DIP_HPIB1_SYS_CONTROLLER   0x00000020
#define DIP_HPIB1_ADDRESS_MASK     0x0000001f


/************************************************************************/
/* Structures and constants describing the HPIB interface hardware.	*/
/************************************************************************/

typedef unsigned int volatile u_int_v;	/* card registers must be volatile */

typedef struct {                      

        dma_A_lower_page_type  	shazam;       /* Protected Page */

	 				/* register 512 - byte 0x800 */
        u_int_v unused_0[256];          /*          through          */
	 				/* register 787 - byte 0xbff */
	u_int_v medusa_int_cond;	/* register 768 - byte 0xc00 */
	u_int_v medusa_int_mask;	/* register 769 - byte 0xc04 */
	u_int_v medusa_fifo;		/* register 770 - byte 0xc08 */
	u_int_v medusa_status;		/* register 771 - byte 0xc0c */
	u_int_v medusa_control;		/* register 772 - byte 0xc10 */
	u_int_v medusa_address;		/* register 773 - byte 0xc14 */
	u_int_v medusa_ppoll_mask;	/* register 774 - byte 0xc18 */
	u_int_v medusa_ppoll_sense;	/* register 775 - byte 0xc1c */

#define medusa_id1 medusa_ppoll_mask
#define medusa_id2 medusa_ppoll_sense

        u_int_v n_bus_status;		/* register 776 - byte 0xc20 */

	u_int_v unused_1;               /* register 777 - byte 0xc24 */

	union {				/* register 778 - byte 0xc28 */
	    u_int_v control;
	    u_int_v status;
	} module_reg;

	u_int_v unused_2[245];

} h1_card_type;



/* Medusa uses address 30 to talk and listen, regardless of what is	*/
/* in the address register (R5).					*/
#define	CIC			30

/* HPIB commands */
	/* Addressed commands: */
#define	GTL			0x01	/* Go To Local			*/
#define	SDC			0x04	/* Selective Device Clear	*/
#define	PPC			0x05	/* PPoll Configure		*/
#define	GET			0x08	/* Group Execute Trigger	*/
#define	TCT			0x09	/* Take ConTrol			*/
	/* Universal commands: */
#define	LLO			0x11	/* Local LOckout		*/
#define	DCL			0x14	/* Device CLear			*/
#define	PPU			0x15	/* PPoll Unconfigure		*/
#define	SPE			0x18	/* SPoll Enable			*/
#define	SPD			0x19	/* SPoll Disable		*/
	/* Listen address: */
#define	LTN(addr)       (addr | 0x20)   /* make Listener at bus address */
#define	TLK(addr)       (addr | 0x40)   /* make Talker at bus address 	*/
#define UNL             LTN(31) 	/* UNListen			*/
#define UNT             TLK(31) 	/* UNTalk			*/

#define SC_CMD(n)       (0x60 | n)	/* Secondary command n 		*/

#define CMD(val)       (CMD_MASK | (val))  /* Make Command Value */
#define DATA(val)      (DATA_MASK | (val)) /* Make Data Value */
#define DATA_EOI(val)  (EOI_MASK | (val))  /* Make Data with EOI */

/* Identify command sequence 						*/
#define IDY_1           TLK(31)      	/* First byte of Identify 	*/
#define	IDY_2(addr)	SC_CMD(addr)    /* Secondary Identify Command 	*/


/* CS/80 secondary commands */
#define SC_CS80_TRANS         SC_CMD(0x12)   /* Transparent phase */
#define SC_CS80_CMD           SC_CMD(0x05)   /* Command     phase */
#define SC_CS80_EXEC          SC_CMD(0x0E)   /* Execution   phase */
#define SC_CS80_REPORT        SC_CMD(0x10)   /* Reporting   phase */

/* CRC secondary commands */
#define	SC_CRC	 	      SC_CMD(0x11)   /* CRC secondary command	*/





/* This constant is used as a parameter to busywait() to delay for the	*/
/* 100us required by IEEE-488 after changing the IFC or REN lines.	*/
/* We're going to fudge a little here to minimize the actual wait time.	*/
/* Some of the wait time will be consumed by calling and returning from	*/
/* busywait() plus any other overhead.					*/
#define	IEEE_100US		95


#define	HPIB_QUEUE_DEPTH   	1
#define	NUM_SUBQUEUES 		32
#define	NUM_STD_MSGS		(3 * NUM_SUBQUEUES)


#define	HPIB_POWERON_Q		0  /* non-maskable */

#define	HPIB_LOCK_Q		1  /* maskable (with care) */
#define	HPIB_ABORT_Q		1
#define	HPIB_CONFIG_Q		1

#define	HPIB_NON_DEST_DIAG_Q	2  /* maskable */  /* not used now */

#define HPIB_INT_EVENT_Q        3  /* maskable */

#define HPIB_TIMER_Q            4  /* maskable */

/* Polls H/W periodically */
#define HPIB_HW_POLL_Q          5  /* maskable */

#define HPIB_LOCK_GRANT_Q       6  /* maskable */

#define HPIB_DM_Q0              7  /* maskable (with care) */

#define	HPIB_INIT_SUBQS	      (	ELEMENT_OF_32(HPIB_POWERON_Q)	     | \
				ELEMENT_OF_32(HPIB_LOCK_Q)	     | \
				ELEMENT_OF_32(HPIB_ABORT_Q)	     | \
				ELEMENT_OF_32(HPIB_CONFIG_Q)	     | \
				ELEMENT_OF_32(HPIB_INT_EVENT_Q)      | \
				ELEMENT_OF_32(HPIB_TIMER_Q)          | \
				ELEMENT_OF_32(HPIB_HW_POLL_Q)        | \
                                ELEMENT_OF_32(HPIB_LOCK_GRANT_Q) )


#define	HPIB1_REV_CODE		900801

/* (8 daps) lp1/tape1, (8) disc1, (1) instr0, (1) diaghpib1 */
#define	MAX_HPIB_DEV_MGRS       (NUM_SUBQUEUES - HPIB_DM_Q0)

#define	MAX_DEVICES_PER_MGR    15

#define	HPIB_MAX_PPOLL		8
#define MAX_CS80_DEVICES        8

/* Only one transaction per device pending (a sensible assumption) on   */
/* an interface. This is fairly safe, since multiple transactions per   */
/* device is not defined. 						*/
#define	MAX_HPIB_QUEUED_REQS	MAX_DEVICES_PER_MGR

/* Define some DAP related numbers */
#define MAX_DAP_PGRMS       8
#define MAX_DAP_PGRM_SIZE   512
#define MAX_PINDEX          62
#define MAX_SINDEX          62

/* bit defintions for dap[].status */
#define DAP_LOADED	    0x01
#define DAP_IN_USE	    0x02
#define DAP_LOCKED	    0x04

#define DAP_ADDR_MASK 	    0x7


#define	KERNEL_SID		0
#define	LF			0x0A

#define XFR_BUFFER_SIZE         NBPG    /* Greater than 8 and a multiple too */

#define ODD(val)  (val & 0x1)

/* status.u.proc_num values */
#define	LOCAL			-1  /* Local DAM errors */
#define	GLOBAL			 0  /* LLIO errors      */

#define SET_TSTAT(proc, error)  h1set_status(proc, error, &tptr->reply_status);

#define TR_END	     0x01	/* End with EOI			       */
#define TR_LF	     0x02	/* Match a LF on input		       */
#define	TR_COUNT     0x04       /* Terminate(d) on character count     */
#define TR_MATCH     0x08   	/* Terminated(d) on character match          */
#define TR_UNA	     0x10	/* do SPD, UNT, and UNL after data transfer  */
#define	TR_IFC       0x20       /* Clear interface with SPD, UNT, UNL  */
				/* or IFC when transfer interrupted    */

/*	TR_ERROR		   Terminated on error (eg, MSA)	*/
/*	TR_TIMEOUT		   Terminated on timeout		*/
/*	TR_END			   Terminated on device EOI		*/
/*	TR_MATCH     0x08	   Terminate(d) on character match	*/
/*	TR_NOTERM    0x00	   No read done yet			*/




enum write_type {
	WR_DATA_EOI = 0,
	WR_DATA_NO_EOI,
	WR_CMD
};



enum tran_states {
	ST_INIT = 0, ST_INIT_1,
	ST_COMMAND,  ST_CMD_1,  ST_CMD_2,  ST_CMD_3,  ST_CMD_4,
	ST_EXECUTE,  ST_EXEC_1, ST_EXEC_2, ST_EXEC_3, ST_EXEC_4,
	             ST_EXEC_5, ST_EXEC_6, ST_EXEC_7, ST_EXEC_8,
	ST_REPORT,   ST_REP_1,  ST_REP_2,  ST_REP_3,
	ST_DSR_COMMAND, ST_DSR_CMD_1,
	ST_DSR_EXECUTE,
	ST_DSR_EXEC_1, ST_DSR_EXEC_2, ST_DSR_EXEC_3,
	ST_DSR_REPORT, ST_DSR_REP_1, ST_DSR_REP_2,
	ST_TIMEOUT, ST_ABORT,
	ST_PF_ABORT
        /* switch instructions sizes are SMALLer when this is set to 100 */
};




/* Internal states (modes) */
#define DA_REMOTE_ON          0x01
#define DA_RESP_TO_PPOLL_ON   0x02
#define DA_SPOLL_ON           0x04
#define DA_ONLINE             0x08
#define DA_ATN_ASSERTED       0x10
#define DA_TLK_ALWAYS         0x20
#define DA_LTN_ALWAYS         0x40
#define DA_PARITY_FREEZE_ON   0x80


/* Interrupt mask enables */       /* Interrupt generated when:		*/
#define DA_INTR_CIC 	 0x8000    /* controller in charge		*/
#define DA_INTR_REM      0x4000    /* remote control mode		*/
#define DA_INTR_SRQ      0x1000    /* service request issued		*/
#define DA_INTR_TLK      0x0800    /* set to talk			*/
#define DA_INTR_LTN      0x0400    /* set to listen			*/
#define DA_INTR_DAT      0x0200    /* inbound unclaimed byte available	*/
#define DA_INTR_NCC      0x0100    /* not controller in charger		*/
#define DA_INTR_MSA      0x0080    /* secondary address intercepted	*/
#define DA_INTR_PTY      0x0040    /* parity error occurred 		*/
#define DA_INTR_GET      0x0020    /* group execut trigger occurred	*/
#define DA_INTR_IFC      0x0002    /* IFC occurred			*/
#define DA_INTR_DCL      0x0001    /* universal or address device clear */

#define POLLED_STATES    (DA_INTR_TLK | DA_INTR_LTN | DA_INTR_CIC | \
			  DA_INTR_NCC | DA_INTR_REM)

typedef void (*FUNC_PRV)();
typedef int  (*FUNC_PRI)();


enum dma_modes {
	DMA_IDLE = 0,
	DMA_ACTIVE,
	DMA_PAUSED
};


struct port_hpa_rec {
	h1_card_type        	*hpa;
	port_num_type        	 port_num;
	struct port_hpa_rec 	*next;
};



typedef	struct {
	port_num_type                 	port_num;
	shortint			probe_id;
	bind_req_type            	bind_info;
	u_int				dm_index;
	u_char				in_pon_cycle;
	u_short 			misc_int_enable;
	u_char				ppoll_int_enable;
	u_char				unsolicited_ppoll_int_enable;
	u_short                         misc_int_log;
	u_char                          ppoll_int_log;
} hpib_dm_info_type;



typedef struct {
	u_char 		*p_buf;
	u_char		*s_buf;
        u_char		*pc_ptr;
        u_char		*timer_pc_ptr;
        u_int 		 timer_sindex;
#ifdef DAP_RW_BURST_ON
	u_char           burst_size_used;
	u_int 		 num_bursts;
	u_int 		 bursts_sent;
	u_int 		 burst_size;
	u_int 		 burst_count;  /* Actual count */
#endif /* DAP_RW_BURST_ON */
} dap_tran_type;




typedef struct hpib_tran_rec {
        u_char				use_dma;
        u_char				loopback;
	dam_req_type			dam_req;
	int                             cmd_residue;
	int                             data_residue;
	int                             status_residue;
	llio_status_type		reply_status;
	FUNC_PRI		        handler;
	hpib_dm_info_type	       *dm_p;
	enum tran_states		state;
	struct hpib_tran_rec 	       *link;
	hpib_message_type	       *msg_ptr;
	enum tran_states		prev_state;
	int 				timer_used;
	int				timer_id;
        dap_tran_type                   dap;
        dma_A_chain_type	       *chain;
} hpib_tran_type;



typedef	struct {

#if defined(DAP_CRC_ON) || defined (CS80_CRC_ON) || defined(GENERIC_CRC_ON)
	unsigned int            crc_mode;
	u_char			crc_da_byte [2];
	u_char			crc_dev_byte[2];
#endif

	unsigned char	       *aligned_buf;
	unsigned int 		buf_cnt;        /* bytes in buffer */
	unsigned int		sid;
	anyptr			vaddr;
	unsigned int		init_cnt;       /* initial count   */
	unsigned int		count;
	u_short			s_scratch;      /* short int scratch area  */
	u_char                  a_scratch[4];   /* char array scratch area */
	u_char			uses_intr;
	u_char			out_fifo_size;
	u_char			dap_eoi_mode;
	u_char			qstat;
	u_char			t_cmd;
	u_char			last_sec_addr;
	union {
		struct {
			/* Valid end conditions for read: MATCH, CNT, LF, EOI */
			u_int 	end_conds;
			u_int 	last_byte;
			u_int 	match_byte;
		} rd;
		struct {
			u_int	top_bits;
			u_int	end_top_bits;
		} wr;
	} ctrl;
} hpib_xfr_type;




typedef	struct {
	hpib_tran_type		*owner;
	hpib_tran_type		*queued;
	hpib_tran_type       	*free_list;
} hpib_bus_trans_type;




typedef	struct {
	FUNC_PRI	 in_fifo_avail;
	FUNC_PRI	 out_fifo_idle;
	FUNC_PRI	 talker;
} hpib_isr_type;





typedef struct {
	hpib_tran_type 	       *ppoll_wait[HPIB_MAX_PPOLL];
	u_int          	        ppoll_wait_cnt;
	u_int			delay;
        u_int			io_ctrl_reg;
	u_char      		modes;
	u_char			hpib_addr;
	u_char 			amigo_idy1;
	u_char 			amigo_idy2;
	u_char 			ppoll_sense;
	u_char			ppoll_int_log;
#ifdef HP27110A_COMPAT
	u_short 		misc_int_wait;
	u_char			ppoll_int_wait;
#endif /* HP27110A_COMPAT */
	u_short 		misc_int_log;
	u_short 		misc_int_enable;
	u_char			ppoll_int_enable;
	u_char			disable_interrupt;
} da_state_type;





typedef struct {
    u_int		status;
    u_char	       *program;
} dap_program_type;




typedef struct {
    u_char              waiting_on_int;
    u_char              dma_waiting_on_int;
    int                 last_trans_num;
    shortint            last_msg_id;
    port_num_type       event_port;
    io_subq_type        event_subq;
    int                 diag_timer_id;
    diag_option_type    current_opt;
    u_int               reg_value;
    u_int               iodc_rtn[4];
    diag_req_info_type *diag_req_info;
    hpib_tran_type     *tptr;
} diag_info_type;



/*******************************************************************/
/* port_num:    is this managers port number     		   */
/* hpa:         is a ptr to the virtual card i/o address           */
/* eim:         is the extern interrupt message address		   */
/* hw_class:    is this module's hardware class (Shazam vs Lo-quix)*/
/* bus_trans:   is the data to manage the bus trans req and alloc  */
/* xfr:         is the data associated with the current tranfer    */
/* dm_array:    is an array of ptrs to device manager info records */
/* isr:         contains the function ptrs used during interrupts  */
/* dap:         contains the dap program and status for the device */
/* da_state:    contains state info on device adapter		   */
/*******************************************************************/

typedef struct {
	u_char				cs80_device_busy[MAX_CS80_DEVICES];
        llio_std_header_type 	        cs80_timer_msg[MAX_CS80_DEVICES];

	u_char				port_bound;            /* boolean */
        u_char				reset_during_lock;     /* boolean */
	u_char				in_pon_cycle;	       /* boolean */

	int				power_on_trn_num;
        shortint                        power_on_msg_id;
        port_num_type			power_on_port_num;

	u_char			        nested_pon;	       /* boolean */
	int				nested_pon_trn_num;
        shortint                        nested_pon_msg_id;
        port_num_type			nested_pon_port_num;

        /* Variables used for locking (always valid) */
        u_char				port_locked;           /* boolean */
	u_char				port_being_locked;     /* boolean */
        u_char				doing_diagnostics;     /* boolean */
	/* valid when either port_locked or port_being_locked */
	hpib_tran_type 	               *lock_tptr;

	hpib_tran_type		       *queued_pf_aborts;

	/* Variables used for probing */
	u_char				probe_addr;    
           /* for hpib interface, char or u_char address works out
            better than the "int" that is passed from probe_req */
	u_char				probe_option;    /* bit8    */
	shortint			probe_id;
	int     			desc_retry;
	cs80_2cmd_t			cs80cmd;
	u_char				desc[MAX_CS80_DEVICES][DESCRIBE_LEN];
	u_char				status[MAX_CS80_DEVICES][CS80_STATUS_LEN+QSTAT_BYTES];

	h1_card_type	 	       *hpa;
	u_int				eim;
	u_int				hw_class;
	port_num_type			port_num;
	hpib_bus_trans_type 		bus_trans;
	hpib_xfr_type	        	xfr;
	hpib_isr_type			isr;


	enum dma_modes			dma_mode;
        u_char                          granting_lock;
        int                             lock_grant_timer_id;
	int                             hw_poll_timer_id;
	da_state_type			da_state;
        diag_info_type                  diag_info;
	dap_program_type		dap[MAX_DAP_PGRMS];

	u_int				dm_count;
	u_int			        dm_pon_count;
	u_int			        dm_max_index;
	hpib_dm_info_type	       *dm_array[MAX_HPIB_DEV_MGRS];
} hpib_pda_type;





#define	CMD_PTR			request_parms.cmd_ptr
#define	CMD_COUNT		request_parms.cmd_count
#define	CMD_CLASS		request_parms.cmd_class
#define	DATA_PTR		request_parms.data_ptr
#define	DATA_COUNT		request_parms.data_count
#define	DATA_CLASS		request_parms.data_class
#define	STATUS_PTR		request_parms.status_ptr
#define	STATUS_COUNT		request_parms.status_count
#define	STATUS_CLASS		request_parms.status_class

#define	CMD_SID			request_parms.cmd_ptr.u.data_sid
#define	CMD_VBA		        request_parms.cmd_ptr.u.data_vba
#define	DATA_SID		request_parms.data_ptr.u.data_sid
#define	DATA_VBA		request_parms.data_ptr.u.data_vba
#define	STATUS_SID		request_parms.status_ptr.u.data_sid
#define	STATUS_VBA		request_parms.status_ptr.u.data_vba


#define PPOLL_DELAY(delay) (25*((delay)+1)*((delay)+1))

#define	HPIB_IS_CIC(cp)    ((cp)->medusa_status & CONTROLLER)
#define	HPIB_IS_SC(cp)	   ((cp)->medusa_status & SYS_CONTROLLER)
#define	HPIB_IS_TLK(cp)    ((cp)->medusa_status & TALK_MODE)
#define	HPIB_IS_LTN(cp)    ((cp)->medusa_status & LISTEN_MODE)
#define	HPIB_IS_FROZEN(cp) ((cp)->medusa_status & OUTBOUND_FREEZE)
#define	HPIB_IS_PPR(cp)    ((cp)->medusa_control & PPOLL_RESP_ON)
#define	HPIB_IS_ONLINE(cp) ((cp)->medusa_address & ONLINE)


#ifdef DEBUG
#define DMA_CHIP(pda)	(((pda)->hw_class == DMA_A_LOQUIX_HW) ? "Lo-quix" : "Shazam")
#endif /* DEBUG */

#define THERE_IS_A_BYTE(pda)   \
	   h1wait_for_8bit(pda, IN_FIFO_AVAIL, h1_try_in_fifo)
#define OUT_FIFO_IS_EMPTY(pda) \
	   h1wait_for_8bit(pda, OUT_FIFO_IDLE, h1_try_out_fifo)

/* Poll cost is about (CNT * 5us)+70us, whereas an interrupt is 700us */
#define PPR_POLL_CNT(pda) (h1_try_ppr << (pda->da_state.ppoll_wait_cnt >> 1))

#define POWER_FAIL(hpa) \
    (((hpa)->shazam.srs.io_status & FATAL_ERROR) ? io_em_fe(hpa) : 0)

#define	DIO_LINE_MASK		0x07
#define PPOLL_RESP_BIT		0x80

#define	CS80_REPORT_STATUS	0x0D
#define	CS80_STATUS_LEN		20

#define CS80_TIMEOUT      u.cs80_req.timeout_value
#define CS80_CRC          u.cs80_req.crc
#define CS80_TRANS        u.cs80_req.transparent

/* Since all bus lines float high when no devices are      */
/* listening, software knows that no devices are connected */
/* to HPIB when it sees NRFD and NDAC both HIGH (1:FALSE). */
/* If this condition is not checked, Medusa has no way of  */
/* knowing by itself whether the data it is handshaking    */
/* out is actually being received by a device.  Having no  */
/* device listening looks to Medusa like an infinitely     */
/* fast device.  Should be called after addressing done.   */

#define NO_LISTENER_ON_BUS(hpa)  ((((hpa)->n_bus_status & 		  \
                                  (BUS_STATUS_NRFD | BUS_STATUS_NDAC)) == \
                                  (BUS_STATUS_NRFD | BUS_STATUS_NDAC)) &&  \
                                  (! HPIB_IS_LTN(hpa)))


/* If "addr" is not aligned on a "size" byte boundary then this macro	*/
/* aligns it on the next highest "size" byte boundary.  "size" must be	*/
/* a power of two.							*/
#define	ALIGN(size, addr)   \
    ((unsigned char *)(((unsigned int)(addr)+((size)-1)) & (~((size)-1))))


#define	M_HDR			msg_ptr->msg_header
#define	M_CREATE		msg_ptr->u.creation_info
#define	M_DO_BIND_REQ           msg_ptr->u.do_bind_req
#define	M_DO_BIND_REPLY         msg_ptr->u.do_bind_reply
#define	M_BIND_REQ		msg_ptr->u.bind_req
#define	M_BIND_REPLY		msg_ptr->u.bind_reply
#define	M_DO_UNBIND_REQ         msg_ptr->u.do_unbind_req
#define	M_DO_UNBIND_REPLY       msg_ptr->u.do_unbind_reply
#define	M_UNBIND_REQ		msg_ptr->u.unbind_req
#define	M_UNBIND_REPLY		msg_ptr->u.unbind_reply
#define	M_DIE_REQ		msg_ptr->u.die_req
#define	M_DIE_REPLY		msg_ptr->u.die_reply
#define	M_IOREQ		        msg_ptr->u.dam_req
#define	M_IOREPLY		msg_ptr->u.dam_reply
#define M_EVENT                 msg_ptr->u.dam_event
#define	M_DIAG_REQ		msg_ptr->u.diag_req
#define M_UTIL_REQ              msg_ptr->u.diag_req.u.hw_util_info
#define M_IDENT_REQ             msg_ptr->u.diag_req.u.hw_identify_info
#define M_SELFTEST_REQ          msg_ptr->u.diag_req.u.hw_self_test_info
#define	M_DIAG_REPLY		msg_ptr->u.diag_reply
#define	M_RESET_REQ		msg_ptr->u.reset_req
#define	M_RESET_REPLY		msg_ptr->u.reset_reply
#define	M_LOCK_REQ		msg_ptr->u.lock_req
#define	M_LOCK_REPLY		msg_ptr->u.lock_reply
#define	M_UNLOCK_REQ		msg_ptr->u.unlock_req
#define	M_UNLOCK_REPLY		msg_ptr->u.unlock_reply

#define T_HDR                   tptr->msg_ptr->msg_header
#define T_IOREQ                 tptr->dam_req

#define	MSGLEN(msgtype)		(sizeof(llio_std_header_type)+sizeof(msgtype))

#define STATUS_IFC_INT        0x4
#define STATUS_MEDUSA_INT     0x2
#define STATUS_GET_INT        0x1

#define STATUS_POLL_TIME      100000    /* tenth of a second */
#define LOCK_GRANT_TIME       1         /* minimum time      */

#define NORMAL_COMPLETION     0x0


#ifdef	_KERNEL

/* INTERNAL hpib1.c ROUTINES */
extern llio_status_type 	hpib1();
extern void 			h1creation_event();
extern void 			h1do_bind_request();
extern void 			h1do_bind_reply();
extern void 			h1bind_request();
extern void 			h1bind_reply();
extern void 			h1unbind_request();
extern void 			h1unbind_reply();
extern void 			h1do_unbind_request();
extern void 			h1do_unbind_reply();
extern void 			h1die_request();
extern void 			h1die_reply();
extern void 			h1io_request();
extern void 			h1dam_reply();
extern void 			h1io_reply();
extern void 			h1int_direct_event();
extern void 			h1timer_event();
extern void 			h1abort_event();
extern void 			h1power_on_request();
extern void 			h1power_on_reply();
extern void 			h1lock_request();
extern void 			h1lock_grant();
extern void 			h1lock_abort();
extern void 			h1lock_reply();
extern void 			h1probe_request();
extern void 			h1start_probe();
extern void 			h1probe_reply();
extern void 			h1reset_request();
extern void 			h1reset_reply();
extern void 			h1unlock_request();
extern void 			h1unlock_reply();
extern void             	h1diag_request();
extern void             	h1diag_selftest();
extern void             	h1diag_identify();
extern void             	h1diag_utility();
extern void             	h1diag_rd_wr_reg();
extern void             	h1diag_dma();
extern void             	h1diag_dma_rd_term();
extern void             	h1diag_interrupt();
extern void             	h1diag_process_event();
extern void             	h1diag_send_event();
extern void             	h1diag_abort();
extern void             	h1diag_reply();

extern int 			h1get_bus ();
extern int			h1get_dm_index();
extern void 			h1release_bus ();
extern hpib_tran_type *		h1get_tran ();
extern int			h1wait_for_8bit();
extern void			h1abort_io_tran();
extern int			h1any_active_io_trans();
extern void			h1abort_all_active_io_trans();
extern int 			h1unaddress_bus();
extern u_int			h1get_byte_to_write ();
extern int			h1copy_bytes ();
extern void			h1flush_read_bytes ();
extern void			h1cpu_rw_cleanup ();
extern int 			h1do_the_cpu_rd ();
extern int 			h1do_the_cpu_wr ();
extern int			h1cpu_rd ();
extern int			h1cpu_wr ();
#ifdef	DEBUG
extern void			h1print_internal_state ();
extern void			h1print_transaction_state ();
extern void 			h1check_out_fifo_avail();
extern void			h1print_val ();
extern void			h1print_val_8bit ();
extern void			h1check_bus_release ();
#endif	/* DEBUG */
extern u_int			h1is_bit_set();
extern void             	h1disable_EOI();
extern void             	h1enable_EOI();
extern void             	h1enable_ifc();
extern void             	h1enable_get();
extern void			h1enable_ppoll ();
extern void			h1disable_ppoll ();
extern void			h1enable_in_fifo ();
extern void			h1disable_in_fifo ();
extern void			h1save_secondary ();
extern void			h1set_status ();
extern void			h1fifo_idle_so_call_hdlr();
extern void			h1da_enable_hw_poll_timer();
extern void			h1da_recalc_and_enable_int();
#ifdef HP27110A_COMPAT
extern void			h1wait_int_execute();
#endif /* HP27110A_COMPAT */
extern void			h1event_int_execute();
extern void    			h1da_control ();
extern void    			h1cs80_execute ();
extern void    			h1generic_execute ();
extern void    			h1dap_download ();
extern void    			h1dap_execute ();
extern void			h1unsupported_req();
extern void    			h1int_dev_clear();
extern void    			h1int_out_fifo_idle();
extern void    			h1int_in_fifo_avail();
extern void    			h1int_service_req();
extern void    			h1int_ppoll_resp();
extern void    			h1int_hdshk_abort();
extern void    			h1int_parity_error();
extern void    			h1int_ifc();
extern void    			h1int_get();
extern void    			h1ppoll_int_exec ();
extern void 			h1cs80_wrap_up ();
extern void 			h1start_longtimer();
extern void 			h1stop_longtimer();
extern void 			h1power_on_module();
extern void			h1queue_pf_abort();
extern void			h1abort_queued_pf_aborts();
extern void			h1pon_cycle_done();
extern void 			h1send_dm_power_on_req();
extern void 			h1send_power_on_reply();
#if	defined(CS80_CRC_ON) || defined(GENERIC_CRC_ON)
extern int			h1crc_init();
extern int			h1crc_check1();
extern int			h1crc_check2();
extern int			h1crc_compare();
#endif	/* CS80_CRC_ON || GENERIC_CRC_ON */

extern struct timeval ms_gettimeofday();

#ifdef PORT_TRACE
#define io_send h1IO_SEND
extern void			h1IO_SEND ();
#endif /* PORT_TRACE */

/* MEDUSA I/O INTERACE ROUTINES */
extern u_int 		h1med_rd10_int_cond ();
extern u_int 		h1med_rd10_int_mask ();
extern u_int 		h1med_rd10_fifo ();
extern u_int 		h1med_rd10_control ();
#ifdef	DEBUG
extern u_int 		h1med_rd10_address ();
#endif	/* DEBUG */

extern void	 	h1med_wr_int_cond ();
extern void 		h1med_wr_int_mask ();
extern void 		h1med_wr_fifo ();
extern void 		h1med_wr_control ();
extern void 		h1med_wr_address ();
#endif	/* _KERNEL */


/* These routines set and clear the bits specified in rmask     */
/* These routines must not be interrupted, as state information */
/* is stored in the status register.  Medusa must be in 8-bit   */
/* and DMA activity must be paused of idled.			*/

#define H1MED_SET8_INT_COND(cp,rmask)    cp->medusa_int_cond    |= rmask
#define H1MED_SET8_INT_MASK(cp,rmask)    cp->medusa_int_mask    |= rmask
#define H1MED_SET8_CONTROL(cp,rmask)     cp->medusa_control     |= rmask
#define H1MED_SET8_ADDRESS(cp,rmask)     cp->medusa_address     |= rmask
#define H1MED_SET8_PPOLL_MASK(cp,rmask)  cp->medusa_ppoll_mask  |= rmask
#define H1MED_SET8_PPOLL_SENSE(cp,rmask) cp->medusa_ppoll_sense |= rmask

#define H1MED_CLR8_INT_COND(cp,rmask)    cp->medusa_int_cond    &= ~rmask
#define H1MED_CLR8_INT_MASK(cp,rmask)    cp->medusa_int_mask    &= ~rmask
#define H1MED_CLR8_CONTROL(cp,rmask)     cp->medusa_control     &= ~rmask
#define H1MED_CLR8_ADDRESS(cp,rmask)     cp->medusa_address     &= ~rmask
#define H1MED_CLR8_PPOLL_MASK(cp,rmask)  cp->medusa_ppoll_mask  &= ~rmask
#define H1MED_CLR8_PPOLL_SENSE(cp,rmask) cp->medusa_ppoll_sense &= ~rmask

#define H1MED_SET10_INT_COND(cp,rmask)  \
           h1med_wr_int_cond(cp, (rmask | h1med_rd10_int_cond(cp)))
#define H1MED_SET10_INT_MASK(cp,rmask)  \
           h1med_wr_int_mask(cp, (rmask | h1med_rd10_int_mask(cp)))
#define H1MED_SET10_CONTROL(cp,rmask)   \
           h1med_wr_control(cp,  (rmask | h1med_rd10_control(cp)))
#define H1MED_SET10_ADDRESS(cp,rmask)   \
           h1med_wr_address(cp,  (rmask | h1med_rd10_address(cp)))

#define H1MED_CLR10_INT_COND(cp,rmask)     \
           h1med_wr_int_cond(cp, ((~rmask) & h1med_rd10_int_cond(cp)))
#define H1MED_CLR10_INT_MASK(cp,rmask)     \
           h1med_wr_int_mask(cp, ((~rmask) & h1med_rd10_int_mask(cp)))
#define H1MED_CLR10_CONTROL(cp,rmask)      \
           h1med_wr_control(cp,  ((~rmask) & h1med_rd10_control(cp)))
#define H1MED_CLR10_ADDRESS(cp,rmask)      \
           h1med_wr_address(cp,  ((~rmask) & h1med_rd10_address(cp)))


#define H1RELEASE_TRAN(pda,tptr)			\
	    tptr->link = pda->bus_trans.free_list;	\
	    pda->bus_trans.free_list = tptr

/* END OF FILE */
