/*
 * @(#)iocam.h: $Revision: 1.26.106.1 $ $Date: 97/07/07 18:22:43 $
 * $Locker: CRT $
 * 
 */
 

/******************************************				     */
/**					  *				     */
/**   CIO Control Function Declarations	  *				     */
/**					  *				     */
/******************************************				     */

#define CIO_AEK_COMMAND	       1	
#define CIO_DA_SELFTEST	       2	
#define CIO_DA_STATUS	       3	
#define CIO_DA_ASYNC_ENABLE    4	
#define CIO_DA_ASYNC_DISABLE   5	
#define CIO_SET_DA_BLOCKSIZE   6	
#define CIO_DA_PARITY_ENABLE   7	
#define CIO_DA_LOGCH_ENABLE    8	
#define CIO_DA_LOGCH_DISABLE   9	
#define CIO_GET_DIRECT_IO_PTR  10
#define CIO_GET_DA_TYPE        11

 
/****************************************       			     */
/**					*				     */
/**   CIO Direct I/O Space Structure    *				     */
/**					*				     */
/****************************************       			     */
typedef struct  cio_quadrant  {
	volatile int	rsvd1[3];  
	volatile int	normal;
	volatile int	rsvd2[3];  
	volatile int 	cend;
	volatile int	rsvd3[3];  
	volatile int	cbyte;
	volatile int	rsvd4[3];  
	volatile int	cendbyte;
};

typedef struct	ciodio {
	struct	cio_quadrant	data;
	struct	cio_quadrant	ctlsens;
	struct	cio_quadrant	order;
	struct	cio_quadrant	statcmd;
} ciodio;


/************************************************/
/*						*/
/* CIO Level 1 Control Operators                */
/*						*/
/************************************************/

#define CIO_DA_CTRL_ARD		0x01
#define CIO_DA_CTRL_ARE		0x02
#define CIO_DA_CTRL_RQA		0x04
#define CIO_DA_CTRL_NMK		0x08
#define CIO_DA_CTRL_DEN		0x10
#define CIO_DA_CTRL_DCL		0x20
#define CIO_DA_CTRL_PEN		0x40



/************************************************/
/*						*/
/* CIO Level 1 Sense Decoding                   */
/*						*/
/************************************************/

#define CIO_DA_SENSE_ARQ	0x01
#define CIO_DA_SENSE_ARE	0x04
#define CIO_DA_SENSE_LV1	0x08
#define CIO_DA_SENSE_NMI	0x10
#define CIO_DA_SENSE_PRE	0x20
#define CIO_DA_SENSE_PST	0x40
#define CIO_DA_SENSE_RFC	0x80
 

/*********************************************/
/* CIO SUBC commands                         */
/*********************************************/

#define	CIO_SUBC_CMD_DSC	0x00
#define	CIO_SUBC_CMD_AEK	0x10
#define	CIO_SUBC_CMD_CSC	0x20
#define CIO_SUBC_CMD_RSC	0x40
#define CIO_SUBC_CMD_ETA	0x50


/*********************************************/
/* CIO SUBC status decoding                  */
/*********************************************/

#define CIO_SUBC_STATUS_RFC	0x00
#define CIO_SUBC_STATUS_AES	0x10
#define CIO_SUBC_STATUS_DPE	0x20
#define CIO_SUBC_STATUS_SCD	0x30
#define CIO_SUBC_STATUS_SCR	0x50
#define CIO_SUBC_STATUS_DOD	0x60
#define CIO_SUBC_STATUS_PER	0x70
#define CIO_SUBC_STATUS_ERT	0x80


/*********************************************/
/* CIO RTS control                           */
/*********************************************/

#define CIO_RTS_CTRL_RLC	0x01
#define CIO_RTS_CTRL_EOD	0x02
#define CIO_RTS_CTRL_DLC	0x03
#define CIO_RTS_CTRL_AEK	0x04


/*********************************************/
/* CIO RTS status decoding                   */
/*********************************************/

#define CIO_RTS_STATUS_IDL	0x00
#define CIO_RTS_STATUS_SWI	0x01
#define CIO_RTS_STATUS_EOD	0x02
#define CIO_RTS_STATUS_LCD	0x03
#define CIO_RTS_STATUS_AES	0x04
#define CIO_RTS_STATUS_ERT	0x05


/****************************************				      */
/**					*				      */
/**   CIO Event Constant Declarations	*				      */
/**					*				      */
/****************************************				     */

#define CIO_TRANSPARENT_STATUS 0  
#define CIO_ARQ_STATUS	       1  

 
/********************************************************		     */
/**							*		     */
/**	 CIO Virtual Quad Command Word Definition	*		     */
/**							*		     */
/********************************************************		     */

typedef union { 
    int cio_cmd; 
    struct { 
	bit8     order; 
	unsigned suppression:2;
	unsigned logch_break:1;
	unsigned blocked:1;
	unsigned reserved:17;
	unsigned read_write:1; 
	unsigned continue_dma:1; 
	unsigned exact:1; 
    } u; 
} cio_cmd_type; 


/*********************************************/
/*                                           */
/* CIO CA DMA command words                  */
/*                                           */
/*********************************************/

#define	CIOCA_IDY	0x08000000
#define	CIOCA_DLD	0x10000004
#define	CIOCA_DLD1	0x11000004
#define	CIOCA_PAU	0x20000000
#define	CIOCA_DIS	0x30000000
#define	CIOCA_RS	0x40000000
#define	CIOCA_RS_D	0x41000000
#define	CIOCA_WC	0x50000004
#define	CIOCA_RD	0x60000000
#define	CIOCA_WD	0x70000004
#define	CIOCA_RTS	0x88000000
#define	CIOCA_WTC	0x98000004
#define CIOCA_RDS	0xA0000000
#define CIOCA_RDS1	0xA1000000
#define	CIOCA_CLC	0xB8000004
#define CIOCA_LNKST     0x00000008

#define BYTE_MODE	0x08000000
#define CIOCA_S1S2	0x00800000
#define LOGCH_BREAK	0x00200000
#define BLOCKED 	0x00100000
#define CONTINUE	0x00000002
 

/*******************************************				     */
/**					   *				     */
/**	 CIO Virtual Quad Definition	   *				     */
/**					   *				     */
/*******************************************				     */

typedef struct cio_vquad_type { 
    cio_cmd_type	  command; 
    int			  count; 
    int			  residue; 
    struct cio_vquad_type *link; 
    data_ptr_type	  buffer; 
    addr_class_type	  addr_class; 
} cio_vquad_type; 

typedef cio_vquad_type *cio_vquad_ptr; 
 

/*******************************************				     */
/**					   *				     */
/**	 CIO Physical Quad Definition	   *				     */
/**					   *				     */
/*******************************************				     */

typedef struct cio_pquad_type { 
    struct cio_pquad_type *link; 
    cio_cmd_type	  command; 
    int			  *address;
    int			  count; 
} cio_pquad_type; 

typedef cio_pquad_type *cio_pquad_ptr; 

#define CIO_PQUAD_EOC (cio_pquad_ptr)0xffffffff


/***************************************				     */
/**				       *				     */
/**  Link status block definitions     *				     */
/**				       *				     */
/***************************************				     */

typedef struct cio_lnkst_type {
    int			  *link;
    int			  status;
    int 		  *lastlink;
    int			  count;
} cio_lnkst_type;

typedef struct cio_comp_list_head_type {
    int			  semaphore;
    int			  *link;
    int			  filler1;
    int			  filler2;
} cio_comp_list_head_type;


/***************************************				     */
/**				       *				     */
/**	 CIO Message Definitions       *				     */
/**				       *				     */
/***************************************				     */

typedef struct { 
    io_subq_type  reply_subq; 
    bit8	  da_number; 
    cio_vquad_ptr vquad_chain; 
} cio_dma_req_type; 

typedef struct { 
    io_subq_type  reply_subq; 
    bit8	  da_number; 
    cio_pquad_ptr firstpquad; 
    cio_pquad_ptr lastpquad; 
} cio_pquad_dma_req_type; 

/*
 * Note that the following struct is the format of the reply message for
 * both vquad dma requests (cio_dma_req_type) and pquad dma requests 
 * (cio_pquad_dma_req_type).  Since it is used for 2 different requests,
 * the 2nd field has a different meaning depending on which request
 * generated the reply.  Namely, if the request is a vquad dma request, then
 * vquad_chain is a pointer to the original vquad_chain (just as one would
 * expect).  If instead the request is a pquad dma request, then vquad_chain
 * is actually a pointer to the firstpquad.  Therefore, in the pquad case,
 * the name of the 2nd field "vquad_chain" is a misnomer.
 */
typedef struct { 
    llio_status_type llio_status; 
    cio_vquad_ptr    vquad_chain; 
} cio_dma_reply_type; 

typedef struct { 
    cio_vquad_ptr vquad_chain; 
    port_num_type cam_port;
    bit8	  da_number; 
} cio_dump_req_type;

typedef struct { 
    io_subq_type reply_subq; 
    bit8	 da_number; 
    bit8	 ctrl_func; 
    int		 ctrl_parm; 
} cio_ctrl_req_type; 

typedef struct { 
    llio_status_type llio_status; 
    int 	     ctrl_info; 
    int 	     extra_info; 
} cio_ctrl_reply_type; 

typedef struct { 
    int event; 
    union { 
	int  int_info[(3)+1]; 
	bit8 byte_info[(15)+1]; 
    } u; 
} cio_io_event_type; 


typedef struct cio_poll_chain {
    struct cio_poll_chain *link;
    cio_pquad_ptr	  pquad_chain;
    llio_status_type	  llio_status;
    bit8		  da_number;
} cio_poll_chain;


typedef struct {
    cio_poll_chain *link;
} cio_poll_dma_req;

typedef struct {
    cio_poll_chain *link;
} cio_poll_sync_reply;
 
 
/**************************************					     */
/**				      *					     */
/**	 CIO Order Declarations	      *					     */
/**				      *					     */
/**************************************					     */

#define CIO_IDY_ORDER	      0x08
#define CIO_DLD_ORDER	      0x10
#define CIO_DLD1_ORDER	      0x11
#define CIO_PAU_ORDER	      0x20
#define CIO_DIS_ORDER	      0x30
#define CIO_RS_WORD_ORDER     0x40
#define CIO_RS_WORD_DIS_ORDER 0x41
#define CIO_RS_BYTE_ORDER     0x48
#define CIO_RS_BYTE_DIS_ORDER 0x49
#define CIO_WC_WORD_ORDER     0x50
#define CIO_WC_BYTE_ORDER     0x58
#define CIO_RD_WORD_ORDER     0x60
#define CIO_RD_BYTE_ORDER     0x68
#define CIO_WD_WORD_ORDER     0x70
#define CIO_WD_BYTE_ORDER     0x78
#define CIO_RTS_ORDER	      0x88
#define CIO_WTC_ORDER	      0x98
#define CIO_RDS_ORDER	      0xA0
#define CIO_RDS1_ORDER	      0xA1
#define CIO_CLC_ORDER	      0xB8

/*************************************************************/
/**                                                         **/
/**   CIO Channel Adapter Manager Error Code Declarations   **/
/**                                                         **/
/*************************************************************/

#define CIO_MEMORY_SEND_FAILED 		  -1
#define CIO_NO_FRAME_AVAILABLE 		  -2
#define CIO_DO_BIND_REPLY_FAILURE 	  -3
#define CIO_ILLEGAL_META_FOR_BINDING 	  -4
#define CIO_DA_ADDRESS_ILLEGAL 		  -5
#define CIO_DAM_IS_ALREADY_BOUND 	  -6
#define CIO_BIND_REPLY_FAILURE 		  -7
#define CIO_DAM_IS_NOT_BOUND 		  -8
#define CIO_DAM_HAS_PENDING_REQUEST 	  -9
#define CIO_UNBIND_REPLY_FAILURE       	 -10
#define CIO_ILLEGAL_MEMORY_REPLY 	  11	/* Warning */
#define CIO_MEM_SEND_AND_RF_FAILED     	 -12
#define CIO_MEMORY_REQUEST_FAILED      	 -13

/*  Base for Resource Warning Numbers */

#define CIO_NO_DATA_RESOURCES 		 -14
#define CIO_NO_LOGCHANNEL_RESOURCES 	 -14
#define CIO_NO_QUAD_ENTRY_RESOURCES 	 -15
#define CIO_NO_STATUS_BLOCK_RESOURCES 	 -16
#define CIO_NO_REQUEST_BLOCK_RESOURCES 	 -17
#define CIO_TABLE_INCONSISTENT 		 -18
#define CIO_REQUEST_LIST_BAD 		 -19
#define CIO_INVALID_ADAPTER_TYPE 	 -20
#define CIO_INVALID_ADAPTER_MODEL 	 -21
#define CIO_INVALID_ADAPTER_REVISION 	 -22
#define CIO_CHANNEL_NOT_READY 	 	 -23
#define CIO_CHANNEL_MSTAT_ERROR 	 -24
#define CIO_BUFFER_NOT_IN_MEMORY 	 -25
#define CIO_BUFFER_NOT_ALIGNED 		 -26
#define CIO_CLC_BUFFER_ERROR 		 -27
#define CIO_ILLEGAL_CONTINUE 		 -28
#define CIO_ILLEGAL_TERMINATION 	 -29
#define CIO_ILLEGAL_DISCONNECT_ORDER 	 -30
#define CIO_ILLEGAL_CLC_ORDER 		 -31
#define CIO_SUBCH_NOT_READY	 	 -32
#define CIO_SUBCH_PAUSED 	 	 -33
#define CIO_SUBCH_ERROR_TRAP	 	 -34
#define CIO_DEV_ADAPTER_NOT_READY 	 -35
#define CIO_NON_ZERO_RESIDUE 		  36	/* Warning */
#define CIO_ILLEGAL_ADDR_CLASS	 	 -37
#define CIO_UNSTARTED_ABORT 		 -38
#define CIO_ABORTED_WHILE_ACTIVE 	 -39
#define CIO_ABORTED_REPLY_FAILURE	 -40
#define CIO_NO_REQUEST_FOUND_TO_ABORT	 -41
#define CIO_SPONTANEOUS_ABORT		 -42
#define CIO_LOGCH_ERROR_TRAP	 	 -43
#define CIO_SPURIOUS_INTERRUPT	 	  44	/* Warning */
#define CIO_ILLEGAL_REQ_NUM_IN_TIMER	 -45
#define CIO_RCVD_SPURIOUS_TIMER_MSG 	  46	/* Warning */
#define CIO_CONTROL_FUNCTION_ILLEGAL 	 -47
#define CIO_DA_SELFTEST_SEQ_ILLEGAL 	 -48
#define CIO_DMA_WAIT_SEQ_ILLEGAL 	 -49
#define CIO_NO_REPLY_TO_ILLEGAL_MSG 	 -50
#define CIO_ALREADY_LOCKED		 -51
#define CIO_IS_NOT_LOCKED		 -52
#define CIO_WRONG_KEY_LOCK		 -53
#define CIO_UNSUPPORTED_DIAG_OPTION	 -54
#define CIO_ILLEGAL_LOCK_EVENT_OPTION	 -55
#define CIO_RESET_NOT_ALLOWED		 -56
#define CIO_CHANNEL_RESET_IN_PROGRESS	 -57
#define CIO_POWER_RECOVERY_IN_PROGRESS	 -58
#define CIO_RESIDUE_ADDR_INVALID	 -59
#define CIO_COMPLTN_ADDR_INVALID	 -60
#define CIO_MERR_DEPEND_CODE		 -80
#define CIO_MERR_COMMAND_CODE		 -81
#define CIO_MERR_LOOPBACK_CODE		 -84
#define CIO_MERR_DATA_CODE		 -85
#define CIO_MERR_PROTOCOL_CODE		 -86
#define CIO_MERR_ADDRESS_CODE		 -87
#define CIO_NO_MESSAGE_ERROR		-101
#define CIO_ILLEGAL_MESSAGE_ERROR	-102
#define CIO_ILLEGAL_STATE_ERROR		-103
#define CIO_UNKNOWN_MESSAGE_ERROR	-104
#define CIO_UNKNOWN_ACTION_ERROR	-105
#define CIO_CHANNEL_BROKEN_ERROR	-106
#define CIO_LVL1_CARD			 107

/***********************************************************/
/*                                                         */
/*     CIO Channel Adapter Procedure Number Definition     */
/*                                                         */
/***********************************************************/

#define	CIO_DO_BIND_PROC           16
#define	CIO_BIND_PROC              17

#define	CIO_TIMER_PROC             33
#define	CIO_ABORT_PROC             34

#define	CIO_INIT_CA_PROC           65

#define	CIO_BUILD_SUBCH_PROC       98
#define	CIO_BUILD_LOGCH_PROC       99
#define	CIO_IO_DMA_PROC           100

#define	CIO_START_SUBCH_PROC      101
#define	CIO_START_LOGCH_PROC      102

#define	CIO_IO_CMPLT_PROC         128

#define	CIO_AEK_DA_PROC           160

#define	CIO_BUILD_SEND_PROC       192
