/*
 * @(#) iodam.h: $Revision: 1.18.106.1 $ $Date: 97/07/07 18:22:45 $
 */

 
/*=============================== ddammeta ===============================*/
 

/* definition of the HPIB hardware address				      */

#define C_MIN_HPIB_ADDR	 0  /* Min hpib device address.			      */
#define C_MAX_HPIB_ADDR	 31 /* Max hpib device address.			      */

/* definition of the type of the HPIB DAM I/O requests			      */

#define CS80_EXECUTE	 0  /* Command Set 80 I/O request.		      */
#define HPIB_EXECUTE	 1  /* Command HPIB trnasfer.			      */
#define MT7970_EXECUTE	 2  /* 7970 magnetic tape transfer.		      */
#define DA_REQUEST	 3  /* DA control request.			      */
#define DAP_SEND	 4  /* Loading a device adapter program.	      */
#define DAP_EXECUTE	 5  /* Execute a device adapter program.	      */
#define SET_BLOCKING	 6  /* Enable the block mode transfer size.	      */
#define CLEAR_BLOCKING	 7  /* Disable the block mode transfer size.	      */

			    /* definition of the data transfer direction      */

#define NO_DATA		 ((bit8) 0)  /* No data transfer.                     */
#define READ_DATA	 ((bit8) 1)  /* Read data transfer.                   */
#define WRITE_DATA	 ((bit8) 2)  /* Write data transfer.                  */

			    /* definition of the CRC functions		      */

#define CRC_NONE	 0  /* No CRC check is done and CRC is disabled.      */
#define CRC_CHECK	 1  /* CRC is enabled for the read		      */
#define CRC_WRITE	 2  /* CRC is enabled for the write data.	      */

/* definition of the types of the device adapter events			      */

#define DEVICE_PPOLL	 0  /* Device parallel polls.			      */
#define DAM_UNLOCK_READY 1  /* DAM ready after unlock.			      */
#define DEAD_OR_DYING	 2  /* Device adapter is sick.			      */
#define DA_INTERRUPT	 3  /* Miscellaneous interrupt			      */
#define DIAG_INTERRUPT	 4  /* Diagnostic event interrupt		      */


/* The following defines the types for all the requests			      */

typedef bit16 block_size_type; /* Size of the block mode transfer.	      */

/* This record is used for executing a downloaded DAP.			      */

typedef struct { 
#ifdef BROKEN_COMPILER
    bit8 require_break; 
#else
    unsigned int  require_break:1; 
#endif
} dap_execute_type; 

/* This record is used for CS-80 requests.				      */

typedef struct { 
    shortint	  timeout_value; 
    bit8	  crc; 
#ifdef BROKEN_COMPILER
    bit8 transparent; 
#else
    unsigned int  transparent:1; 
#endif
} cs80_req_type; 

/* This type is used for "raw" HP-IB transfer operations.		      */

typedef struct { 
    bit8 termination_conds; 
    bit8 matched_byte; 
    bit8 crc; 
} hpib_req_type; 

/* This type is used for the mag tape non_concurrent operations.	      */

typedef struct { 
    bit8	  termination_conds; 
    bit8	  motion_cmd; 
#ifdef	BROKEN_COMPILER
    bit8	require_break;
#else
    unsigned int  require_break:1; 
#endif
    bit8	  crc; 
} mt7970_req_type; 

typedef struct { 
    data_ptr_type   cmd_ptr;	  /* Address of command buffer.		      */
    data_ptr_type   data_ptr;	  /* Address of data to transfer.	      */
    data_ptr_type   status_ptr;	  /* Address of status buffer.		      */
    int		    cmd_count;	  /* Number of command bytes.		      */
    int		    data_count;	  /* Number of bytes to transfer.	      */
    int		    status_count; /* Number of status bytes returned.	      */
    addr_class_type cmd_class;	  /* Address class for cmd_ptr.		      */
    addr_class_type data_class;	  /* Address class for data_ptr.	      */
    addr_class_type status_class; /* Address class for status_ptr.	      */
} req_parm_type; 

typedef struct { 
    io_subq_type  reply_subq;	 /* Subqueue to send reply to.		      */
    bit8	  device_number; /* Address of device for transfer.	      */
    bit8	  transfer_dir;	 /* Which way does the data go ?	      */
#ifdef BROKEN_COMPILER
    bit8 block_mode;	 /* No blocking if set to false.	      */
#else
    unsigned int  block_mode:1;	 /* No blocking if set to false.	      */
#endif
    req_parm_type request_parms; /* Pointers for the I/O request.	      */

    bit8	  request; 
    union { 
	cs80_req_type	 cs80_req; 
	hpib_req_type	 hpib_req; 
	mt7970_req_type	 mt7970_req; 
	bit8		 da_req; 

	dap_execute_type dap_execute_req; 
	block_size_type	 block_size; 

    } u; 
} dam_req_type; 

typedef struct { 
    llio_status_type reply_status;   /* DAM's result from request.	      */
    block_size_type  block_size;     /* current block size in DA.	      */
    req_parm_type    request_parms;  /* original request information.	      */

    int		     cmd_residue;    /* # cmd bytes NOT sent.		      */
    int		     data_residue;   /* # bytes NOT transferred.	      */
    int		     status_residue; /* # status bytes NOT sent.	      */
} dam_reply_type; 

typedef struct { 
    bit8 device_number; /* Hardware address of event.			      */
    bit8 event_info;	/* Type of the async event.			      */
    int	 event_detail;  /* Miscellaneous interrupt cause.		      */
} dam_event_type; 
 
/*=============================== dsdafcod ===============================*/
 

/***begin_table	       HPIB Device Adapter FCODES	 **begin_table**      */
/***end_table************************************************end_table**      */

#define FCODE_RESERVED	      0	 /* Obvious				      */

#define DA_SET_REN	      1	 /* Set remote enable			      */
#define DA_CLEAR_REN	      2	 /* Clear remote enable			      */
#define DA_SEND_IFC	      3	 /* Send interface clear		      */
#define CONFIG_RESPONSE_PPOLL 4	 /* Configure card's response to PPOLL	      */
#define READ_PPOLL_CONFIG     5	 /* Read card's PPOLL response config	      */

#define CONFIG_SRQ_RESPONSE   6	 /* Configure card's SRQ response	      */
#define DA_READ_INTERRUPT     7	 /* Read reason for interrupt		      */
#define DA_WRITE_INT_MASK     8	 /* Write interrupt mask		      */
#define DA_READ_INT_MASK      9	 /* Read interrupt mask			      */
#define DA_WAIT_INTERRUPT     10 /* Wait for interrupt conditions	      */

#define DA_GO_ONLINE	      11 /* Go online				      */
#define DA_GO_OFFLINE	      12 /* Go offline				      */
#define DA_ASSERT_ATN	      13 /* Assert ATN				      */
#define DA_DE_ASSERT_ATN      14 /* De_assert ATN			      */
#define DA_CONF_PPOLL_SENSE   15 /* Configure Bus ppoll sense		      */

#define DA_CONDUCT_PPOLL      16 /* Conduct ppoll and read response	      */
#define CONFIG_HPIB_ADDRESS   17 /* Configure HPIB address		      */
#define DA_READ_STATUS	      18 /* Read DA current status		      */
#define IDENTIFY_CONFIG	      19 /* AMIGO identify configuration	      */
#define LAST_SEC_ADDRESS      20 /* Last secondary address		      */

#define ENABLE_PARITY_FREEZE  21 /* Enable freeze on parity failure	      */
#define DISABLE_PARITY_FREEZE 22 /* Disable freeze on parity failure	      */
#define LOOPBACK	      23 /* Loopback test			      */
#define Z8_ACCESS_DIAG	      24 /* Z8 RAM access diag  ostic		      */
#define EXT_MEM_ACCESS_DIAG   25 /* External memory access diagnostics	      */

#define ENABLE_INT_MASK	      26 /* Enable interrupt mask		      */
#define DISABLE_INT_MASK      27 /* Disable interrupt mask		      */
#define DA_IDY_DEVICE	      28 /* Amigo identify device		      */
#define DA_MAKE_DA_CIC	      29 /* Online_IFC_assert_REN		      */
#define DA_ENABLE_PP_MASK     30 /* Unsolicited PPOLL detect on		      */

#define DA_DISABLE_PP_MASK    31 /* Unsolicited PPOLL detect off	      */
#define DA_CONFIG_SPEED       32 /* Set interface speed			      */
#define DA_CONFIG_SC	      33 /* Set System Controller status	      */
#define DA_DISABLE_INTR	      34 /* Disable interrupts during command byte    */
				 /* transfer -- special for GE/Kinetics	      */

#define FCODE_MAX	      34 /* Last function code of DA		      */

 
/*=============================== dhpiberr ===============================*/
 
/*------------------------------------------------------------------------*/
/*  DHPIBERR.hpibdam.vio                                                  */
/*     Definition of error numbers for a Device Adapter Manager. These    */
/*     error numbers would be put into error_num of llio_status type.     */
/*                                                                        */
/*------------------------------------------------------------------------*/

/*  OK completion with no exceptional conditions */

#define DAM_REQ_OK 	0 /*  Dam request finished successfully          */

/*  OK read completion with exceptional conditions */

#define READ_EOI_EVEN 	     1 /*  Terminated by eoi         - even count */
#define READ_EOI_ODD 	     2 /*  Terminated by eoi         - odd count  */
#define READ_COUNT_EVEN      3 /*  Terminated by count       - even count */
#define READ_COUNT_ODD 	     4 /*  Terminated by count       - odd count  */
#define READ_LF_EVEN 	     5 /*  Terminated by lf          - odd count  */
#define READ_LF_ODD 	     6 /*  Terminated by lf          - even count */
#define READ_MSA_EVEN 	     7 /*  Terminated by msa         - even count */
#define READ_MSA_ODD 	     8 /*  Terminated by msa         - odd count  */
#define READ_CEND            9 /*  Terminated by host                     */
#define DA_NO_DEVICE_IDY    10
#define FIFO_EOI_COUNT_EVEN 11 /*  Terminated by eoi & count - even count */
#define FIFO_EOI_COUNT_ODD  12 /*  Terminated by eoi & count - odd count  */
#define FIFO_EOI_LF_EVEN    13 /*  Terminated by eoi & lf    - even count */
#define FIFO_EOI_LF_ODD     14 /*  Terminated by eoi & lf    - odd count  */
#define FIFO_EOI_MATCH_EVEN 15 /*  Terminated by eoi & match - even count */
#define FIFO_EOI_MATCH_ODD  16 /*  Terminated by eoi & match - odd count  */
#define FIFO_MATCH_EVEN     17 /*  Terminated by char match  - even count */
#define FIFO_MATCH_ODD      18 /*  Terminated by char match  - odd count  */

/* Diagnostic  Warnings */
#define DIAG_SHORT_READ     62 /*  DMA Short Read */


/*   Request errors */

#define DAM_NO_SW_RESOURCES 	-5 /* Dam ran out of some sw resource    */
#define DAM_REQ_PTRS_ERR 	-6 /* Cam detected error in I/O req ptrs */
#define DAM_GET_MSG_FRAME_ERR 	-7 /* Dam cannot get a msg frame         */
#define DAM_GET_IO_QUADS_ERR 	-8 /* Dam has io_get_quads error	 */
#define DM_SEND_ERROR 	       -10 /* Dam cannot send msg to dm          */
#define CAM_SEND_ERROR 	       -20 /* Dam cannot send msg to cam         */
#define CAM_NO_RESOURCES       -21 /* Cam has no resources available     */
#define CAM_IO_ERROR 	       -22 /* Cam has returned a nonzero status  */
#define INVALID_TRON_UID       -23 /* Invalid tron/uid Dam encountered   */
#define DAP_DOWNLOAD_ERR       -26 /* DAP download error                 */
#define DAP_DEV_LOCKED_ERR     -27
#define DAP_SW_IN_USE 	       -28 /* DAP is in use                      */
#define DAP_NOT_PRESENT        -29 /* DAP does not exist                 */

/*  General hardware initialization errors */

#define DA_FCODE_UNIMP 	       -30 /* Unimplemented fcode for the da     */
#define DA_NOT_SC 	       -31 /* DA not sc and should be            */
#define DA_NOT_CIC 	       -32 /* DA not cic and should be           */
#define DA_CIC 		       -33 /* DA cic and should not be           */
#define DA_CIC_OR_NOT_ADDR     -34 /* DA cic or not addressed            */
#define DA_DCL_DETECTED        -35 /* DA detected DCL                    */
#define DA_IFC_DATA_ABORT      -36 /* DA IFC aborted data transfer       */
#define DA_DATA_FROZEN 	       -37 /* Outbound data frozen by inbound    */
#define DA_LOOPBACK_FAILED     -38 /* DA loopback failed                 */
#define DA_PPOLL_INT_ENABLED   -39 /* DA PPOLL intrpt mustn't be enabled */
#define DA_SST_FAILED 	       -40 /* DA selftest failed                 */
#define DA_IDY_FAILED 	       -41 /* DA identify failed                 */
#define DA_NO_TSTAT 	       -43 /* DA did not return any tstat        */
#define DA_DOD_PER  	       -44 /* DA Dead_or_dying/protocol error	 */
#define DA_DAP_ILLEGAL_DAL     -45 /* bad DAL instruction		 */
#define DA_DAP_LENGTH_EXCEEDED -46 /* DAP is too long			 */
#define DA_IOS_EXCEEDED	       -50 /* concurrent xactions exceeded on DA */
#define DA_INVALID_BLOCKSIZE   -51 /* DA has been set to different block */

/* Diagnostic Errors */

#define DIAG_INVALID_REG_OFFSET     -60 /* Register offset out of     */
                                        /* bounds                     */
#define DIAG_BUF_FULL               -61 /* Buffer full when trying    */
                                        /* to read to EOI             */
#define DIAG_MSA_READ               -63 /* Received Secondary         */
                                        /* address during DMA read    */
#define DIAG_NOT_ADDR               -64 /* incorrect addressing for   */
                                        /* DMA transaction            */
#define DIAG_CHECKSUM_ERROR         -65 /* Checksum error in reading  */
                                        /* iodc                       */
#define DIAG_CARD_NOT_PRESENT       -66 /* Card not present when      */
                                        /* reading iodc               */

/* Cs80 errors */

#define CS80_CMD_BUFFER_ERROR  -90 /* Data of cs80 WC buffer missing     */
#define CS80_CMD_TO 	       -91 /* Timeout during command phase       */
#define CS80_CMD_TO_NCIC       -92 /* Ditto plus da no longer cic        */
#define CS80_CMD_TO_NUN        -93 /* Ditto plus da cannot send unt, unl */
#define CS80_EX_TO 	       -94 /* Timeout occurred during execution  */
#define CS80_EX_TO_NCIC        -95 /* Ditto plus da no longer cic        */
#define CS80_EX_TO_NUN 	       -96 /* Ditto plus da cannot send unt, unl */
#define CS80_REP_TO 	       -97 /* Timeout occurred during report     */
#define CS80_REP_TO_NCIC       -98 /* Ditto plus da no longer cic        */
#define CS80_REP_TO_NUN        -99 /* Ditto plus da cannot send unt, unl */
#define CS80_DRP_TO 	      -100 /* Timeout during detailed report     */
#define CS80_DRP_TO_NCIC      -101 /* Ditto plus da no longer cic        */
#define CS80_DRP_TO_NUN       -102 /* Ditto plus da cannot send unt, unl */

/* DAM errors */

#define DAM_SW_ERROR 	      -110 /* Dam has hit some sw error          */
#define DAM_DAM_TABLE_ERR     -111 /* Dam_info table integrity error     */
#define DAM_DM_TABLE_ERR      -112 /* Dm_info table integrity error      */
#define DAM_REQ_TABLE_ERR     -113 /* Req_info table integrity error     */
#define DAM_NO_UIDS_AVAIL     -114 /* Dam no uids available              */
#define RS_BLOCK_UID_ERR      -115 /* Rs_block entry has wrong uid       */
#define RS_BLOCK_VALID_ERR    -116 /* Rs_block entry is in use           */
#define RS_BLOCK_PTR_ERR      -117 /* Rs_block_ptr in req_info_entry bad */
#define REQ_INFO_UID_ERR      -118 /* Req_info entry has wrong uid       */
#define REQ_INFO_VALID_ERR    -119 /* Req_info entry is in use           */
#define DAM_UNKNOWN_TSTAT     -127 /* DA card apparently barfed		 */
