/*
 * @(#)iodiag.h: $Revision: 1.13.106.2 $ $Date: 97/07/12 23:34:24 $
 * $Locker: CRT $
 * 
 */

/*----------------------------------------------------------------------------*/
/* These constants are for the diagnostic option types			      */
/*----------------------------------------------------------------------------*/
#define HW_IDENTIFY    0  /* Manager goes to hardware and reports identity.   */
#define HW_SELFTEST    1  /* Entity performs requested selftest.	      */
#define HW_LOOPBACK    2  /* Entity loopback.	      		              */
#define HW_STATUS      3  /* Entity's last error status or actual status.     */
#define HW_DUMP	       4  /* Dumps Entity's local storage.	              */
#define HW_DESCRIPTION 5  /* Additional hardware descriptive data is reported.*/
#define HW_ROLLCALL    6  /* Manager reports hardware connectivity in layer   */
			  /* away from cpu.			              */
#define HW_UTILITY     7  /* A entity specific utility will be started.	      */
#define MGR_CONTROL    8  /* Manager options will be set.		      */

#define MGR_SET_STATE  9  /* Manager is informed of state of entity: broken   */
			  /* or fixed	      				      */
#define IO_REDIRECTING 10 /* Requesting LLIO drivers to reassign the io       */
			  /* interrupts away from a processor before it will  */
			  /* be deallocated.                                  */

/*------------------------------------------------------------*/
/*Valid diagnostic options are in the range 0 .. diag_max_tag */
/*Required diagnostic options are in the range 0..diag_min_tag*/
/*------------------------------------------------------------*/
#define DIAG_MAX_TAG   IO_REDIRECTING
#define DIAG_MIN_TAG   HW_STATUS
#define DIAG_BASE_TAG  HW_IDENTIFY

typedef bit8 diag_option_type; 

/*--------------------- HARDWARE IDENTIFY -------------------*/
typedef struct { 
    bit8 hw_identification[32]; 
} hw_identify_format_type; 

typedef struct { 
    int		  return_length; /*num bytes hw_identification defined by mgr */
    data_ptr_type return_buff; 
} hw_identify_info_type; 

/*--------------------- HARDWARE SELFTEST -------------------*/
#define ST_DESTRUCTIVE	   0		/*power-on or destructive selftest    */
#define ST_NON_DESTRUCTIVE 1		/*default non-destructive test	      */
#define ST_SPECIFIC	   2		/*device specific selftest	      */
#define ST_MAX_TAG	   ST_SPECIFIC	

typedef struct { 
    int		  status_length;  /*maximum length of status_buff	      */
    data_ptr_type status_buff; 
    bit8	  selftest_option; 
    union { 
	int hw_test_num; 
    } u; 
} hw_self_test_type; 

/*--------------------- HARDWARE LOOPBACK -------------------*/
#define LOOPBACK_GENERIC   0		      /*perform default loopback      */
#define LOOPBACK_FRONTDOOR 1		      /*perform frontdoor loopback    */
#define LOOPBACK_BACKDOOR  2		      /*perform backdoor loopback     */
#define LOOPBACK_RESERVE1  3		      /*reserved global option	      */
#define LOOPBACK_RESERVE2  4		      /*reserved global option	      */
#define LOOPBACK_RESERVE3  5		      /*reserved global option	      */
#define LOOPBACK_RESERVE4  6		      /*reserved global option	      */
#define LOOPBACK_RESERVE5  7		      /*reserved global option	      */
#define MAX_LOOPBACK_TAG   LOOPBACK_BACKDOOR  

/*----------------------------------------------------------------------      */
/*valid global loopback options are in range 0..7, individual managers	      */
/*may use options in the range 8..255 for specific needs.		      */
/*----------------------------------------------------------------------      */

typedef struct {
    bit8	  loopback_option; 
    int		  loopback_length;   /*length of source loopback data	      */
    data_ptr_type loopback_data_ptr; /*pointer to source data		      */
    data_ptr_type return_data_ptr;   /*pointer to target buffer.	      */
				     /* Max length is in loopback_length      */
} hw_loopback_info_type; 

/*--------------------- HARDWARE STATUS -------------------*/
typedef struct { 
    status_option_type status_option; 
    int		       return_length; 
    data_ptr_type      return_buff; 
} hw_status_info_type; 

/*------------------------ HARDWARE DUMP ----------------------*/
#define HW_DUMP_RAM	0	       /* obtain ram of device		      */
#define HW_DUMP_CONTR	1	       /* obtain control state information    */
#define HW_DUMP_RESER2	2	       /* reserved global option	      */
#define HW_DUMP_RESER3	3	       /* reserved global option	      */
#define HW_DUMP_RESER4	4	       /* reserved global option	      */
#define HW_DUMP_RESER5	5	       /* reserved global option	      */
#define HW_DUMP_RESER6	6	       /* reserved global option	      */
#define HW_DUMP_RESER7	7	       /* reserved global option	      */
#define HW_MAX_DUMP_TAG HW_DUMP_CONTR  

/*-----------------------------------------------------------------	      */
/*valid global ram dump options are in the range 0..7, individual	      */
/*managers may use options in the range 8..255 for specific needs.	      */
/*-----------------------------------------------------------------	      */

typedef struct { 
    bit8	 dump_opt;	/*Manager specific option.		      */
    int		 offset_buff;	/*starting address to transfer		      */
    int		 return_length; /*maximum requested transfer		      */
    data_ptr_type return_buff;	/*must be frozen by requester, I/O size align */
} hw_dump_info_type; 

/*------------------------ HARDWARE DESCRIPTION ----------------------*/
typedef struct { 
    int		 return_length; 
    data_ptr_type return_buff; 
} hw_descrip_info_type; 

/*------------------------ HARDWARE ROLLCALL ----------------------*/
typedef struct { 
    bit8	  hardware_list[64]; 
    unsigned char lower_level_exists:1; 
} hw_rollcall_format; 

typedef struct { 
    int		 return_length; /*maximum length of return_buff		      */
    data_ptr_type return_buff; 
} hw_rollcall_info_type; 

/*------------------------ HARDWARE UTILITY ----------------------*/
typedef struct { 
    shortint	 util_opt;   /*device/manager dependent			      */
    int		 util_len;   /*Length of the buffer in bytes.		      */
    data_ptr_type util_buff; /*Memory area frozen by the caller.	      */
} hw_util_info_type; 

/*------------------------ HARDWARE CONTROL ----------------------*/
#define INFO_DEFAULT_RETRIES -1 /*mgr sets default retry count when this is   */
				/*passed via retry_count		      */

typedef struct { 
    shortint	  retry_count;	 /*number of retries for I/O failure	      */
    unsigned char autodiag_on:1; /*Request auto-diag on failure?	      */
    unsigned char errorlog_on:1; /*Log errors to S800DIAG?		      */
} mgr_control_info_type; 

/*----------------------- HARDWARE SET STATE ---------------------*/
#define MGR_STATE_BROKEN  0		   /*something is malfunctioning      */
#define MGR_STATE_FIXED	  1		   /*resume normal operation	      */
#define MGR_MAX_STATE_TAG MGR_STATE_FIXED  

typedef bit8 mgr_set_state_info_type; 
/*----------------------------------------------------------------	      */
/* legal values in range of 0 .. mgr_max_state_tag			      */
/*----------------------------------------------------------------	      */

/*------------------------ IO REDIRECTING ------------------------*/
typedef struct {  /*IO redirecting request information. Used by               */
		  /*Processor Deallocation to redirecting I/O interrupt       */
		  /*away from the processor that we want to deallocate        */
    int		 spu;    /*Processor index of the deallocating SPU            */
} io_redirecting_info_type; 

typedef struct { /*Diagnostic request information			      */
    io_subq_type     reply_subq;  /*Send reply to this subqueue.	      */
    diag_option_type diag_option; 
    union { 
	hw_identify_info_type	 hw_identify_info; 
	hw_self_test_type	 hw_self_test_info; 
	hw_loopback_info_type	 hw_loopback_info; 
	hw_status_info_type	 hw_status_info; 
	hw_dump_info_type	 hw_dump_info; 
	hw_descrip_info_type	 hw_descrip_info; 
	hw_rollcall_info_type	 hw_rollcall_info; 
	hw_util_info_type	 hw_util_info; 
	mgr_control_info_type	 mgr_control_info; 
	mgr_set_state_info_type  mgr_set_state_info; 
	io_redirecting_info_type io_redirecting_info; 
    } u; 
} diag_req_type; 

typedef struct { /*Diagnostic reply information				      */
    llio_status_type reply_status; 
    int		     transfer_count; /* # bytes transferred to return_buff.   */
    diag_option_type diag_option; 
    union { 
	shortint subsys_idy;         /*Subsystem performing hw_identify       */
    } u; 
} diag_reply_type; 

#define MAX_BUFF_BYTES	 72  
#define MAX_HWLOG_BYTES	 40  

typedef bit8 diag_class_type; /* One of the 3 values below please             */

#define DIAG_HW_EVENT	 ((diag_class_type) 0) /* Constants for DIAG_CLASS */
#define DIAG_SW_EVENT	 ((diag_class_type) 1) 
#define DIAG_OTHER_EVENT ((diag_class_type) 2) /* - Things like device Power Fail, etc */

typedef bit8	 bit8_buff_type[MAX_BUFF_BYTES]; 
typedef char	 char_buff_type[MAX_BUFF_BYTES]; 
typedef int	 int_buff_type [MAX_BUFF_BYTES/sizeof(int)];
typedef shortint si_buff_type  [MAX_BUFF_BYTES/sizeof(shortint)];

/* Format of the variant buffer (buff) in the diagnostics event:
 *
 * The hardware status and mgr specific info are stored in the buffer
 * at the end of this record.  Hardware status must precede the mgr
 * information. (Either may be zero)
 *
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
 *	|						|
 *	|     	Hardware status	(variable size)		|
 *	|						|
 *	|===============================================|
 *	|			|			|
 *	| Mgr subsystem number	|  key to MSI format 	|
 *	|     (short)		|  (short)		|
 *	|			|			|
 *	|===============================================|
 *	|	    |		|	    |		|
 *	| Number of | Extra   	| Extra     |     	|
 *	| Extra     | Address	| Address   |     	|
 *	| Levels of | Level #1  | Level #2  | (etc.)	|
 *	| Addressing|           |           | 		|
 *	| (bit8)    | (bit8)	| (bit8)    |		|
 *	|	    |		|	    |		|
 *	|===============================================|
 *	|						|
 *	|	Manager Specific Information		|
 *	|		(variable size)			|
 *	|						|
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
 */

typedef struct { 
    llio_status_type llio_status;	/* Error Status			      */
    diag_class_type  diag_class;	/* What type of event?		      */

    bit8	     hw_status_len;	/* Len (bytes) of hw status	      */
    bit8	     mgr_info_len;	/* Len (bytes) of mgr info	      */

/* The next four fields are used to specify retry information.		      */
/* Depending upon the value of LOG_ALL_RETRIES, the other three		      */
/* take on various interpretations.  See the discussion in the		      */
/* Diagnostic Events Operation section of the SIOSD for details		      */

    bit8	     retry_count;	/* This is the N'th retry *OR*	      */
					/* - total retries attempted	      */
    unsigned char    log_all_retries:1; /* If TRUE, log EACH retry	      */
    unsigned char    retry_again:1;	/* Will mgr do another retry?	      */
    unsigned char    io_worked:1;	/* Did this retry succeed? *OR*	      */
					/* - did last retry done work?	      */
    unsigned char    run_autodiag:1;	/* Run autodiagnostics?		      */
    unsigned char    more_log_info:1;	/* True means another msg is coming   */
    unsigned char    filler:3;          /* Make things start on byte bounds   */
    shortint         logging_subsys;    /* subsys of logging manager          */

    union { 
	bit8_buff_type bit8_buff; 
	char_buff_type char_buff; 
	int_buff_type  int_buff; 
	si_buff_type   si_buff; 
    } u; 
} diag_event_type; 

typedef struct { 
    llio_std_header_type msg_header; 
    diag_event_type	 diag_event; 
} diag_event_msg_type; 

/* Stuff for HPIB1 diagnostics that has no reason to be in this file... */
#define HPIB_DIAG_REQ       1

#define HPIB_READ_REG       20
#define HPIB_WRITE_REG      21
#define HPIB_INPUT          22
#define HPIB_OUTPUT         23

typedef struct {
    unsigned int               diag_opt;
    unsigned int               hpib_timeout;
    unsigned int               hpib_reg;
    data_ptr_type              diag_data;
    unsigned int               data_length;
    unsigned char              wait_opt;
    unsigned char              eoi_opt;
} diag_req_info_type;

#define DIAG_SHZM           0x0800
#define DIAG_GET            0x0400
#define DIAG_IFC            0x0200
#define DIAG_PARITY         0x0100
#define DIAG_STAT_CHNG      0x0080
#define DIAG_HNDSHK_ABORT   0x0040
#define DIAG_PPOLL          0x0020
#define DIAG_SRQ            0x0010
#define DIAG_OUT_AVL        0x0008
#define DIAG_IN_AVL         0x0004
#define DIAG_FIFO_IDL       0x0002
#define DIAG_DCL            0x0001
