/*
 * @(#)iotree.h: $Revision: 1.16.106.1 $ $Date: 97/07/07 18:22:53 $
 * $Locker: CRT $
 * 
 */

#ifndef _IOTREE_INCLUDED /* allows multiple inclusion */
#define _IOTREE_INCLUDED

#if ! defined(ANALYZE) /* anaylze is not defined */

#ifdef _KERNEL_BUILD
#include "../h/types.h"
#else  /* ! _KERNEL_BUILD */
#include <sys/types.h>
#endif /* _KERNEL_BUILD */

#ifndef NONE
#define NONE -1                     /* no port number */
#endif

typedef int          port_num_type; /* port number type */
typedef unsigned int ent_id_type;   /* entity id type */

#else /* anaylze is defined */

/*
  Definitions for "iotree" are found here

  WARNING: The structure element and bit field ordering and the values of
	   many of the constants in this file are hardcoded into uxgen(1m).
	   Changing any of these requires a corresponding change in uxgen.
*/

#ifdef _KERNEL_BUILD
#include "../h/types.h"
#include "../h/sem_beta.h"
#else  /* ! _KERNEL_BUILD */
#include <sys/types.h>
#include <sys/sem_beta.h>
#endif /* _KERNEL_BUILD */

#define MAX_IO_PATH_ELEMENTS 15	    /* length of hw/module path */
#define MAX_ADD_CHILDREN      8	    /* number of add_children per module */

/* MAX_ID, NONE, and io_name_type are also defined in ioconfig.h */

#ifndef MAX_ID
#define MAX_ID 16
#endif

#ifndef NONE
#define NONE -1
#endif

#ifndef _IO_NAME_TYPE
#define _IO_NAME_TYPE
typedef char io_name_type[MAX_ID];
#endif

typedef char	*io_name_ptr;
typedef int	port_num_type;

/* AUTOBAHN: keep these only until we nuke the old code using them */
#define MAX_TREE_SIZE 10

#define PATH_SHORT		-1
#define PATH_LONG		-2
#define NO_SUCH_MGR		-3
#define NOT_LDM			-4
#define DUP_PATH		-5
#define PATH_CONFLICT		-6
#define NO_SPARE		-7
#define NO_SUCH_MODULE		-8
#define PATH_NOT_CONFIGURED	-9

#define VMUNIX_OPEN		-1
#define KMEM_OPEN		-2
#define KMEM_READ		-3
#define IOHEADER_NLIST		-4
#define IOTREE_NLIST		-5
#define MGR_TABLE_NLIST		-6
#define IOTREE_NAMES_NLIST	-7
#define VERSION_NLIST		-8
#define VERSION_WRONG		-9
#define NO_MEMORY		-10

/* AUTOBAHN: do we need to keep these for old iotree? */
/*
  defines for state
*/
#define IOTREE_SPARE    0
#define IOTREE_ASSIGNED 1
#define IOTREE_BOUND    2

#if	defined(_KERNEL) || defined(ANALYZE)

typedef struct io_NAME_entry {
	io_name_type		name;
	struct io_NAME_entry	*next;
} io_name_entry;

#endif /* _KERNEL || ANALYZE */

/* AUTOBAHN: keep for backward compatibility for 8.0 only */
#define iotree_name_type	io_name_type
typedef iotree_name_type	iotree_names_type[];

struct ioheader_type {
	/* AUTOBAHN: keep for backward compatibility for 8.0 only */
	int          version;
	unsigned int num_mgr_table_recs;
	unsigned int num_iotree_recs;
	unsigned int num_iotree_names;
	/* new for autoconfig */
	unsigned int num_io_class_recs;
	unsigned int num_io_mgr_recs;
	unsigned int num_io_sw_mod_recs;
	unsigned int num_user_sw_mod_recs;
	unsigned int num_io_hw_mod_recs;
	unsigned int num_io_tree_recs;
	unsigned int num_kern_dev_recs;
};

/* AUTOBAHN: keep for backward compatibility for 8.0 only */
struct mgr_table_type {
	int	driver;		/* replaced with name in io_mgr_entry */
	int	(*entry)();	/* same in io_mgr_entry */
	int	(*attach)();	/* same in io_mgr_entry */
	int	(*init)();	/* same in io_mgr_entry */
	int	ldm;		/* options.ldm in io_mgr_entry */
	int	b_major;	/* same in io_mgr_entry */
	int	c_major;	/* same in io_mgr_entry */
	int	state;		/* options.bound in io_mgr_entry */
};

/* AUTOBAHN: keep for backward compatibility for 8.0 only */
struct iotree_type {
	int parent;	/* replaced with parent ptr in io_mgr_entry */
	int sibling;	/* replaced with sibling ptr in io_mgr_entry */
	int child;	/* replaced with chile ptr in io_mgr_entry */
	int state;	/* replaced with state.bound_state in io_mgr_entry */
	int parm;	/* replaced with hdw_address in io_mgr_entry */
	int lu;		/* split into lu & mgr_index in io_mgr_entry */
	int port;	/* same in io_mgr_entry */
	int mgr;	/* replaced with conn->manager in io_mgr_entry */
	int module;	/* replaced with conn->module in io_mgr_entry */
};

/* The remainder of this file is new for autoconfig */

typedef unsigned int ent_id_type;	/* an entity id */

typedef struct {
	unsigned int pseudo:1,	/* pseudo driver vs. real driver */
		ldm:1,		/* is this an LDM? */
		dynamic_ds:1,	/* has dynamic data structures */
		probe:1,	/* supports probing of hardware */
		reconfig:1,	/* supports unbind and die */
		nonexistent:1,	/* not really present in system */
		bound:1,	/* at least one instance bound */
		shared_port:1,	/* shares a port number amongst instances */
		mp:1,           /* is this driver MP-safe? */
		R:23;           /* reserved bits */
} mgr_opt_type;

typedef struct {
	unsigned int overlay:1,	/* overlays another manager */
		add_child:1,	/* automatically adds a child */
		nonexistent:1,	/* not really present in system */
		R:29;		/* reserved bits */
} mod_opt_type;

typedef struct {
	unsigned int nonexistent:1, /* not really present in system */
		R:31;		    /* reserved bits */
} class_opt_type;

typedef struct {
	unsigned int shared_parents:1,	/* parent list is shared */
		shared_children:1,	/* children list is shared */
		nonexistent:1,		/* not really present in system */
		R:29;			/* reserved bits */
} conn_opt_type;

typedef struct {
	unsigned int from_gen:1,	/* from uxgen io{} statement */
		from_probe:1,		/* from hardware probe */
		from_file:1,		/* from /etc/ioconfig */
		forced:1,		/* forced into kernel by ioscan */
		cannot_unbind:1,	/* tried to unbind but couldn't */
		native_hdw:1,		/* native hardware module */
		bound_state:4,		/* is node bound or why not */
		ent_id_state:4,		/* validity of entity id */
		R:18;			/* reserved bits */
} node_state_type;

/* defines for bound_state */

/* BS_NEVER_ACCESSED must be 0 so uxgen can ignore the bound_state field */
#define BS_NEVER_ACCESSED	0	/* no open or probe has been done */
#define BS_BOUND		1	/* the node is bound */
#define BS_UNBOUND		2	/* the node has been unbound */
#define BS_TOO_MANY		3	/* mgr returned LLIO_TOO_MANY */
#define BS_NO_RESOURCES		4	/* could not allocate space */
#define BS_MGR_IN_WAY		5	/* other mgr couldn't unbind */
#define BS_NO_MODULE		6	/* no software module matches */
#define BS_BIND_FAILED		7	/* nonspecific bind failure */

/* defines for ent_id_state */

/* ES_NEVER_PROBED must be 0 so uxgen can ignore the ent_id_state field */
#define ES_NEVER_PROBED		0	/* node has never been probed */
#define ES_VALID		1	/* the ent_id is valid */
#define ES_NO_HARDWARE		2	/* no hardware connected */
#define ES_UNRECOGNIZED		3	/* hardware present, but no ent_id */
#define ES_PARENT_WONT_PROBE	4	/* parent manager doesn't probe */
#define ES_PROBE_FAILED		5	/* nonspecific probe failure */

#if	defined(_KERNEL) || defined(ANALYZE)

typedef struct io_CLASS_entry {
	io_name_ptr		name;
	struct io_TREE_entry	*class_link;
	class_opt_type		options;
	struct io_CLASS_entry	*next;
} io_class_entry;

typedef struct io_MGR_entry {
	io_name_ptr		name;
	int			(*entry)();
	int			(*attach)();
	int			(*init)();
	mgr_opt_type		options;
	int			b_major;
	int			c_major;
	struct io_TREE_entry	*mi_link;
	struct io_MGR_entry	*next;
} io_mgr_entry;

typedef struct io_MOD_entry {
	io_name_ptr		name;
	io_class_entry		*class;
	mod_opt_type		options;
	int			(*idy)();
	struct io_MOD_entry	*next;
} io_mod_entry;

typedef struct io_CONN_link {
	struct io_CONN_entry	*entry;
	struct io_CONN_link	*next;
} io_conn_link;

typedef struct io_CONN_entry {
	io_conn_link		*parents;
	io_conn_link		*children;
	io_mgr_entry		*manager;
	io_mod_entry		*module;
	conn_opt_type		options;
	struct io_CONN_entry	*next;
} io_conn_entry;

typedef struct {
	io_mgr_entry	*manager;
	io_class_entry	*class;
} io_major_link;

typedef struct io_TREE_entry {
	struct io_TREE_entry	*parent;
	struct io_TREE_entry	*sibling;
	struct io_TREE_entry	*child;
	io_conn_entry		*conn;
	ent_id_type		ent_id;
	int			mgr_index;
	int			lu;
	port_num_type		port;
	node_state_type		state;
	int			hdw_address;
	struct io_TREE_entry	*mi_link;
	struct io_TREE_entry	*class_link;
} io_tree_entry;

/* Various shorthand casts */

#define NULL_INT_FUNC_PTR	(0L)
#define NULL_IO_CLASS_PTR	((io_class_entry *)0L)
#define NULL_IO_MOD_PTR		((io_mod_entry *)0L)
#define NULL_IO_MGR_PTR		((io_mgr_entry *)0L)
#define NULL_IO_NAMES_PTR	((io_name_entry *)0L)
#define NULL_IO_TREE_PTR	((io_tree_entry *)0L)
#define NULL_IO_CONN_PTR	((io_conn_entry *)0L)
#define NULL_IO_CONN_LINK	((io_conn_link *)0L)

#endif /* _KERNEL || ANALYZE */

#define NULL_IO_NAME_PTR	((io_name_ptr)0L)

typedef struct {
	unsigned int gen_nonexistent:1,	/* uxgen did not specify device */
		gen_defaulted:1,	/* uxgen specified device "default" */
		gen_pseudo:1,		/* uxgen specified pseudo device */
		boot_defaulted:1,	/* hpuxboot forced device to default */
		hdw_path_overridden:1,	/* hpuxboot overrode hardware path */
		mod_path_overridden:1,	/* hpuxboot overrode module path */
		gen_lvol:1,		/* uxgen specified LVM */
		boot_nodevice:1,	/* hpuxboot forced no device */
		sectiondefaulted:1,	/* user did not specify section */
		logical_volume:1,	/* boot disk specified logical volume */
		pdisc:1,		/* disk specified pdisc device */
		R:21;			/* reserved bits */
} kern_flag_type;

/* Defines for Kernel Device Table array indices */

#define KDT_CONSOLE	0	/* console device entry */
#define KDT_ROOT	1	/* root device entry */
#define KDT_ROOT_MIRROR	2	/* root mirror device entry */
#define KDT_DUMP	3	/* dump device entry */
#define KDT_SWAP	4	/* primary swap device entry */
#define KDT_SWAP_MIRROR	5	/* primary swap mirror device entry */
				/* KDT_ADDED_SWAP MUST be the last value */
#define KDT_ADDED_DUMP	6	/* first additional dump device entry -
				   others are at 7, 8, etc. */
#define KDT_END_DUMP    36      /* last additional dump entry */
#define KDT_ADDED_SWAP  37      /* first additional swap entry */


#if	defined(_KERNEL) || defined(ANALYZE)

typedef struct {
	io_tree_entry	*node;		/* io_tree pointer to device LDM */
	io_tree_entry	*gen_node;	/* io_tree pointer to device LDM
					   specified in uxgen - differs
					   from node if overridden by
					   hpuxboot */
	kern_flag_type	flags;		/* flags (see above) */
	dev_t		dev_num;	/* current device dev_t */
	dev_t		gen_dev_num;	/* original uxgen dev_t */
} kern_dev_entry;

/*
 * NOTE: With LVM, information in labels on disk is used to identify any
 *	 root, primary swap and dump pseudo-devices (logical_volume or pdisc). 
 *	 For pseudo-devices, all io_tree_entry fields are null. 
 */
/*
 * Set up the given kernel device table entry.
 */
#define KDT_SET(i,n,f,d)	{ \
				  kern_flag_type	flags; \
				  bzero((char *)&flags, sizeof(flags)); \
				  flags.f = 1; \
				  kdt_set(i, n, flags, d); \
				}

#define KDT_SET_DUMP(i,e,d)	kdt_set(i, (io_tree_entry *)e, 0, d)

#endif /* _KERNEL || ANALYZE */

#endif /* ! defined(ANALYZE)) */

#endif /* not _IOTREE_INCLUDED */
