/*
 * @(#)llio.h: $Revision: 1.26.106.2 $ $Date: 97/07/12 23:39:52 $
 * $Locker: CRT $
 */

#ifndef LLIO_H_INCLUDED
#define LLIO_H_INCLUDED

/*============================ llio defines ==================================*/
#ifdef _KERNEL_BUILD
#    include "../sio/iotree.h"
#else  /* ! _KERNEL_BUILD */
#    include <sio/iotree.h>
#endif /* _KERNEL_BUILD */

typedef int            integer;
typedef unsigned int   bit32;
#define MININT         (-1)

#define shortint       short
typedef unsigned short bit16;

typedef unsigned char  bit8;

#define NIL            0L
typedef char          *anyptr;
typedef unsigned int   objectsize;

typedef char           boolean;
#define TRUE           1
#define FALSE          0

typedef unsigned int   subrange_32;

#define KERNEL_DIAG_FROM_PORT     0

typedef	bit8 	       io_subq_type;

/*============================ set defines ===================================*/

typedef unsigned int	set_of_32;
typedef set_of_32	subqueue_set;
typedef set_of_32	mask_type;
#define EMPTY_SET  		     0
#define ELEMENT_OF_32(x)	     ((unsigned int)1<<(31-((x)&31)))
#define SET_CONST_OF_1(X1)	     ELEMENT_OF_32(X1)
#define SET_CONST_OF_2(X1,X2)  	    (ELEMENT_OF_32(X1) | ELEMENT_OF_32(X2))

/*===================== constants for data_ptr_type ==========================*/

/* Addr_class_type supplies the context in which to interpret data_ptr_type   */
/* For any pointer variable P of type data_ptr_type, with a companion class   */
/* variable Class of type addr_class_type:				      */
/*									      */
/*   Class = virtual_buffer  ==>  P points to a single buffer of data.	      */
/*   Class = virtual_blocks  ==>  P points to the first of a list of	      */
/*				  chained virtual_block_type records.	      */
/* WARNING: Do not try to dereference a pointer of data_ptr_type until        */
/* you know the address class to which the pointer belongs!		      */

typedef bit8  addr_class_type;

#define VIRTUAL_BUFFER    ((addr_class_type) 0)
#define VIRTUAL_BLOCKS    ((addr_class_type) 1)
/*efine PAGE_BLOCKS       ((addr_class_type) 2)	Reserved */
#define CONTIGUOUS_REAL   ((addr_class_type) 3)
#define EQUIV_MAPPED      ((addr_class_type) 4)

typedef union {
    char *data_ptr;
    struct {
	space_t data_sid; /* Space id part of data_ptr                        */
	anyptr  data_vba; /* Virtual byte address part of data_ptr            */
    } u;
} data_ptr_type;

typedef struct virtual_block_type {
    struct virtual_block_type *next_ptr;  /* Ptr to next virtual address      */
					  /* descriptor block.                */
    data_ptr_type             buffer_ptr; /* Long ptr to buf in virtual space */
    int                       buffer_len; /* Length of buffer at buffer_ptr   */
} virtual_block_type;

/*============================ subsystem numbers =============================*/

/*     Subsystem numbers 1000-1399 have been reserved for HP-UX          */
/*     Subsystem numbers 1400-1499 are reserved for HP-UX customer use   */
/*     Subsystem numbers 2000-2999 have been reserved for strategic alliances */

#define SUBSYS_NULL		0

/* subsystem numbers 1000-1019 misc. */
#define SUBSYS_IO_SERVICES	1000
#define SUBSYS_ENVIRON		1001	/* environmental subsystem */
#define SUBSYS_LVM		1002	/* LVM subsystem */
#define SUBSYS_SPT		1003	/* SCSI pass through  subsystem */

/* subsystem numbers 1020-1039 LDMs  */
#define SUBSYS_DIAG0		1020
#define SUBSYS_MUX0		1021
#define SUBSYS_MUX1		1022
#define SUBSYS_DISC0		1023
#define SUBSYS_LPR0		1024
#define SUBSYS_TAPE0		1025
#define SUBSYS_INSTR0		1026
#define SUBSYS_GPIO0		1027
#define SUBSYS_UXEC0		1028
#define SUBSYS_DISC1		1029
#define SUBSYS_DISC2		1030
#define SUBSYS_DIAGHPIB1	1031
#define SUBSYS_GPIO1		1032
#define SUBSYS_DISPLAY0		1033
#define SUBSYS_DISC3		1034
#define SUBSYS_AUTOX0		1035
#define SUBSYS_MUX4		1036
#define SUBSYS_LANTTY0		1037

/* subsystem numbers 1040-1059 DMs   */
#define SUBSYS_LPR10		1040
#define SUBSYS_TAPE10		1041
#define SUBSYS_TAPE11		1042
#define SUBSYS_LPR11		1043
#define SUBSYS_TAPE14		1044
#define SUBSYS_DISC30		1045
#define SUBSYS_CENT0		1046  /* cent0 is a DM/DAM monolith */
#define SUBSYS_DISC31		1047
#define SUBSYS_LPR12 		1048
#define SUBSYS_ISDNNET		1050

/* subsystem numbers 1060-1079 DAMs  */
#define SUBSYS_HPIB0		1060
#define SUBSYS_GPIO10		1061
#define SUBSYS_GPIO11		1062
#define SUBSYS_HPIB1		1063
#define SUBSYS_SCSI1		1065
#define SUBSYS_SCSI2		1066
#define SUBSYS_HPFL1		1067
#define SUBSYS_LANMUX0		1068
#define SUBSYS_SCSI3		1069

/* subsystem numbers 1080-1099 NAMs  */
#define SUBSYS_CIO_CA0		1080
#define SUBSYS_GRAPH0		1081
#define SUBSYS_GRAPH2		1082
#define SUBSYS_GRAPH3		1083
#define SUBSYS_ITE		1084
#define SUBSYS_FRAMEBUF		1085
#define SUBSYS_HIL		1086
#define SUBSYS_HILKBD		1087

/*============================ status defines ================================*/

typedef union {
    int is_ok; 			/* Examine 32 bits at once.		      */
    struct {
	sbit8    error_num;     /* < 0 is error; > 0 is warning               */
	sbit8    proc_num;      /* < 0 means local; >= 0 means global         */
	shortint subsystem;     /* Subsystem detecting the error.             */
    } u;
} llio_status_type;

#define LLIO_OK 		     0    /* No error                         */
#define ALL_OK  		     0    /* Uh, no error                     */
#define LLIO_NORMAL                  0    /* Uh, er, no error                 */

					  /* Warnings & Notes                 */
#define LLIO_RETRY_INCURRED          1    /* Successful, but used retries     */
#define LLIO_BEGINNING_OF_MEDIA      2    /* Start of device's media          */
#define LLIO_LOCK_STAT_ALREADY       3    /* Lock of already locked OR        */
					  /* unlock of not locked req'd.      */
#define LLIO_MARGINAL_DATA           4    /* marginal data on disk            */
#define LLIO_BIND_HW_TO              5    /* timeout error during bind        */
#define LLIO_BIND_HW_ERR             6    /* hw error during bind             */
#define LLIO_EOF_FOUND               7    /* hw EOF detected                  */
#define LLIO_END_MEDIA               8    /* end of media detected            */
#define LLIO_NORMAL_STATUS           9    /* normal status, but read          */
					  /* hardware status                  */
#define LLIO_READ_TERM_BY_AEOR       10   /* read terminated by aeor          */
#define LLIO_SETMARK_FOUND           11   /* hw save setmark detected         */
#define LLIO_CANNOT_IDY		         12   /* something probed, but not id'able*/
#define LLIO_HWRELOC_SUCCESS         13   /* Hw relocattion is done by disc31 */
#define LLIO_NO_REDUNDANCY           14   /* Data Redundancy nonfunctional    */
#define LLIO_MOST_POS_STAT           14	  /* Positive Boundary for Status     */

					  /* General Software Errors          */
#define LLIO_ABORTED                 -1   /* Aborted by user; usually expected*/
#define LLIO_POWERFAIL_ABORTED       -2   /* Sys pwrfail caused to abort      */
					  /* or be unstarted.                 */
#define LLIO_NOT_IMPLEMENTED         -3   /* Feature not implemented.         */
#define LLIO_NOT_UNDERSTOOD          -4   /* Message, whole or part, is junk  */
#define LLIO_NO_STATUS_YET           -5   /* "last_avail" status requested    */
					  /* but there is none yet.           */
#define LLIO_REQ_IN_INV_STATE        -6   /* Understood & Implemented but     */
					  /* mgr state is inappropriate       */
					  /* for this msg.                    */
#define LLIO_SW_PROBLEM              -7   /* The ultimate non-specific status */

					  /* Config-type errors               */
#define LLIO_BIND_BAD_SUBSYS         -8   /* For bind or do_bind req.         */
#define LLIO_BIND_BAD_META           -9   /*           ditto                  */
#define LLIO_BIND_BAD_REV_CODE       -10  /*           ditto                  */
#define LLIO_BIND_BAD_HW_ADDR        -11  /*           ditto                  */
#define LLIO_CANNOT_DO_UNBIND        -14  /* LLM rejected do_unbind req       */
#define LLIO_CANNOT_BIND_LOWER       -15  /* Bind to lower manager failed.    */
#define LLIO_STILL_BOUND             -16  /* Bind req'd of already bound.     */
#define LLIO_NOT_BOUND               -17  /* Bind didn't work for other reason*/
#define LLIO_NO_UNBD_ACTIVE_IO       -18  /* LLM cannot unbind with I/Os      */
					  /* outstanding.                     */
#define LLIO_NOT_LM_BOUND            -19  /* Do_unbind, but NO lower mgr!     */
#define LLIO_BAD_LM_PORT_NUM         -20  /* Port num of lower mgr wrong.     */
#define LLIO_TOO_MANY                -21  /* More binds than mgr can handle   */
#define LLIO_BAD_HM_PORT_NUM         -22  /* Incorrect higher mgr port num    */

					  /* Resource Control Errors          */
#define LLIO_CANNOT_RELEASE_RESOURCE -23  /* Some resource wasn't released    */
#define LLIO_CANNOT_RELEASE_MEM	     -24  /* Memory wasn't released	      */
#define LLIO_CANNOT_GET_MEM	     -25  /* Couldn't get memory.	      */
#define LLIO_MEMORY_ERROR	     -26  /* General mem req error.	      */

					  /* Lock/unlock Errors               */
/* Either a LOCK_REQ/UNLOCK_REQ was received by a manager different           */
/* from that which is current locker, OR some request was received by         */
/* a manager other than the current locker.                                   */
#define LLIO_LOCKED_BY_OTHER         -27 
#define LLIO_MUST_BE_LOCKED          -28  /* Req requires mgr to be locked    */
					  /* and it is not locked.            */
/* See also Warning: LLIO_LOCK_STAT_ALREADY                                   */

/* Mostly Hardware Errors                                                     */
#define LLIO_NOT_READY               -29  /* This mgr's device is NOT         */
					  /* ready, responding, online.       */
#define LLIO_HW_EOF                  -30  /* Unexpected end of "file" or      */
					  /* resource.  eg. disk transfer     */
					  /* hit end of volume; tape hit EOT  */
#define LLIO_END_OF_MEDIA            -31  /* Operator attn required.          */
					  /* eg. printer out-of-paper;        */
					  /* tape needs next reel.            */
#define LLIO_NO_WRITE_ACCESS         -32  /* Attempt write to read-only.      */
#define LLIO_DEV_POWER_ON            -33  /* This mgr's device powered on.    */
#define LLIO_DEVICE_RESET            -34  /* Either this mgr's device was     */
					  /* H/W RESET or request aborted by  */
					  /* RESET_REQ.                       */
#define LLIO_DATA_ERROR              -35  /* Parity, CRC or other data-type   */
					  /* error was detected.              */
#define LLIO_UNINIT_MEDIA            -36  /* Unformatted disk, unblocked      */
					  /* tape, unpunched card, etc.       */
#define LLIO_INV_ADDR                -37  /* Addr req of device is invalid    */
#define LLIO_DATA_OVERRUN            -38  /* Data lost due to transfer error  */
#define LLIO_LOWER_MGR_ERROR         -39  /* Some error in lower mgr for      */
					  /* which no other stat avail.       */
#define LLIO_TIMEOUT                 -40  /* Request failed due to timeout    */
#define LLIO_CHANNEL_TIMEOUT         -41  /* CHANNEL TIMEOUT caused failure   */
#define LLIO_PROTOCOL_ERROR          -42  /* Format or order of commands      */
					  /* invalid.  Primarily for H/W      */
					  /* component of msg (eg. CIPER).    */
#define LLIO_INV_REQ_TRANS_LEN       -43  /* Length of transfer invalid       */
					  /* for device.                      */
#define LLIO_HW_PROBLEM              -44  /* This mgr's device had H/W err    */
					  /* for which no other stat available*/
#define LLIO_SUSPECT_DATA            -45  /* Suspect data encountered         */
					  /* during I/O operation             */
#define LLIO_REQ_NOT_STARTED         -46  /* Request no start/completed       */
					  /* due to some error condition      */
#define LLIO_BAD_FUNC_CODE           -47  /* Invalid request code             */
#define LLIO_PARITY_ERROR            -48  /* Data parity error                */
#define LLIO_TRANSMISSION_ERR        -49  /* Transmission error               */
#define LLIO_READ_TIMEOUT            -50  /* Read timeout                     */
#define LLIO_RECEIVER_OVERRUN        -51  /* Receiver overrun                 */
#define LLIO_DEVICE_OFFLINE          -52  /* Device offline                   */
#define LLIO_NO_CTRL_Y_PIN           -53  /* No Ctrl-Y Pin                    */
#define LLIO_READ_TIME_RETURNED      -54  /* Read time returned               */
#define LLIO_READ_TERM_BY_BREAK      -55  /* Break                            */
#define LLIO_INTERNAL_ERROR          -56  /* Invalid internal software error  */
#define LLIO_ERROR_RD_STATUS         -57  /* Error occurred, read status      */
#define LLIO_RETRANSMIT_BLOCK        -58  /* Send data block again            */
#define LLIO_WRONG_ID_CODE           -59  /* ID code not as expected          */
#define LLIO_CLEAR_FAILURE           -60  /* Device could not be cleared      */
#define LLIO_DOWNLOAD_FAIL           -61  /* Could not download firmware      */

					  /* Mirror driver special status     */
#define LLIO_REIMAGE_DONE	     -62  /* state changed REIMAGE to ONLINE  */
#define LLIO_REIMAGE_FAILED	     -63  /* state changed REIMAGE to OFFLINE */
#define LLIO_ONLINE_DISK_FAILED	     -64  /* state changed ONLINE to OFFLINE  */
#define LLIO_NO_MIRRORLOG	     -65  /* no MSTATE ioctl in 60 sec        */

#define LPMC_ERR_NUM                 -66  /* Low priority machine check	      */

					  /* Additional locking status        */
#define LLIO_LOCK_GRANTED_SAFE       -67  /* Lock granted, no pending i/o     */
#define LLIO_LOCK_STAT_ALREADY_SAFE  -68  /* Lock previously granted, no      */
					  /* pending i/o		      */
#define LLIO_LOCK_GRANTED_BUSY       -69  /* Lock granted, other i/o pending  */
#define LLIO_LOCK_STAT_ALREADY_BUSY  -70  /* Lock previously granted, other   */
					  /* i/o pending	              */

#define LLIO_TIMEOUT_ABORT           -71  /* Abort due to timeout             */
#define LLIO_TIMEOUT_ABORT_FAILED    -72  /* Abort due to timeout failed      */

					  /* Probe status		      */
#define LLIO_NOT_PRESENT	     -73  /* no hardware left OR none there   */
#define LLIO_BUSY		     -74  /* mgr too busy to probe now	      */
#define LLIO_TRY_AGAIN		     -75  /* mgr will probe OK next time      */

#define LLIO_OVER_HUMIDITY           -76  /* device detects over humidity     */
					  /* condition			      */
#define LLIO_OVER_TEMPERATURE        -77  /* system detects over temperature  */
 					  /* condition			      */
#define LLIO_MEDIUM_ERROR            -78  /* Unrecoverable error status       */
#define LLIO_BLANK_CHECK             -79  /* Blank or overwrite on WORM       */
#define LLIO_MECH_FAULT              -80  /* Data Storage mech/module error   */
#define LLIO_LVM_SPARE		     -81  /* Data Block Spared by LVM 	      */

#define LLIO_MOST_NEG_STAT           -81  /* Negative Boundary for Status     */

/* Io_find_msg can report the following error_nums in the llio_status */
#define INVALID_SEARCH_KEY           -90
#define MESSAGE_NOT_FOUND            -92
#define UNSUPPORTED_OPTION 	     -93

/*============================ message descriptors ===========================*/

/* The message descriptors are in 5 groups:				      */
/*    descriptors reserved for I/O services,				      */
/*    descriptors for messages common to all managers,			      */
/*    descriptors reserved for future messages common to all managers,	      */
/*    descriptors reserved for user-written drivers (and the like), and	      */
/*    descriptors specific to messages for HP's I/O managers.		      */

#define RESERVED_DO_NOT_USE    0	

#define ABORT_EVENT_MSG	       1

#define CREATION_MSG	       2	

#define DO_BIND_REQ_MSG	       3	
#define DO_BIND_REPLY_MSG      4	

#define BIND_REQ_MSG	       5	
#define BIND_REPLY_MSG	       6	

#define DO_UNBIND_REQ_MSG      7
#define DO_UNBIND_REPLY_MSG    8	

#define UNBIND_REQ_MSG         9	
#define UNBIND_REPLY_MSG      10	

#define DIE_REQ_MSG           11
#define DIE_REPLY_MSG         12

#define INT_COMPLETION_MSG    13
#define INT_DIRECT_MSG        14

#define PROBE_REQ_MSG         15
#define PROBE_REPLY_MSG       16

#define POWER_ON_REQ_MSG      17 
#define POWER_ON_REPLY_MSG    18 

#define TIMER_EVENT_MSG	      19 

#define DIAG_REQ_MSG	      20 
#define DIAG_REPLY_MSG	      21 
#define DIAG_EVENT_MSG	      22 

#define STATUS_REQ_MSG	      23 
#define STATUS_REPLY_MSG      24 

#define RESET_REQ_MSG	      25 
#define RESET_REPLY_MSG	      26 

#define LOCK_REQ_MSG	      27 
#define LOCK_REPLY_MSG	      28 
#define UNLOCK_REQ_MSG	      29 
#define UNLOCK_REPLY_MSG      30 

/* Descriptors reserved for messages common to all managers.		      */

#define RESERVED_HP_1	      31 
#define RESERVED_HP_2	      32 
#define RESERVED_HP_3	      33 
#define RESERVED_HP_4	      34 
#define RESERVED_HP_5	      35 
/* ... */
#define RESERVED_HP_19	      49 

/* Descriptors for use OUTSIDE of HP (for user-written drivers, etc.).	      */

#define START_RESERVED	      50 
#define RESERVED_NON_HP_1     START_RESERVED+0
#define RESERVED_NON_HP_2     START_RESERVED+1
#define RESERVED_NON_HP_3     START_RESERVED+2

#define RESERVED_NON_HP_30    START_RESERVED+29	

/* Descriptors reserved for uses by the LDMs.				      */

#define START_LDM	      80 
#define RESERVED_LDM_1	      START_LDM+0 
#define RESERVED_LDM_2	      START_LDM+1 
#define DH1_START_MSG	      START_LDM+2
#define SWITCHOVER_MSG        START_LDM+3
#define RESERVED_LDM_20	      START_LDM+19 

/* Descriptors specific to messages for HP's I/O managers.  Extend this	      */
/* list in a fashion similar to that already used (groups of 10).	      */

#define START_CIO_MSG	      100 
#define CIO_DMA_IO_REQ_MSG    START_CIO_MSG+0 
#define CIO_DMA_IO_REPLY_MSG  START_CIO_MSG+1 
#define CIO_CTRL_REQ_MSG      START_CIO_MSG+2 
#define CIO_CTRL_REPLY_MSG    START_CIO_MSG+3 
#define CIO_IO_EVENT_MSG      START_CIO_MSG+4 
#define CIO_INTERRUPT_MSG     START_CIO_MSG+5 
#define CIO_POLL_DMA_REQ_MSG  START_CIO_MSG+6 
#define CIO_POLL_ASYNC_MSG    START_CIO_MSG+7 
#define CIO_DMA_DUMP_REQ_MSG  START_CIO_MSG+8 
#define CIO_PQUAD_DMA_REQ_MSG START_CIO_MSG+9 

#define START_HPIB_MSG	      110 
#define HPIB_IO_REQ_MSG	      START_HPIB_MSG+0
#define HPIB_IO_REPLY_MSG     START_HPIB_MSG+1
#define HPIB_IO_EVENT_MSG     START_HPIB_MSG+2
#define HPIB_DUMP_REQ_MSG     START_HPIB_MSG+3

#define START_DM_MSG	      120 
#define DM_IO_REQ_MSG	      START_DM_MSG+0 
#define DM_IO_REPLY_MSG	      START_DM_MSG+1 
#define DM_IO_EVENT_MSG	      START_DM_MSG+2 

#define START_SCSI_MSG        130
#define SCSI_IO_REQ_MSG       START_SCSI_MSG+0
#define SCSI_IO_REPLY_MSG     START_SCSI_MSG+1
#define SCSI_CTRL_REQ_MSG     START_SCSI_MSG+2
#define SCSI_CTRL_REPLY_MSG   START_SCSI_MSG+3
#define SCSI_IO_EVENT_MSG     START_SCSI_MSG+4
#define SCSI_UNFREEZE_MSG     START_SCSI_MSG+5

#define START_FLEX_MSG        140
#define FL_IO_REQ_MSG         START_FLEX_MSG+0
#define FL_IO_REPLY_MSG       START_FLEX_MSG+1
#define FL_IO_EVENT_MSG       START_FLEX_MSG+2

/*========================= configuration messages ===========================*/

typedef  set_of_32             creation_options;

#define  MGR_CODE_RESIDENT     0 
#define  MGR_DATA_RESIDENT     1 
#define  MGR_POOL_RESIDENT     2 
#define  MGR_ICS_EXECUTABLE    3 
#define  MGR_PROC_EXECUTABLE   4 
#define  MGR_PROBE             5 
#define  MGR_DYNAMIC_DS        6 
#define  MGR_RECONFIG          7 

typedef struct {
    creation_options create_options;  /* Residency options for manager        */
				      /* code and message pool, and manager   */
				      /* executable optns.                    */
    int              server_data_len; /* Size in bytes of mgr's port data area*/
    int              max_msg_size;    /* Size in bytes of mgr's largest msg   */
    int              num_msgs;        /* Num msg frames required for manager  */
				      /* in its normal, maximum configuration */
    bit8             num_subqueues;   /* Number of subqs (2<=n<=32) in port   */
    port_num_type    port_num;        /* port number to re-use (optional)     */
} creation_info_type;

typedef struct {
    io_subq_type  reply_subq;    /* Send reply to this subqueue number.       */
    port_num_type mgr_port_num;  /* The manager's port number.                */
    /*
     * Data type of config_addr_1, config_addr_2, and config_addr_3 has been 
     * changed from bit32 to ulong_t to accomodate 64-bit HPA/SPA values.
     */
    ulong_t       config_addr_3; /* EIM					      */
    ulong_t       config_addr_2; /* spa | IODC_SPA                            */
    ulong_t       config_addr_1; /* hpa or device address                     */

    port_num_type lm_port_num;   /* port number of lower manager to bind with */
    ent_id_type   ent_id;	 /* Entity identifier of the device           */
    boolean       ent_id_valid;  /* Is the entity identifier set?             */
    int           mgr_index;	 /* Instance of the manager                   */
} do_bind_req_type;

typedef struct {
    llio_status_type reply_status;
} do_bind_reply_type;

typedef struct {
    io_subq_type reply_subq;	   /* Send reply to this subqueue #.	      */
    io_subq_type hm_event_subq;	   /* Subqueue of the hm to send event msgs to*/
    shortint	 hm_subsys_num;	   /* The hm's (higher manager) subsystem #.  */
    shortint	 hm_meta_lang;	   /* The metalanguage that will be used by hm*/
    int		 hm_rev_code;	   /* The hm's revision code.		      */
    /*
     * Data type of config_addr_1, config_addr_2, and config_addr_3 has been
     * changed from bit32 to ulong_t to accomodate 64-bit HPA/SPA values.
     */
    ulong_t	 hm_config_addr_3;
    ulong_t	 hm_config_addr_2;
    ulong_t	 hm_config_addr_1; /* The hm's hardware address.	      */
} bind_req_type;

typedef struct {
    llio_status_type reply_status;     /* Result of binding.                  */
    int              lm_rev_code;      /* The (lower manager) revision code.  */
    shortint         lm_queue_depth;   /* The number of requests the lm can   */
				       /*   process concurrently.             */
    io_subq_type     lm_low_req_subq;  /* The lm's low priority request       */
				       /*   subqueue number.                  */
    io_subq_type     lm_hi_req_subq;   /* The lm's high priority request      */
				       /*   subqueue number.                  */
    unsigned char    lm_freeze_data:1; /* The lm requires I/O buffers to be   */
				       /*   frozen.                           */
    unsigned char    lm_alignment:1;   /* The lm requires I/O buffers to be   */
				       /*   aligned to a boundary based on    */
				       /*   cache line size (64 bytes).       */
} bind_reply_type;

/* Constants for probe option */
#define PROBE_FIRST	0	       /* Start probing                       */
#define PROBE_NEXT	1	       /* Continue probing                    */
#define PROBE_ADDRESS	2	       /* Probe the specific address          */

#define	MAX_PROBE_LAYERS  6

typedef struct {
    io_subq_type reply_subq;	       /* send reply to this subqueue         */
    bit8         probe_option;	       /* what type of probe                  */
    int          hdw_address;	       /* address to probe                    */
    bit8         num_layers;	       /* additional addressing information   */
    int          hdw_layers[MAX_PROBE_LAYERS]; /* ditto                       */
} probe_req_type;

#define	MAX_DESC_LEN  32

typedef struct {
    llio_status_type 	reply_status;  /* result of probe operation           */
    int                 hdw_address;   /* address of entity                   */
    ent_id_type         ent_id;        /* identification of entity            */
    bit8                num_layers;    /* additional addressing information   */
    int                 hdw_layers[MAX_PROBE_LAYERS]; /* ditto                */
    char                description[MAX_DESC_LEN];    /* description string   */
} probe_reply_type;

typedef struct {
    io_subq_type 	reply_subq;     /* send reply to this subq num        */
    port_num_type	hm_port_num;	/* manager to unbind from             */
} unbind_req_type;

typedef struct {
    llio_status_type 	reply_status; 	/* results of unbinding 	      */
} unbind_reply_type;

typedef struct {
    io_subq_type 	reply_subq;     /* send reply to this subq num        */
    port_num_type	lm_port_num;	/* port number to unbind from	      */
} do_unbind_req_type;

typedef struct {
    llio_status_type 	reply_status;   /* results of doing unbind            */
} do_unbind_reply_type;

typedef struct {
    io_subq_type 	reply_subq;     /* send reply to this subq num        */
} die_req_type;

typedef struct {
    llio_status_type 	reply_status;   /* results of dying                   */
} die_reply_type;

/*========================= defines for io_get_mem() =========================*/

#define  COMPL_LISTHD_TYPE	1
#define  COMPL_ENTRY_TYPE	2
#define  QUAD_TYPE		3
#define  EQUIV_MEM_TYPE		4
#define  NONEQUIV_MEM_TYPE	5

typedef struct {
	bit16		num_bytes;
	bit8		alignment;
	bit8		type_object;
} mem_type;

typedef bit8 status_option_type;

#define ACTUAL	   0 /* Must get status from hardware.			      */
#define LAST_AVAIL 1 /* Return last status read.			      */

typedef struct {
    io_subq_type reply_subq; /* Send reply to this subqueue #.		      */
} status_req_type;

/* These types describe the information returned in a status reply msg.	      */

typedef set_of_32 generic_hw_set;

#define GEN_ONLINE         0  /* This indicates that the hardware is commun-  */
			      /* icating with the system.  It does not nec-   */
			      /* essarily mean it is in a state to perform    */
			      /* an operation.                                */
#define GEN_READY          1  /* The hardware is ready to perform an opera-   */
			      /* tion.                                        */
#define GEN_DEVICE_BUSY    2  /* The device hardware is currently busy pro-   */
			      /* cessing a request.                           */
#define GEN_TIMEOUT        3  /* The hardware device timed-out.               */
#define GEN_CMD_REJECT     4  /* The device rejected a command because it     */
			      /* was busy or it was an invalid command.       */
#define GEN_DEVICE_END     5  /* This indicates that the device has encoun-   */
			      /* tered a hardware EOF condition.              */
#define GEN_MEDIA_END      6  /* The hardware has consumed all its media.     */
#define GEN_MEDIA_START    7  /* The device's media is positioned at the      */
			      /* start or beginning.                          */
#define GEN_FILE_MARK      8  /* A file mark has been encountered on the      */
			      /* device.                                      */
#define GEN_IRREC_DATA_ERR 9  /* The device encountered an irrecoverable      */
			      /* data, timing, parity, or CRC error.          */
#define GEN_REC_DATA_ERR   10 /* The device encountered an error which was    */
			      /* recovered, via retry, CRC, or other means.   */
#define GEN_HW_FAILURE     11 /* The device hardware has encountered an       */
			      /* error which is not data related.             */
#define GEN_POWERFAIL      12 /* The device encountered a powerfail.          */
#define GEN_SPECIAL_END    13 /* The previous request terminated with a       */
			      /* "special" condition.                         */
#define GEN_WRITE_ENABLED  14 /* The device is enabled to allow "write" op-   */
			      /* erations to be performed.                    */

typedef set_of_32 generic_sw_set;

#define GEN_REQ_PENDING 0 /* The manager issued a request to another          */
			  /* manager and is awaiting a reply.                 */
#define GEN_POWERON     1 /* The manager is recovering from powerfail.        */
#define GEN_LOCKED      2 /* The manager is locked.                           */
#define GEN_RETRYING    3 /* The manager is working on a retry.               */
#define GEN_RECOVERY    4 /* The manager is attempting hardware error         */
			  /* recovery.                                        */
#define GEN_HW_SICK     5 /* The manager has encountered errors in the        */
			  /* hardware's operation.                            */
#define GEN_HW_DEAD     6 /* The manager has determined that its hard-        */
			  /* ware is not at all functional.                   */
#define GEN_RESOURCES   7 /* The manager attempted an operation but           */
			  /* could not obtain a required resource to          */
			  /* complete the operation.                          */

typedef struct {
    llio_status_type reply_status;
    generic_hw_set   hw_status;
    generic_sw_set   sw_status;
} status_reply_type;


typedef struct {
    io_subq_type reply_subq; /* Send reply to this subqueue #.		      */
} reset_req_type;

typedef struct {
    llio_status_type reply_status;
} reset_reply_type;


#define LOCK_EVENT_IGNORE 0 /* Manager ignores (sinks) events.                */
#define LOCK_EVENT_NORMAL 1 /* Manager processes events normally.             */
#define LOCK_EVENT_COPY   2 /* Manager processes events normally,             */
			    /* but in addition it forwards a copy             */
			    /* of any event message it sends to               */
			    /* port & subqueue from the lock rqst             */
#define LOCK_EVENT_DIVERT 3 /* Send event only to the event port              */
			    /* and subqueue specified in lock                 */
			    /* request, not to higher manager.                */

typedef struct {
    io_subq_type reply_subq; /* Send reply to this subqueue #.		      */
    bit8	 lock_event;
    struct {
	    io_subq_type  event_subq; /* On this subqueue & port              */
	    port_num_type event_port; /* send events                          */
    } u;
} lock_req_type;

typedef struct {
    llio_status_type reply_status;
    io_subq_type     request_subq; /* Subqueue # locker is to use for         */
				   /* sending requests to the manager.        */
} lock_reply_type;


typedef struct {
    io_subq_type reply_subq; /* Send reply to this subqueue #.		      */
} unlock_req_type;

typedef struct {
    llio_status_type reply_status;
} unlock_reply_type;

typedef struct {
    anyptr		comp_entry;    /* pointer to completion list entry    */
} int_completion_type;

typedef struct {
    anyptr		hpa;           /* hpa of interrupting module          */
} int_direct_type;

typedef struct {
    shortint      msg_descriptor;  /* Describes the format of the message-    */
				   /* specific text that follows this header. */
    shortint      message_id;      /* Number generated by the sender of the   */
				   /*   message.  Request-Reply pairs must    */
				   /*   use the same number here.             */
    int           transaction_num; /* The number of the logical transaction   */
				   /*   to which the message belongs.         */
    port_num_type from_port;       /* Port number of message's sender.        */
#ifdef __LP64__
    int           filler;          /* Pads header struct to 64-bit boundary.  */
#endif /* __LP64__ */
} llio_std_header_type;

/* Meta language definitions.						      */
/* Meta language numbers 100-199 have been reserved for strategic alliances  */
#define CIO_INTERRUPT_TAG  1 /* Hardware interrupt handler.		     */
#define CIO_META_TAG	   2 /* CIO channel adapter.			     */
#define HPIB_META_TAG	   3 /* CIO HP-IB device adapter.		     */
#define DM_META_TAG	   4 /* All device manager's.			     */
#define MUX_DAM_META_TAG   5 /* Terminal MUX.				     */
#define ALINK_DAM_META_TAG 6 /* ALINK device manager.			     */
#define PSI_DAM_META_TAG   7 /* Programmable Serial Interface.		     */
#define GPIO_META_TAG	   8 /* General Purpose I/O.			     */
#define GRAPH_META_TAG	   9 /* Graph02 port server.			     */
#define SCSI_META_TAG	  10 /* SCSI device adapter.			     */
#define DISPLAY0_META_TAG 11 /* Display0 port server.			     */
#define ITE_META_TAG	  12 /* ITE port server.			     */
#define HIL_META_TAG	  13 /* HIL port server.			     */
#define FRAMEBUF_META_TAG 14 /* Framebuf port server.			     */
#define HILKBD_META_TAG	  15 /* HILKBD port server.			     */
#define FL_META_TAG	  16 /* HPFL device adapter.			     */
#define LAN_META_TAG	  17 /* LAN CONSOLE device adapter.		     */
#define LASI_META_TAG	  18 /* Device under Lasi (LAN only for rel 10.0)    */


#define HPFL_META_TAG FL_META_TAG

#ifdef _KERNEL_BUILD
#    include "../sio/iocam.h"
#    include "../sio/iodam.h"
#    include "../sio/iodm.h"
#    include "../sio/iodiag.h"
#    include "../sio/scsi_meta.h"
#else  /* ! _KERNEL_BUILD */
#    include <sio/iocam.h>
#    include <sio/iodam.h>
#    include <sio/iodm.h>
#    include <sio/iodiag.h>
#    include <sio/scsi_meta.h>
#endif /* _KERNEL_BUILD */


/*============================ system port defines ===========================*/

#define  IO_CONFIGURATOR_PORT   0
#define  IO_DIAGNOSTICS_PORT    1

#define  PFAIL_SUBQ	      0
#define  IO_CONFIG_SUBQ       1
#define  IO_DIAGNOSTICS_SUBQ  1
#define  INT_SUBQ             3


/*============================ io_find_msg defines ===========================*/

#define  DESCR_SEARCH   0
#define  MSGID_SEARCH   1
#define  TRN_SEARCH     2
#define  MSGID_AND_TRN  3
#define  ANY_MSG        4

#define  SEARCH_ALL_SUBQS    1
#define  SEARCH_ENABLED_ONLY 2
#define  SEARCH_AND_DEQUEUE  3

typedef struct {
	shortint        msgid;
	int             trn;
} search_value_type;

/*============================ io_message_type ===============================*/

typedef struct {
    llio_std_header_type msg_header;
    union {
	creation_info_type      creation_info;  /* creation_msg         */
	do_bind_req_type        do_bind_req;    /* do_bind_req_msg      */
	do_bind_reply_type      do_bind_reply;  /* do_bind_reply_msg    */
	bind_req_type           bind_req;       /* bind_req_msg         */
	bind_reply_type         bind_reply;     /* bind_reply_msg       */
	probe_req_type          probe_req;      /* probe_req_msg        */
	probe_reply_type        probe_reply;    /* probe_reply_msg      */
	unbind_req_type         unbind_req;     /* unbind_req_msg       */
	unbind_reply_type       unbind_reply;   /* unbind_reply_msg     */
	do_unbind_req_type      do_unbind_req;  /* do_unbind_req_msg    */
	do_unbind_reply_type    do_unbind_reply;/* do_unbind_reply_msg  */
	die_req_type            die_req;        /* die_req_msg          */
	die_reply_type          die_reply;      /* die_reply_msg        */
	reset_req_type          reset_req;      /* reset_req_msg        */
	reset_reply_type        reset_reply;    /* reset_reply_msg      */
	diag_req_type           diag_req;       /* diag_req_msg         */
	diag_reply_type         diag_reply;     /* diag_reply_msg       */
	diag_event_type         diag_event;     /* diag_event_msg       */
	status_req_type         status_req;     /* status_req_msg       */
	status_reply_type       status_reply;   /* status_reply_msg     */
	lock_req_type           lock_req;       /* lock_req_msg         */
	lock_reply_type         lock_reply;     /* lock_reply_msg       */
	unlock_req_type         unlock_req;     /* unlock_req_msg       */
	unlock_reply_type       unlock_reply;   /* unlock_reply_msg     */
	cio_dma_req_type        cio_dma_req;    /* cio_dma_io_req_msg   */
	cio_pquad_dma_req_type  cio_pquad_dma_req;
	cio_dma_reply_type      cio_dma_reply;  /* cio_dma_io_reply_msg */
	cio_ctrl_req_type       cio_ctrl_req;   /* cio_ctrl_req_msg     */
	cio_ctrl_reply_type     cio_ctrl_reply; /* cio_ctrl_reply_msg   */
	cio_io_event_type       cio_io_event;   /* cio_io_event_msg     */
	dam_req_type            dam_req;        /* hpib_io_req_msg      */
	dam_reply_type          dam_reply;      /* hpib_io_reply_msg    */
	dam_event_type          dam_event;      /* hpib_io_event_msg    */
	scsi_req_type           scsi_req;       /* scsi_io_req_msg      */
	scsi_reply_type         scsi_reply;     /* scsi_io_reply_msg    */
	dm_io_req_type          dm_io_req;
	dm_io_reply_type        dm_io_reply;
	dm_io_event_type        dm_io_event;
	int_direct_type		int_direct;
	int_completion_type	int_completion;
    } union_name;
} io_message_type;

/*=========================== interrupt defines ==============================*/
#define  EIM_TO_EIR(eim)  (eim&0x1f)

#define MAX_CACHE_LINE	64

struct compl_entry {
    uint32_t		link;
    uint32_t		status;
    uint32_t		save_link;
    uint32_t		save_count;
    port_num_type	port;
    uint32_t		filler[3];
};

struct compl_head {
    uint32_t		sema;
    uint32_t		link;
    uint32_t		filler[2];
};

/*==================== structure for io_port_info() ==========================*/

typedef struct {
    port_num_type   parent;
    port_num_type   sibling;
    port_num_type   child;
    int             hdw_address;
    int             mgr_index;
    int             lu;
    int             device_instance;
    int             b_major;
    int             c_major;
    char            *manager;
    char            *module;
    char            *class;
} io_port_info_type;

/* NEW and Improve I/O port info query interface
** See ../sioserv/sio_util.c for what goes into this.
*/
typedef struct {
    port_num_type   parent;
    port_num_type   sibling;
    port_num_type   child;
    int             hdw_address;
    int             drv_flags;
    int             b_major;
    int             c_major;
    char            *drv_name;
    char            *mod_name;
    char            *class;
    void            *reserved[6];
} sio_port_info_t;

/*=============== I/O Services functions declarations ====================*/
#ifdef _KERNEL

extern void io_send __((
	port_num_type port_num, io_subq_type subq, anyptr msg,
	size_t msg_length));
extern void io_find_msg __((
	port_num_type search_port, io_subq_type search_subq,
	short search_key, anyptr search_value, set_of_32 search_options,
	anyptr *msg_ptr, llio_status_type *status));
extern void io_get_frame __((
	port_num_type port_num, anyptr *msg_ptr));
extern void io_release_frame __((
	port_num_type port_num, anyptr msg_ptr));
extern void io_get_timer __((
	int microsecs, port_num_type reply_port, io_subq_type reply_subqueue,
	int transaction_num, short message_id, int *id));
extern void io_start_timer __((
	int microsecs, int id));
extern int io_stop_timer __((
	int id));
extern int io_reset_timer __((
	int microsecs, int id));
extern int io_release_timer __((
	int id));
extern void *io_get_mem __((
	mem_type mem));
extern void io_rel_mem __((
	void *address, mem_type mem));
extern void io_get_trn __((
	int *trn));
extern void io_rel_trn __((
	int trn));
extern port_num_type io_sys_port_id __((
	int sys_port));
extern int io_port_info __((
	port_num_type port_num, io_port_info_type *info));
extern int sio_port_info __((
	port_num_type port_num, sio_port_info_t *info));
extern void init_compl_head __((
	uint32_t eim, port_num_type port));
extern struct compl_head *io_get_compl_head __((
	uint32_t eim));
extern void io_restore_comp_sem __((
	struct compl_head *io_listhead));
extern void io_block_server __((
	port_num_type port_num));
extern void io_unblock_server __((
	port_num_type port_num));
extern int io_em_fe __((
	void *hpa));
extern int io_clear_em_fe __((
	void *hpa, uint32_t num_pages, int trn));

#endif /* _KERNEL */

#ifndef _KERNEL
/*
	These defines are used by the table connect_tab in the file
	llio_idy.c.  The table connect_tab is used by the command
	regen to determine what I/O modules are to be included in
	the S800 file built at install time.
*/

#define LDM_META_TAG          -1
#define TARGET_META_TAG       -2
#define HW_META_TAG           -3
struct connect_tab_entry {
	char *mod_name;		/* Name of the module		*/
	int  (*mod_idy)();	/* The idy routine to call	*/
	int  module_found;	/* The module has been found.	*/
				/* > 0	- module found.		*/
				/*   0	- module not found.	*/
				/*  -1	- end of table 		*/
	int  my_meta;		/* The meta type of this module	*/
	int  child_meta;	/* The meta type of the child	*/
};
#endif /* not _KERNEL*/

#endif /* not LLIO_H_INCLUDED */

