/*
 * @(#)scsi_meta.h: $Revision: 1.6.106.1 $ $Date: 97/07/07 18:26:55 $
 * $Locker: CRT $
 */

#ifndef SCSI_META_H_INCLUDED
#define SCSI_META_H_INCLUDED

/* SCSI I/O Request and Reply           */

typedef struct {
   uintptr_t            scratch1;       /* Scratch area for DAM use           */
   uintptr_t            scratch2;       /* Scratch area for DAM use           */
   io_subq_type         reply_subq;     /* Subqueue to reply on               */
   bit8                 function;       /* Requested function                 */
   bit8                 target;         /* SCSI target ID                     */
   bit8                 id_index;       /* IDENTIFY message byte              */
   bit8                 tag_index;      /* Tag for SCSI queued commands       */
   int                  timer;          /* Timer interval for requests        */
   addr_class_type      cmd_class;      /* Address class for cmd_ptr          */
   addr_class_type      data_class;     /* Address class for data_ptr         */
   addr_class_type      status_class;   /* Address class for status_ptr       */
   data_ptr_type        cmd_ptr;        /* Request buffer                     */
   data_ptr_type        data_ptr;       /* Data buffer                        */
   data_ptr_type        status_ptr;     /* Status buffer                      */
   int                  msg_out_len;    /* Number of message out bytes        */
   int                  cdb_len;        /* Number of command bytes            */
   int                  data_len;       /* Number of data bytes               */
   int                  status_len;     /* Length for status_ptr              */
   int                  sense_len;      /* Number of sense data bytes         */
} scsi_io_req_type;

typedef struct {
   llio_status_type     reply_status;     /* LLIO reply status                */
   llio_status_type     auto_sense_status;/* LLIO reply status                */
   int                  data_residue;     /* Residue for data_ptr buffer      */
   int                  sense_count;      /* # sense bytes in status buffer   */
   int                  msg_in_1_count;   /* # msg bytes(1) in status buffer  */
   int                  msg_in_2_count;   /* # msg bytes(2) in status buffer  */
   bit8                 bad_msg_index;    /* Index into msg-out of bad message*/
} scsi_io_reply_type;

/* SCSI I/O Request Functions           */
#define CMD_DATA_IN     1               /* SCSI Command-Data_In-Status        */
#define CMD_DATA_OUT    2               /* SCSI Command-Data_Out-Status       */
#define CMD_NO_DATA     3               /* SCSI Command-Status                */
#define MSG_OUT         4               /* Send SCSI message                  */


/* SCSI Control Request and Reply       */

typedef struct {
   bit8                 target;         /* SCSI target ID                     */
   bit8                 identify;       /* IDENTIFY message byte              */
   bit8                 queue_depth;    /* Number of requests active on HW    */
   bit8                 auto_unfreeze:1;/* Automatic unfreeze of frozen state */
   bit8                 auto_abort:1;   /* Automatic abort of timed out reqs  */
   bit8                 aen_enable:1;   /* Enable target for AENs             */
   bit8                 sync_enable:1;  /* Enable synchronous transfers       */
   bit8                 reserved:4;     /* Unused/Reserved                    */
} set_option_type;

typedef struct {
   bit8                 target;         /* SCSI target ID                     */
   bit8                 identify;       /* IDENTIFY message byte              */
   bit8                 tag;            /* SCSI queue tag                     */
   bit8                 abort_type;     /* Type of SCSI abort                 */
} scsi_abort_type;

typedef struct {
   bit8                 target;         /* SCSI target ID                     */
   bit8                 identify;       /* IDENTIFY message byte              */
	bit8						transfer_type;	 /* Type of data transfer mode			*/
} transfer_mode_type;

typedef struct {
   bit8                 scsi_id;        /* Adapter's SCSI ID                  */
   bit8                 data_width;     /* Adapter's SCSI data bus width      */
   bit8                 parity:1;       /* SCSI parity checking               */
   bit8                 sync:1;         /* Synchronous transfers supported    */
   bit8                 diff:1;         /* Differential SCSI cabling          */
   bit8                 spifi2:1;       /* SPIFI2 chip; use async if CIO DDS  */
   bit8                 cio:1;          /* TRUE if CIO; use async if CIO DDS  */
   bit8                 reserved:3;     /* Unused/Reserved                    */
} adapter_info_type;

typedef struct {
   uintptr_t            scratch1;       /* Scratch area for DAM use           */
   uintptr_t            scratch2;       /* Scratch area for DAM use           */
   io_subq_type         reply_subq;     /* Subqueue to reply on               */
   bit8                 function;       /* Requested function                 */
   union {
      set_option_type   option;         /* Parameters for SET_OPTIONS         */
      scsi_abort_type   abort;          /* Parameters for SEND_SCSI_ABORT     */
      transfer_mode_type transfer_mode; /* Parameters for SET_TRANSFER_MODE   */
   } u;
} scsi_ctrl_req_type;

typedef struct {
   llio_status_type     reply_status;   /* LLIO reply status                  */
   union {
      adapter_info_type adapter_info;   /* Data for ADAPTER_INFO              */
   } u;
} scsi_ctrl_reply_type;

/* SCSI I/O Control Functions           */
#define ADAPTER_INFO      1             /* Return information on adapter      */
#define SET_OPTIONS       2             /* Set DAM operating options          */
#define SEND_SCSI_ABORT   3             /* Abort I/O request(s)               */
#define RESET_SCSI_BUS    4             /* Assert SCSI RST                    */
#define SET_TRANSFER_MODE 5             /* Set data transfer mode             */

/* abort_type                           */
#define SCSI_ABORT       1              /* Send ABORT message                 */
#define SCSI_ABORT_TAG   2              /* Send ABORT TAG message             */
#define SCSI_CLEAR_QUEUE 3              /* Send CLEAR QUEUE message           */
#define SCSI_BDR         4              /* Send BUS DEVICE RESET message      */

/* transfer_type                        */
#define SCSI_SYNC_TRANS    1            /* SCSI synchronous data transfers    */
#define SCSI_ASYNC_TRANS   2            /* SCSI asynchronous data transfers   */
#define SCSI_NARROW_MASK   3		/* Mask to exclude wide requests      */
#define SCSI_16_WIDE_TRANS 4            /* SCSI 16 bit wide data transfers    */
#define SCSI_32_WIDE_TRANS 8            /* SCSI 32 bit wide data transfers    */
					/* If both wide bits omitted, narrow
					   assumed for backwards compatibility
					   It is undefined to set both wide
					   bits.
					   SCSI_32_WIDE_TRANS is not supported*/


/* SCSI I/O Event                       */
typedef struct {
   char                 *aen_info;      /* Buffer for AEN information         */
   mem_type             mem;            /* Parameters for aen_info buffer     */
} aen_event_type;

typedef struct {
   int                  event;          /* Type of event                      */
   union {
      aen_event_type    aen_event;      /* AEN event information              */
   } u;
} scsi_event_type;

/* SCSI I/O Events                      */
#define REQUEST_TIMEOUT  1              /* Request timer expired              */
#define AEN              2              /* AEN received from device           */
#define UNKNOWN_RESELECT 3              /* Reselect on unknown nexus          */


/* SCSI Unfreeze                        */
typedef struct {
   bit8                 target;         /* SCSI target ID                     */
   bit8                 identify;       /* IDENTIFY message byte              */
} scsi_unfreeze_type;


/* SCSI Errors                          */
#define SCSI_TIMEOUT_ABORT                1
#define SCSI_TIMEOUT_ABORT_FAILED        -1
#define SCSI_ABORT_FAILED                -2
#define SCSI_BUS_ERROR                   -3
#define SCSI_MSG_REJECT                  -4
#define SCSI_NON_GOOD_STATUS              2
#define SCSI_MSG_REJECT_NON_GOOD_STATUS  -5
#define SCSI_PARITY_ERROR		 -6
#define SCSI_EVENT                       -7


/* SCSI Status Bytes                    */
#define GOOD                       0
#define CHECK_CONDITION            2
#define COND_MET_GOOD              4
#define BUSY                       8
#define INT_GOOD                  16
#define INT_COND_MET_GOOD         20
#define RESERVATION_CONFLICT      24
#define COMMAND_TERMINATED        34
#define QUEUE_FULL                40


/* SCSI Sense Keys                      */
#define NO_SENSE                   0
#define RECOVERED_ERROR            1
#define NOT_READY                  2
#define MEDIUM_ERROR               3
#define HARDWARE_ERROR             4
#define ILLEGAL_REQUEST            5
#define UNIT_ATTENTION             6
#define DATA_PROTECT               7
#define BLANK_CHECK                8
#define COPY_ABORTED              10
#define ABORTED_COMMAND           11
#define EQUAL                     12
#define VOLUME_OVERFLOW           13
#define MISCOMPARE                14


/* Popular SCSI Message Bytes				*/
#define ABORT_MSG								0x06
#define ABORT_TAG_MSG						0x0D
#define BUS_DEVICE_RESET_MSG				0x0C
#define CLEAR_QUEUE_MSG						0x0E
#define COMMAND_COMPLETE_MSG				0x00
#define DISCONNECT							0x04
#define IDENTIFY_MSG							0x80
#define IGNORE_WIDE_RESIDUE_MSG			0x23
#define INITIATE_RECOVERY_MSG				0x0F
#define INITIATOR_DETECTED_ERROR_MSG	0x05
#define LINKED_COMMAND_COMPLETE_MSG		0x0A
#define MESSAGE_PARITY_ERROR_MSG			0x09
#define MESSAGE_REJECT_MSG					0x07
#define NO_OPERATION_MSG					0x08
#define HEAD_OF_QUEUE_TAG_MSG				0x21
#define OREDERED_QUEUE_TAG_MSG			0x22
#define SIMPLE_QUEUE_TAG_MSG				0x20
#define RELEASE_RECOVERY_MSG				0x10
#define RESTORE_POINTERS_MSG				0x03
#define SAVE_DATA_POINTERS_MSG			0x02
#define TERMINATE_IO_PROCESS_MSG			0x11
#define EXTENDED_MESSAGE_MSG				0x01

/* Extended Messages 						*/
#define MODIFY_DATA_POINTER_MSG			0x00
#define SDTR_MSG								0x01
#define WIDE_DATA_TRANSFER_REQ_MSG		0x03


/*
 *  Defines and declarations for SCSI Pass-thru diagnostics requests
 */

/* scsi pass-through util_opt      */
#define SCSI_PT           9

typedef struct {
    union {
	scsi_io_req_type	io;
	scsi_ctrl_req_type	ctrl;
    }  u;
}  scsi_req_type;

typedef struct {
    union {
	scsi_io_reply_type	io;
	scsi_ctrl_reply_type	ctrl;
	scsi_event_type		event;
    }  u;
}  scsi_reply_type;

typedef struct {
    shortint		request_msg_type;  /* message type to send to DAM	*/
                                           /* filled in by DAR			*/
    shortint		reply_msg_type;	/* message type returned to caller	*/
                                        /* filled in by DM			*/
    scsi_req_type	req;		/* request to send to DAM		*/
    scsi_reply_type	reply;		/* reply returned to caller		*/
} dm_diag_type;

#endif /* SCSI_META_H_INCLUDED */
