/* $Source: /usr/local/kcs/sys.ROSE_800/h/RCS/ki.h,v $
 * $Revision: 1.18.106.5 $     $Author: bpohl $
 * $State: Exp $        $Locker: CRT $
 * $Date: 97/08/31 13:07:41 $
 */
/* HPUX_ID: @(#)ki.h    54.1            88/11/13 */

#ifndef _SYS_KI_INCLUDED /* allows multiple inclusion */
#define _SYS_KI_INCLUDED

/*
/* To use the structures/types in this header file, one *MUST* be */
/* compiling either 64bit or 32bit enhanced.  This allows for*/
/* the use of 64bit data types.*/
*/

#if !defined(__STDC_32_MODE__)

/*
/* Kernel Instrumentation (aka KI) Definitions:*/
//
/* The file presents definitions mostly trace definitions that are */
/* exposed to:*/
/*    - user-space programs*/
/*    - other kernel components*/
/*    - the KI module.*/
//
//
*/

#ifdef _KERNEL_BUILD         
#  include "../h/vas.h"
#  include "../h/ki_defs.h"   
#else /* ! _KERNEL_BUILD */
#  include <sys/vas.h>
#  include <sys/ki_defs.h>
#endif /* _KERNEL_BUILD */

#define KI_VERSION_64   "11.00 HP-UX LP64"
#define KI_VERSION_32   "11.00 HP-UX ILP32"

#ifdef __LP64__
#define KI_VERSION      KI_VERSION_64
#else 
#define KI_VERSION      KI_VERSION_32
#endif /* !__LP64__ */

/*
/* Kernel Instrumentation NameSpace:*/
*/

typedef enum /* int32_t */ _KI_DEFINITIONS  {  
   /* set max pathname length from user process in trace buffer */
   KI_MAXPATH       = 256,
   /* set min timeout for KI_TIMEOUT_SET at 100 Ms */
   KI_MINTIMEOUT    = (HZ/10),
   KI_MIN_TBSZ      =  16384,   /* MIN size of trace buffer */
   KI_MAX_TBSZ      =  2097152, /* MAX size of trace buffer */
   KI_MAX_LMEM_TBSZ =  4194304, /* MAX size of Large Memory trace buffer */
   KI_MIN_CTSZ      =  4,       /* MIN size of counter buffer */
   KI_MAX_CTSZ      =  2097152, /* MAX size of counter buffer */
   KI_MAX_TRACE_REC =  256,     /* MAX size of USER trace struct */
} KI_DEFINITIONS;

typedef enum /* dev_t */ _KI_DEVICE     {
   /* unique bits to not match with any possible dev_t bits */
   NO_DEV       =   (dev_t)-13,
   DEV_SOCKET   =   (dev_t)-12,     /* communications endpoint */
   DEV_NSIPC    =   (dev_t)-11,     /* CND communication type */
   DEV_UNSP     =   (dev_t)-10,     /* user nsp control */
   DEV_PIPE     =   (dev_t)-9,      /* Pipe (not used by KI) */
   DEV_LLA      =   (dev_t)-8,      /* link-level lan access */
} KI_DEVICE;

/* 
/* Sometimes pid and tid are not valid as on ICS */
*/

typedef enum /* kid_t:int32_t */ _KI_ID   {
   NO_PID = ((int32_t)-1),     /* Process id is not valid */
   NO_TID = ((int32_t)-1),     /* Thread  id is not valid */
} KI_ID;

/*
/* NO_KPTR: invalid KPTR value.*/
*/

#define NO_KPTR   ((kptr_t)(~0L))

/*
/* NO_BP: invalid BP value.*/
*/

#define NO_BP     ((kbuf_t *)(~0L))

/*
/* NO_BPTYPE: invalid BPTYPE value.*/
*/

#define NO_BPTYPE ((bptype_t)-1)

/*
/* NO_SITE: invalid SITE value.*/
*/

#define NO_BSITE   ((ksite_t)-1)

/*
/* NO_RP: invalid region pointer value.*/
*/

#define NO_RP   ((reg_t *)(~0L))

/*
/* NO_FTYPE: invalid FTYPE value.*/
*/

#define NO_FTYPE   ((kftype_t)-1)

/*
/* NO_TYPE: invalid TYPE value.*/
*/

#define NO_TYPE   ((ktype_t)-1)

/*
/* NO_INO: invalid INO value.*/
*/

#define NO_INO   ((ino_t)-1)


/* 
/* The following definitions shorten some typing for structure references.*/
*/

#define kd_recsz        kd_struct.rec_sz
#define kd_recid        kd_struct.rec_id
#define kd_curtime      kd_struct.cur_time
#define kd_seqcnt       kd_struct.seqcnt
#define kd_ktid         kd_struct.kt_tid
#define kd_utid         kd_struct.ut_tid
#define kd_pid          kd_struct.P_pid
#define kd_cpu          kd_struct.P_cpu

/* 
/* Common header for all KI records :*/
*/

struct kd_struct {
   krecsz_t  rec_sz;               /* 2 record length              */
   krecid_t  rec_id;               /* 2 record ident               */
   kseqcnt_t seqcnt;               /* 4 sequence counter for debug */
   kt_t      cur_time;             /* 8 time of record creation    */
   tid_t     kt_tid;               /* 4 kernel thread id           */
   tid_t     ut_tid;               /* 4 user thread id             */
   pid_t     P_pid;                /* 4 process id                 */
   kspu_t    P_cpu;                /* 2 cpu is process is running  */
   short     pad;                  /* 2 filler                     */
/* The sum here should add to mod 8  __ */
/*                                   32 */
};

#if !defined(_KD_T)
#define _KD_T
typedef struct kd_struct kd_t;
#endif /* _KD_T */

/* 
/* KI updated process states */
*/

struct ki_p_state {
   kid_t     p_fsid;                 /*  4 fair share scheduler id    */
   kflag_t   p_flag;                 /*  4 process flags (proc_flag)  */
   kstate_t  p_stat;                 /*  1 process states (proc_stat) */
   knice_t   p_nice;                 /*  1 nice value for cpu usage   */
   uint16_t  p_pad1;                 /*  2 padding                    */
   uint32_t  p_pad2;                 /*  4 padding                    */
                                     /* --                            */
                                     /* 16                            */
};

#if !defined(_KPST_T)
#define _KPST_T
typedef struct ki_p_state kpst_t;
#endif /* _KPST_T */

/* 
/* KI updated thread states */
*/

struct ki_kt_state {
   kflag_t         kt_flag;          /*  4 kthread flags (kthread_flag) */
   kstate_t        kt_stat;          /*  1 kthread state (thread_state) */
   kcpu_t          kt_cpu;           /*  1 cpu usage for scheduling     */
   kpri_t          kt_pri;           /*  2 kthread priority             */
   kschedpolicy_t  kt_schedpolicy;   /*  4 kthread sched. policy        */
   uint32_t  p_pad2;                 /*  4 padding                      */
                                     /* --                              */
                                     /* 16                              */
};

#if !defined(_KTST_T)
#define _KTST_T
typedef struct ki_kt_state ktst_t;
#endif /* _KTST_T */

/* 
/* KI updated process/kthread states  */
//
/* - should replace ki_kt_state and ki_p_state structures when possible*/
*/

struct ki_pk_state {
   kid_t     p_fsid;                 /*  4 process fair share scheduler id   */
   kflag_t   p_flag;                 /*  4 process flags (proc_flag)         */
   kstate_t  p_stat;                 /*  1 process states (proc_stat)        */
   knice_t   p_nice;                 /*  1 process nice value for cpu usage  */
   kstate_t        kt_stat;          /*  1 kthread state (thread_state)      */
   kcpu_t          kt_cpu;           /*  1 kthread cpu usage for scheduling  */
   kflag_t         kt_flag;          /*  4 kthread flags (kthread_flag)      */
   kschedpolicy_t  kt_schedpolicy;   /*  4 kthread sched. policy             */
   kpri_t          kt_pri;           /*  2 kthread priority                  */
   uint16_t        pk_pad;           /*  2 padding                           */
                                     /* --                              */
                                     /* 24                              */
 
};

#if !defined(_KPKST_T)
#define _KPKST_T
typedef struct ki_pk_state kpkst_t;
#endif /* _KPKST_T */


/* 
/* KI-specific iovec structure */
//
/* Note for LP64, ILP32 definition issues:*/
/*    In general, the Kernel should use only ki_iovec.*/
/*    ki_iovec32_t or ki_iovec64_t should be used in user-space.*/
//
*/

struct ki_iovec {
   kptr_t          iov_base;
   uint64_t        iov_len;
};

#if !defined(_KIOV_T)
#define _KIOV_T
typedef struct ki_iovec         kiov_t;
#endif /* _KIOV_T */

#if !defined(_KERNEL)

struct ki_iovec32 {
   uint32_t       iov32_base;
   uint32_t       iov32_len;  
};

#if !defined(_KIOV32_T)
#define _KIOV32_T
typedef struct ki_iovec32      kiov32_t;
#endif /* !_KIOV32_T */

struct ki_iovec64 {
   uint64_t       iov64_base;
   uint64_t       iov64_len;  
};

#if !defined(_KIOV64_T)
#define _KIOV64_T
typedef struct ki_iovec64      kiov64_t;
#endif /* !_KIOV32_T */

#endif /* !_KERNEL */


/* 
/* KI kmem-specific iovec structure */
//
/* Note for LP64, ILP32 definition issues:*/
/*     See above, ki_iovec issues.*/
*/

struct kmemiov {
   /* 
   /* The 1st 2 entries must be equivalent to struct iov */
   */
   kptr_t      buf_addr;
   uint64_t    buf_len;
   kptr_t      kern_addr;
   int         errnum;
   int         kmiov_pad;  /* 64bit padding */
};

#if !defined(_KMIOV_T)
#define _KMIOV_T
typedef struct kmemiov          kmiov_t;
#endif /* _KMIOV_T */

struct kmemiov32 {
   uint32_t   buf_addr;
   uint32_t   buf_len;
   uint32_t   kern_addr;
   int32_t    errnum;
};

#if !defined(_KMIOV32_T)
#define _KMIOV32_T
typedef struct kmemiov32        kmiov32_t;
#endif /* !_KMIOV32_T */

struct ki_region_fields {
   /*
    * Fields needed straight from region
    */
   region_flags_t r_flags; /* 4 :                                       */
   region_type_t r_type;   /* 4 : type of region                        */
   pgcnt_t  r_pgsz;        /* 4 : size in pages                         */
   pgcnt_t r_nvalid;       /* 4 : number of valid pages in region       */
   pgcnt_t r_swalloc;      /* 4 : for RF_SWLAZY,#pgs actually allocated */
   pgcnt_t  r_swapmem;     /* 4 : amount of mem reserved for object swap*/
   pgcnt_t  r_vfd_swapmem; /* 4 : amount of mem reserved for vfd swap   */
   pgcnt_t  r_lockmem;     /* 4 : number of memory locked pages         */
   uint16_t r_refcnt;      /* 2 : number of users pointing at region    */
   int16_t  r_zomb;        /* 2 : set by xinval to indicate text bad    */
   pgcnt_t  r_off;         /* 4 : offset into vnode (page aligned)      */
   kptr_t   r_fstore;      /* 8 : ptr to vnode where blocks come from   */
   kptr_t   r_bstore;      /* 8 : ptr to vnode where blocks go          */
   uint64_t r_byte;        /* 8 : byte off in fstore (support old a.out)*/
   uint64_t r_bytelen;     /* 8 : byte len in fstore (support old a.out)*/
   pgcnt_t  r_poip;        /* 4 : number of page I/Os in progress
                            *
                            *  NOTE: must hold the region lock and the
                            *  sleep_lock to increment the r_poip
                            *  field (start an I/O). Must hold
                            *  the sleep_lock to decrement.
                            */
   pgcnt_t r_pgszhint;     /* 4 : page size hint.                       */
   kptr_t r_next;          /* 8 : links for regions sharing pages/vnode */
   kptr_t r_prev;          /* 8 :                                       */
   union {
      kptr_t r_un_pregskl; /* skiplist of pregions pointing to region   */
      kptr_t r_un_pregion; /* sole pregion pointing to region           */
   } r_preg_un;            /* 8 Total                                   */
   uint16_t r_incore;      /* 2 : number of users pointing at region    */

   uint16_t pad[3];	   /* 6 : pad for alignment. */
};			   /* 112 Total Bytes */

struct kd_p_lle {
        kptr_t lle_next[PSL_NLEVELS];  /* 32 :head/next in pregion skip list*/
       kptr_t lle_prev;                /*  8 : Last pregion in list */
};

struct ki_pregion_fields {
   struct kd_p_lle p_ll;   /* 40: Linked list of pregions in vas         */
   pregion_flags_t p_flags;/* 4 :                                        */
   pregion_type_t p_type;  /* 4 :                                        */
   kptr_t  p_reg;          /* 8 :                                        */
   kptr_t  p_vaddr;        /* 8 : virtual offset for region              */
   space_t p_space;        /* 4 : virtual space for region               */
   pgcnt_t p_off;          /* 4 : offset in region                       */
   pgcnt_t p_count;        /* 4 : number of pages mapped by pregion      */
   pgcnt_t p_ageremain;    /* 4 : remaining number of pages to age       */
   pgcnt_t p_agescan;      /* 4 : indx of nxt scan for vhand's age hand  */
   pgcnt_t p_stealscan;    /* 4 : indx of nxt scan for vhand's steal hand*/
   kptr_t  p_vas;          /* 8 : Pointer to vas we're under             */
   kptr_t  p_regsknode;    /* 8 : assc skiplist node for region list     */
   tid_t   p_tid;          /* 4 : Owner thread id(for uarea pregion)     */
   uint16_t p_nestediomaps;/* 2 : # of nested io_map()'s to this vaddr   */
   uint16_t p_pagein;      /* 2 : Size of pagein for fault handler       */
   int8_t  p_bestnice;     /* 1 : best nice value for all processes sharing
                               * the region used by this pregion.        */
   int8_t  p_deactsleep;   /* 1 : sleep address for self deactivation    */
   uint8_t p_locality;     /* 1 : Locality offset.                       */
   uint8_t p_strength;     /* 1 : Sequential vs random indicator.        */
   pgcnt_t p_nextfault;    /* 4 : Next anticipated fault.                */
   pgcnt_t p_last_rhead;   /* 4 : Location of last read-ahead.           */
   int32_t p_mq_refcnt;    /* 4 : # of references to a MQ (data area)    */
   int16_t p_prot;         /* 2 : protection type (not protid) of region */
   uint16_t pad[3];        /* 6 : compiler pad */
};			   /*136 : Total */





/* 
/*//////////////////////////////////////////////////////////////////////////////*/
//
/* BEGIN the KI SysCall Trace Types */
//
/*//////////////////////////////////////////////////////////////////////////////*/
*/


/*
/* KI SysCall Trace Type:*/
*/

#if !defined(_KSTYPE_T)
#define _KSTYPE_T 
typedef uint16_t kstype_t;
#endif /* _KSTYPE_T */

#define KS_stype        kd_syscall.ks_stype

/*
/* KI SysCall Trace Return Value Type:*/
*/

#if !defined(_KSRVAL_T)
#define _KSRVAL_T
typedef uint64_t ksrval_t;
#endif /* _KSRVAL_T */

#define KD_SYSCALL_GENERIC      0
/* this is the GENERIC system call trace structure */
struct  kd_syscall {
   kd_t       kd_struct;    /* KI trace common header            */
   kt_t       ks_sttime;    /* syscall start time                */
   kstype_t   ks_stype;     /* KI syscall trace type             */
   kboolean_t ks_32bit;     /* syscall is 32bit - true | false   */
   uint8_t    ks_nargs;     /* syscall arguments size (in bytes) */
   uint16_t   ks_syscal;    /* syscall number                    */
   uint16_t   ks_error;     /* uint16_t casted returned u_error  */
   ksrval_t   ks_rval1;     /* syscall return value1             */
   ksrval_t   ks_rval2;     /* syscall return value2             */
   kt_t       ks_sys_time;  /* KT_SYS time - last call or resume */
                            /* --                              */
                            /* 72                              */
/* long       u_arg[N];        syscall arguments                 */
};

#if !defined(_KS_T)
#define _KS_T
typedef struct kd_syscall       ks_t;
#endif /* _KS_T */

/* Currently "f_type"s are DTYPE_VNODE, DTYPE_SOCKET, DTYPE_UNSP, DTYPE_LLA  */

#define KD_SYSCALL_FD           1       
struct  kd_fd {            /* System calls fd = 1st parameter   */
   ks_t      kd_syscall;   /* KI trace common header            */
   dev_t     dev;          /* device major&minor number         */
   ktype_t   v_fstype;     /* file system type                  */
   ktype_t   v_type;       /* vnode type                        */
   kftype_t  f_type;       /* type of f_data                    */
   kptr_t    f_data;       /* vnode, socket pointer etc.        */
                            /* --                              */
                            /* 88                              */
/* long      u_arg[N];        arguments to the system call      */
};

#define F_data f_data.kga_val

#if !defined(_KERNEL)
#   if !defined(__LP64__)
#define F_data64 f_data.kga_val64
#   endif /* __LP64__ */
#endif /* _KERNEL */

#if !defined(_KD_FD_T)
#define _KD_FD_T
typedef struct kd_fd            kd_fd_t;
#endif /* _KD_FD_T */

#define KD_SYSCALL_1STRING      2
struct  kd_1string {                    /* for string in 1st parameter       */
        struct kd_syscall kd_syscall;   /* KI trace common header            */
                            /* --                              */
                            /* 72                              */
/*      long              u_arg[N];        arguments to the system call      */
/*      char              name1[s1];       path name from user process       */
};

#define KD_SYSCALL_2STRING      3
struct  kd_2string {                    /* for string in 1st/2nd parameters  */
        struct kd_syscall kd_syscall;   /* KI trace common header            */
                            /* --                              */
                            /* 72                              */
/*      long              u_arg[N];        arguments to the system call      */
/*      char              name1[s1];       path name from user process       */
/*      char              name2[s2];       path name from user process       */
};

#define KD_SYSCALL_OPEN         4
struct  kd_open {           /* for open/create system call       */
   ks_t       kd_syscall;   /* KI trace common header            */
   dev_t      dev;          /* device major&minor number         */
   ktype_t    v_fstype;     /* file system type                  */
   ktype_t    v_type;       /* vnode type                        */
   kftype_t   f_type;       /* type of f_data                    */
   kptr_t     f_data;       /* vnode, socket pointer etc.        */
                            /* --                              */
                            /* 88                              */
/* long       u_arg[N];        arguments to the system call      */
/* char       name1[s1];       path name from user process       */
};

#if !defined(_KD_OPEN_T)
#define _KD_OPEN_T
typedef struct kd_open kd_open_t;
#endif /* _KD_OPEN_T */

#define KD_SYSCALL_EXEC         5
struct  kd_exec {                       /* for exec system calls             */
        struct kd_syscall kd_syscall;   /* KI trace common header            */
        kptr_t vp;        /* a.out vnode pointer               */
        kptr_t p_vas;     /* virtual address space for process */
                            /* --                              */
                            /* 88                              */
/*      long            u_arg[N];          arguments to the system call      */
/*      char            pname[s1];         exec process argv[0]              */
/*      char            u_comm;            u.u_comm and u.u_ap->fname if     */
};

#define KD_SYSCALL_FORK         6
struct  kd_fork {                       /* for fork and vfork                */
        struct kd_syscall kd_syscall;   /* KI trace common header            */
        kpkst_t ki_pk_state;            /* updated process  & thread states  */
        uint32_t   p_pl_flags;          /* flags accessed w/ proclock held   */
        dev_t             p_ttyd;       /* controlling tty dev               */
        uid_t             p_uid;        /* [tty signals]   proc user  id     */
        uid_t             p_suid;       /* set (effective) proc user  id     */
        pid_t             p_pgrp;       /* name of process group leader      */
        pid_t             p_ppid;       /* process id of parent              */
        kptr_t      p_vas;              /* virtual address space for process */
        uid_t             kd_u_uid;     /*                 user user  id     */
        uid_t             kd_u_ruid;    /* real            user user  id     */
        gid_t             kd_u_gid;     /*                 user group id     */
        gid_t             kd_u_rgid;    /* real            user group id     */
        gid_t             u_sgid;       /* set (effective) user group id     */
/*      long              u_arg[N];        arguments to the system call      */
/*      char            u_comm;            u.u_comm if MS_FORK_EXTRA is on   */
};

#define KD_SYSCALL_FD2          7       
struct  kd_fd2 {          /* System calls fd = 1st parameter   */
   ks_t     kd_syscall;   /* KI trace common header            */
   dev_t    dev1;         /*  4:device major&minor # of f_data1   */
   dev_t    dev2;         /*  4:device major&minor # of f_data2   */
   kptr_t   f_data1;      /*  8: vnode pointer etc.                */
   kptr_t   f_data2;      /*  8: vnode pointer etc.                */
   kftype_t f_type1;      /*  2: type of f_data1                   */
   kftype_t f_type2;      /*  2: type of f_data2                   */
   uint32_t fd2_pad;      /*  4: padding                           */
/* long     u_arg[N];        arguments to the system call      */
};

#define F_data1 f_data1.kga_val
#define F_data2 f_data2.kga_val

#if !defined(_KERNEL)
#   if !defined(__LP64__)
#define F_data1_64 f_data1.kga_val64
#define F_data2_64 f_data2.kga_val64
#   endif /* __LP64__ */
#endif /* _KERNEL */

#if !defined(_KD_FD2_T)
#define _KD_FD2_T
typedef struct kd_fd2           kd_fd2_t;
#endif /* _KD_FD2_T */

#define KD_SYSCALL_MMAP         8
struct  kd_mmap {                       /* for mmap system calls             */
        struct kd_syscall kd_syscall;   /*  72: KI trace common header            */
        kptr_t          r_fstore;       /*   8: front vnode     pointer           */
        kptr_t          r_bstore;       /*   8: back  vnode     pointer           */
        kptr_t          rp;             /*   8: process region  pointer           */
        kptr_t          prp;            /*   8: process pregion pointer           */
        struct ki_region_fields region; /* 112: process region                    */
};				 	/* 216 Total */

#define KD_SYSCALL_MOUNT        9
struct kd_mount {                       /* for mount system calls            */
        struct kd_syscall kd_syscall;   /*  72: KI trace common header            */
/*      long            u_arg[N];          arguments to the system call      */
/*      char            fs[s1];            fs name to be mounted             */
/*      char            path[s1];          Root dir where fs is mounted      */
/*      char            fstype[s1];        type of fs                        */
};

#define KD_SYSCALL_VFSMOUNT     10
struct kd_vfsmount {                    /* for vfsmount system call       */
        struct kd_syscall kd_syscall;   /*  72: KI trace common header            */
/*      long            u_arg[N];          arguments to the system call      */
/*      char            path[s1];          Root dir where fs is mounted      */
};

#define KD_SYSCALL_SWAP         11
struct kd_swap {        /* for swapon system call       */
        struct kd_syscall kd_syscall;   /*  72: KI trace common header            */
        uint32_t        vtype;          /*   4: either VBLK or VDIR               */
        uint32_t        vfstype;        /*   4: fs type                           */
                            /* --                              */
                            /* 80                              */
/* long     u_arg[N];      arguments to the system call       */
/*      char            path[s1];          pathname of swap dev or directory */
};

#define KD_SYSCALL_SCHED   12
struct  kd_sched {        /* for POSIX real time scheduling    */
   ks_t     kd_syscall;   /*  72: KI trace common header       */
   kpkst_t   ki_pk_state; /* updated process & thread states   */
   uint32_t p_pl_flags;   /*   4: irrelevant flags             */
   uint32_t pad;          /*   4: mod 8 padding                */
/* long     u_arg[N];        arguments to the system call      */
};

#if !defined(_KD_SCHED_T)
#define _KD_SCHED_T
typedef struct kd_sched kd_sched_t;
#endif /* !_KD_SCHED_T */


/* 
/*//////////////////////////////////////////////////////////////////////////////*/
//
/* BEGIN the KI Non-System Call Trace Types */
//
/*//////////////////////////////////////////////////////////////////////////////*/
*/


struct  kd_enqueue2 {
   kt_t            b_timeval_lu; /*  8: buffer last used time             */
   kptr_t          bp;           /*  8: buffer ident                      */
   uint32_t        b_flags;      /*  4: too much goes here to describe    */
   int32_t         b_bcount;     /*  4: transfer count                    */
   daddr_t         b_blkno;      /*  4: block # on device                 */
   bptype_t        b_bptype;     /*  2: buffer type                       */
   uint16_t        b2_flags;     /*  2: refer to buf.h          */
   pid_t           b_upid;       /*  4: process id that last used bp       */
   pid_t           b_apid;       /*  4: process id that last allocated bp */
   tid_t           b_uutid;      /*  4: uthread id that last used bp      */
   tid_t           b_autid;      /*  4: uthread id that last allocated bp */
   tid_t           b_uktid;      /*  4: kthread id that last used bp      */
   tid_t           b_aktid;      /*  4: kthread id that last allocated bp */
   int32_t         b_sectno;     /*  4: sect no. */
   int32_t         pad;          /*  4: mod 8 padding                     */
   kptr_t          b_vp;         /*  8: vnode associated with block       */
   kt_t            b_timeval_at; /*  8: buffer allocation time            */
};

struct  kd_enqueue {
   kd_t               kd_struct;     /* KI trace common header            */
   dev_t              b_dev;         /*  4: major+minor device name           */
   bmcnt_t            b_merge_cnt;   /*  2: # of "gathered" I/O requests       */
   bqlen_t            b_queuelen;    /*  2: disc queue length (+long align bp)*/
   struct kd_enqueue2 ki_enq;
};

struct  kd_queuestart2 {
   kptr_t       bp;            /*  8: buffer ident                      */
   kptr_t       b_vp;          /*  8: vnode associated with block       */
   int32_t      b_bcount;      /*  4: transfer count                    */
   uint32_t     b_flags;       /*  4: too much goes here to describe    */
   uint16_t     b2_flags;      /*  2: refer to buf.h          */
   uint16_t     spad;          /*  2: short padding */
   daddr_t      b_blkno;       /*  4: block # on device                 */
   uint32_t     b_sectno;      /*  4: sect no.                      */
   uint32_t     pad;           /*  4: padding */
};

struct  kd_queuestart {
   kd_t                  kd_struct;     /* KI trace common header            */
   dev_t                 b_dev;         /*  4: major+minor device name           */
   bmcnt_t               b_merge_cnt;   /*  2: # of merged I/O requests        */
   uint16_t              spad;          /*  2: short padding */
   struct kd_queuestart2 ki_qstart;
};

struct kd_queuedone2 {
   kt_t     b_timeval_eq; /* 8:  enqueue time of request         */
   kptr_t   bp;           /* 8: buffer ident                    */
   kptr_t   b_vp;         /* 8: vnode associated with block     */
   kptr_t   b_rp;         /* 8: region associated with block    */
   dev_t    b_lvdev;      /* 4: lvm device name         */
   int32_t  b_bcount;     /* 4: transfer count                  */
   daddr_t  b_blkno;      /* 4: block # on device               */
   pid_t    b_upid;       /* 4: process id that last used bp    */
   pid_t    b_apid;       /* 4: process id that last allocated bp */
   tid_t    b_uutid;      /* 4: uthread id that last used bp    */
   tid_t    b_autid;      /* 4: uthread id that last allocated bp */
   tid_t    b_uktid;      /* 4: kthread id that last used bp    */
   tid_t    b_aktid;      /* 4: kthread id that last allocated bp */
   uint32_t b_resid;      /* 4: wds not transferred       */
   ksite_t  b_site;       /* 4: (IPaddr/local) site of request  */ 
   uint32_t b_sectno;     /* 4: Sect no.           */
   uint32_t b_flags;      /* 4: too much goes here to describe  */
   uint16_t b2_flags;     /* 2: refer to buf.h          */
   bptype_t b_bptype;     /* 2: buffer type                     */
   bqlen_t  b_queuelen;   /* 2: disc queue len         */
   uint16_t spad;         /* 2: modulo 4 padding                */
   uint32_t pad;          /* 4: modulo 8 padding                */
};

#define B_p   bp.kga_val
#define B_vp  b_vp.kga_val
#define B_rp  b_rp.kga_val

#if !defined(_KERNEL)
#   if !defined(__LP64__)
#define B_p64  bp.kga_val64
#define B_vp64 b_vp.kga_val64
#define B_rp64 b_rp.kga_val64
#   endif /* __LP64__ */
#endif /* _KERNEL */

#if !defined(_KD_QUEUEDONE2_T)
#define _KD_QUEUEDONE2_T
typedef struct kd_queuedone2 kd_queuedone2_t;
#endif /* !_KD_QUEUEDONE2_T */

struct  kd_queuedone {
   kd_t            kd_struct;    /*    KI trace common header         */
   kt_t            b_timeval_qs; /* 8: queuestart time of request     */
   dev_t           b_dev;        /* 4: major+minor device name        */
   uint16_t        retries;      /* 2: number of retries              */
   bmcnt_t         b_merge_cnt;  /* 2: # of merged I/O requests       */
   kd_queuedone2_t ki_qdone;
};

#if !defined(_KD_QUEUEDONE_T)
#define _KD_QUEUEDONE_T
typedef struct kd_queuedone kd_queuedone_t;
#endif /* !_KD_QUEUEDONE_T */

struct  kd_hardclock {
   kd_t            kd_struct;     /* KI trace common header           */
   kregister_t     pc;            /*  8: interrupted pc value         */
   kregister_t     dp;            /*  8: interrupted dp value         */
   kregister_t     sp;            /*  8: interrupted sp value         */
   kcpustate_t     cpstate;       /*  1: processor state              */
   kboolean_t      is64ASL;       /*  1: TRUE: 64ASL, FALSE: 32ASL    */
   uint16_t        spad;          /*  2: modulo 4 padding             */
   uint32_t        pad;           /*  4: modulo 8 padding             */
};

#if !defined(_KD_HARDCLOCK_T)
#define _KD_HARDCLOCK_T
typedef struct kd_hardclock kd_hardclock_t;
#endif /* _KD_HARDCLOCK_T */

struct  kd_brelse {
   kd_t         kd_struct;     /* KI trace common header            */
   kt_t         sttime;        /*  8: time of removal from lru          */
   kptr_t       bp;            /*  8: buffer ident                      */
   kptr_t       b_vp;          /*  8: vnode associated with block       */
   uint32_t     b_flags;       /*  4: too much goes here to describe    */
   bptype_t     b_bptype;      /*  2: buffer type                       */
   uint16_t     spad;          /*  2: modulo 8 padding                  */
};

struct  kd_getnewbuf {
   kd_t        kd_struct;     /* KI trace common header                */
   kptr_t      bp;            /*  8: buffer ident                      */
   kptr_t      b_vp;          /*  8: vnode associated with block       */
   int32_t     b_bufsize;     /*  4: size of allocated buffer          */
   bptype_t    b_bptype;      /*  2: buffer type                       */
   short       spad;          /*  2: modulo 8 padding                  */
};

struct  kd_miss_alpha {
   kd_t                  kd_struct;    /* KI trace common header             */
   uint32_t              nswitches;    /*  4: # of switches for missed alpha */
   uint32_t              pad;          /*  4: modulo 8 padding               */
   struct   ki_pk_state   ki_pk_state; /* updated process states             */
};

struct   kd_active {
   kd_t  kd_struct; /* KI trace common header       */
   pid_t p_pid;     /*  4: pid being deactivated/reactivated */
   int   flag;      /*  4: PF_DEACT if deactivated           */
                    /*            0 if reactivated           */
};

/* 
/* kd_swtch layout :*/
/*   To maintain binary compatibility between 32 and 64 bit, we were*/
/*   forced to have 4 bytes of padding to every element in ks_swt.  This*/
/*   really bloated this trace, which is very high volume.  To reduce */
/*   that penalty, we are putting data into this "pad".  This will */
/*   reduce the size of this trace, while not loosing any data.  To */
/*   use this data, access it in the following fashion :*/
/*        ks_swtch->ks_swt[<CONST_FOR_FIELD>].kp_data*/
//
/*    This will save 40 bytes per swtch trace.*/
*/

#define KI_VSS		0 
#define KI_RSS		1 
#define KI_P_EUID	2
#define KI_P_RUID	3
#define KI_RU_EUID	4
#define KI_RU_MINFLT	5
#define KI_RU_MAJFLT	6
#define KI_RU_NDEACT	7
#define KI_RU_NSIGNALS	8
#define KI_RU_MSGSND	9
#define KI_RU_MSGRCV   10	

struct  kd_swtch {
   kd_t          kd_struct;       /* KI trace common header            */
   kptr_t        ki_kt_wchan;     /*  8: event kthread is awaiting     */
   kptr_t        ki_sleep_caller; /*  8: return address from sleep     */
   kr_t          ks_swt[KT_NUMB_CLOCKS]; /*16*11c urrent KTC clocks    */
   kt_t          ki_syscallbeg;  /* 8: start time of last systemcall   */
   kt_t          ks_sys_time;    /* 8: KT_SYS time - last call or resume */
   kpkst_t       ki_pk_state;    /* 24:updated process  & kthread states  */
   ksyscallno_t  ks_syscal;      /* 2: last system call number         */
   kpri_t        ki_kt_usrpri;   /* 2: priority based on kt_cpu & p_nice */
   uint32_t	 pad;
				 /*_272_ total bytes */
};

#if !defined(_KD_SWTCH_T)
#define _KD_SWTCH_T
typedef struct kd_swtch kd_swtch_t;
#endif /* _KD_SWTCH_T */

struct  kd_region {
   kd_t      kd_struct;    /*  32: KI trace common header            */
   int       cl_numb;      /*   4: region operation                  */
   uint32_t  pad;          /*   4: modulo 8 padding                  */
   kptr_t    rp;           /*   8: process region pointer            */
   struct ki_region_fields region; /* 112 : process region           */
};			   /* 160 Total */

struct  kd_pregion {
   kd_t     kd_struct;     /*  32: KI trace common header            */
   int      cl_numb;       /*   4: pregion operation                 */
   uint32_t  pad;          /*   4: modulo 8 padding                  */
   kptr_t   prp;           /*   8: process pregion pointer           */
   struct ki_pregion_fields pregion; /* 136: process pregion            */
};			   /* 184 Total */

struct  kd_resume {
   kd_t kd_struct;   /* header placed by ki_data */

   /* current KTC clocks */
   struct ki_runtimes   ks_swt[KT_NUMB_CLOCKS];

   kt_t                ki_syscallbeg; /*  8: start time of last systemcall*/
   kpkst_t             ki_pk_state;   /* updated process & kthread states */
   ksyscallno_t        ks_syscal;     /*  2: last system call number      */
   uint16_t            spad;          /*  2: modulo 4 padding             */
   uint32_t            pad;           /*  4: modulo 8 padding             */
};

/*
* NFS address data definitions and structures
*/
#define MAX_NETBUF_ADDR_LEN  128
#define MAX_NFSVARLEN        128

typedef struct Ki_nfsvarlen_t  {
        short         len;           /* 2:Length of the variable array*/
        short         offset;        /* 2:Offset from beg of struct , */
} Ki_nfsvarlen_t ;

typedef struct clnt_netcfgu_t {
        uint32_t        knc_semantics; /* 4:token name */
        Ki_nfsvarlen_t  knc_protofmly; /* 4:protocol family */
        Ki_nfsvarlen_t  knc_proto;     /* 4:protocol */
        dev_t           knc_rdev;      /* 4:device id */
                                       /*16: Total*/
} clnt_netcfgu_t ;

typedef struct svc_netcfgu_t {
        int32_t         xp_type;       /* 4: Transport type: CLTS|COTS.. */
       dev_t           knc_rdev;      /* 4: device id */
} svc_netcfgu_t ;

typedef struct clnt_netaddru_t {
        clnt_netcfgu_t          knconf; /*16 : */
        Ki_nfsvarlen_t          addr;   /* 4 : */
        int32_t                 pad;    /* 4 : */
} clnt_netaddru_t ;

typedef struct svc_netaddru_t {
        Ki_nfsvarlen_t          addr;   /* 4 : */
                        /* ^^^ This needs to be first as to ensure that
                               knconf lines up */
        svc_netcfgu_t           knconf; /* 8 : */
} svc_netaddru_t ;

struct  kd_rfscall {
        kd_t            kd_struct;    /* KI trace common header         */
        kt_t            sttime;       /*  8:  start time of NFS operation */
        int32_t         kr_which;     /*  4: NFS layer number  */
        int32_t         kr_nfsvers;   /*  4: NFS version number */
        dev_t           kr_mntno;     /*  4: NFS mntinfo index  */
        uint32_t  kr_reqlength;       /*  4: length if read or write request */
        uint32_t  kr_reslength;       /*  4: length if read or write result */
        short     stable;             /*  2: for writes,Whether DATA_SYNC/UNSTABLE specified */
        short     committed;          /*  2: for writes,Whether committed to stable storage */
        clnt_netaddru_t kr_addr;      /* 24: Netbuf address and netconf info*/
};

struct  kd_rfs_dispatch {
        kd_t      kd_struct;    /* KI trace common header                */
        kt_t      sttime;       /*  8: start time of NFS operation       */
        int32_t   kr_which;     /*  4: NFS layer number                  */
        int32_t   kr_nfsvers;   /*  4: NFS version number                */
        uint32_t  kr_reqlength; /*  4: length if read or write request   */
        uint32_t  kr_reslength; /*  4: length if read or write result   */
        short     committed;    /*  2: Whether committed to stable storage */
        short     stable;       /*  2: committ type */
        /* This point needs to be modulo 4, so kr_addr lines up */
        svc_netaddru_t kr_addr ;/* 12: Netbuf address and conf info */
};

struct  kd_do_bio {
   kd_t        kd_struct;     /* 32: KI trace common header            */
   kptr_t       bp;            /*  8: buffer ident                      */
   dev_t       mntno;         /*  4: major+minor device name           */
   uint32_t    b_flags;       /*  4: too much goes here to describe    */
   int32_t     b_bcount;      /*  4: transfer count                    */
   bptype_t    b_bptype;      /*  2: buffer type                       */
   bqlen_t     b_queuelen;    /*  2: disc queue length (+long align bp)*/
   pid_t       b_upid;        /*  4: process id that last used bp      */
   pid_t       b_apid;        /*  4: process id that last allocated bp */
   tid_t       b_uutid;       /*  4: uthread id that last used bp      */
   tid_t       b_autid;       /*  4: uthread id that last allocated bp */
   tid_t       b_uktid;       /*  4: kthread id that last used bp      */
   tid_t       b_aktid;       /*  4: kthread id that last allocated bp */
   kt_t        b_timeval_eq;  /*  8: enqueue time of request           */
   kt_t        b_timeval_qs;  /*  8: queuestart time of request        */
   uint32_t    b_resid;       /*  4: words not transferred             */
   int32_t     kr_nfsvers;    /*  4: NFS version number                */
   uint32_t    commit_buf_cnt;/*  4: count of buffers to commit for the rnode */

   uint32_t    commit_commlen;/*  4: len of data in buffer to commit for rnode*/

   kptr_t      b_vp;          /*  8: vnode associated with block       */
   clnt_netaddru_t kr_addr ;  /* 24: Netbuf address and netconf info*/
};

struct kd_setrq {
   kd_t      kd_struct;    /* 32: KI trace common header             */
   kptr_t    setrq_caller; /*  8: return address from setrq          */
   tid_t     kt_tid;       /*  4: kthread being queued               */
   pid_t     p_pid;        /*  4: process the thread belongs         */
   kptr_t    p_vas;        /*  8: virtual address space for process  */
   kpkst_t   ki_pk_state;  /*     updated process & kthread states   */
};

#if !defined(_KD_SETRQ_T)
#define _KD_SETRQ_T
typedef struct kd_setrq kd_setrq_t;
#endif /* _KD_SETRQ_T */

struct kd_closef {
   kd_t     kd_struct;   /* 32: KI trace common header            */
   kptr_t   f_data;      /*  8: vnode, socket pointer etc.        */
   dev_t    f_dev;       /*  4: device major&minor number         */
   kftype_t f_type;      /*  2: type of f_data                    */
   ktype_t  v_fstype;    /*  1: file system type                  */
   ktype_t  v_type;      /*  1: vnode type                        */
   int32_t  f_count;     /*  4: open count after closef           */
   uint32_t pad;         /*  4: modulo 8 padding                  */
};

#if !defined(_KD_CLOSEF_T)
#define _KD_CLOSEF_T
typedef struct kd_closef kd_closef_t;
#endif /* !_KD_CLOSEF_T */

struct kd_locallookuppn {
   kd_t              kd_struct;   /* 32: KI trace common header            */
   kt_t              sttime;      /*  8: start time of call                */
   kptr_t            vp;          /*  8: vnode                             */
   dev_t             dev;         /*  4: major+minor device name           */
   int32_t           i_number;    /*  4: inode number                      */
   ktype_t           v_fstype;    /*  1: file system type                  */
   ktype_t           v_type;      /*  1: vnode type                        */
   short             error;       /*  2: error code                        */
   uint32_t          pad;         /*  4: modulo 8 padding                  */
};

struct kd_fault {
   kd_t          kd_struct;     /*  32: KI trace common header            */
   int32_t       cl_numb;       /*   4: caller number                     */
   int32_t       wrt;           /*   4: read/write flag                   */
   int32_t       space;         /*   4: space addresss of fault           */
   int32_t       pcause;        /*   4: reason of fault                   */
   kptr_t        vaddr;         /*   8: virtual addresss of fault         */
   kptr_t        pcoq_head;     /*   8: instruction @ of fault            */
   int32_t       r_nvalid;      /*   4: number resident pages in region   */
   uint32_t      pad;           /*   4: modulo 8 padding                  */
   kptr_t        prp;           /*   8: pregion pointer of fault          */
   struct ki_pregion_fields pregion; /* 136: pregion of fault             */
};				/* 216 : Total */

struct kd_asyncpageio {
   kd_t            kd_struct;     /* 32: KI trace common header           */
   kptr_t          bp;            /*  8: buffer ident                     */
   kptr_t          b_vp;          /*  8: vnode associated with block      */
   uint32_t        b_flags;       /*  4: too much goes here to describe   */
   int32_t         b_bcount;      /*  4: transfer count                   */
   daddr_t         b_blkno;       /*  4: block # on device                */
   bptype_t        b_bptype;      /*  2: buffer type                      */
   uint16_t        spad;          /*  2: modulo 8 padding                 */
};

struct kd_memfree {
   kd_t     kd_struct;     /* 32: KI trace common header            */
   kptr_t   rp;            /*  8: process region  pointer           */
   physpfn_t r_nvalid;      /*  4: number resident pages in region   */
   uint32_t pfn;           /*  4: page frame number                 */
};

struct  kd_userproc {
   kd_t          kd_struct;        /* 32: header placed by ki_data          */
   kr_t          ks_swt[KT_NUMB_CLOCKS]; /* current KTC clocks              */
   kt_t          ki_syscallbeg;    /*  8: start time of last systemcall     */
   kt_t          ks_sys_time;      /*  8: KT_SYS time - last call or resume */
   kpkst_t       ki_pk_state;      /*   : updated process  & thread states  */
   utrace_type_t usertype;         /*  4: user specified type               */
   ksyscallno_t  ks_syscal;        /*  2: system call number                */
   uint16_t      spad;             /*  2: modulo 8 padding                  */
/* char          userbuf[];          user passed buffer                     */
};

#if !defined(_KD_USERPROC_T)
#define _KD_USERPROC_T
typedef struct kd_userproc kd_userproc_t;
#endif /* _KD_USERPROC_T */

struct kd_smount {             /* VFS Mount:                        */
   kd_t         kd_struct;     /* 32: KI trace common header            */
   kptr_t       smnt_vfsp;     /*  8: vfs pointer                       */
   fsid_t       smnt_fsid;     /*  8: vfs ID for vfs_get                */
   kflag_t      smnt_flag;     /*  4: vfs flags                         */
   int32_t      smnt_bsize;    /*  4: vfs native block size             */
   int32_t      smnt_mnttime;  /*  4: vfs time mounted                  */
   dev_t        smnt_dev;      /*  4: vfs device (if approp)            */ 
   kmtype_t     smnt_mtype;    /*  2: Type of vfs                       */
   uint16_t     smnt_spad;     /*  2: modulo 4 padding                  */ 
   uint32_t     smnt_pad;      /*  4: modulo 8 padding                  */ 
/* char         smnt_name[];     vfs file sys identifier            */
/* char         smnt_dir[];      mount dir name                     */
};

#define SMNT_vfsp   smnt_vfsp.kga_val

#if !defined(_KERNEL)
#   if !defined(__LP64__)
#define SMNT_vfsp64 smnt_vfsp.kga_val64
#   endif /* __LP64__ */
#endif /* _KERNEL */

#if !defined(_KD_SMOUNT_VFS_T)
#define _KD_SMOUNT_VFS_T
typedef struct kd_smount_vfs kd_smount_vfs_t;
#endif /* !defined(_KD_SMOUNT_VFS_T) */

struct kd_umount_vfs {         /* VFS UnMount:                      */
   kd_t         kd_struct;     /* 32: KI trace common header            */
   kptr_t       umnt_vfsp;     /*  8: vfs pointer                       */
   fsid_t       umnt_fsid;     /*  8: vfs ID for vfs_get                */
   kflag_t      umnt_flag;     /*  4: vfs flags                         */
   dev_t        umnt_dev;      /*  4: vfs device (if approp)            */
   kmtype_t     umnt_mtype;    /*  2: Type of vfs                       */
   short        umnt_spad;     /*  2: modulo 4 padding                  */
   uint32_t     smnt_pad;      /*  4: modulo 8 padding                  */ 
};

#define UMNT_vfsp   umnt_vfsp.kga_val

#if !defined(_KERNEL)
#   if !defined(__LP64__)
#define UMNT_vfsp64 umnt_vfsp.kga_val64
#   endif /* __LP64__ */
#endif /* _KERNEL */

#if !defined(_KD_UMOUNT_VFS_T)
#define _KD_UMOUNT_VFS_T
typedef struct kd_umount_vfs kd_umount_vfs_t;
#endif /* !defined(_KD_UMOUNT_VFS_T) */

struct  kd_thread_create {  /* Kthread creation:                 */
   kd_t                     kd_struct;  /* 32: KI trace common header     */ 
   kthread_create_caller_t  caller;     /*  4: initiator of the trace     */
   kflag_t                  flags;      /*  4: thread_create() parameters */
   tid_t                    ktid;       /*  4: newly created kthread id   */
   tid_t                    utid;       /*  4: user thread id             */
   ktst_t                   ki_kt_state;/* 16: kthread start state        */
   kptr_t		    ktc_start_routine; /* addr of start routine   */
};

#if !defined(_KD_THREAD_CREATE_T)
#define _KD_THREAD_CREATE_T
typedef struct kd_thread_create kd_thread_create_t;
#endif /* !_KD_THREAD_CREATE_T */

struct   kd_thread_exit {   /* Kthread exit:                     */
   kd_t         kd_struct;  /* 32: KI trace common header            */
   tid_t        ktid;       /*  4: exiting kthread id                */
   uint32_t     pad;        /*  4: modulo 8 padding                  */ 
};

#if !defined(_KD_THREAD_EXIT_T)
#define _KD_THREAD_EXIT_T
typedef struct kd_thread_exit kd_thread_exit_t;
#endif /* !_KD_THREAD_EXIT_T */

#endif /* !defined(__STDC_32_MODE__) */
#endif /* _SYS_KI_INCLUDED */


