/* $Source: /usr/local/kcs/sys.ROSE_800/h/RCS/ki_calls.h,v $
 * $Revision: 1.17.106.4 $      $Author: bpohl $
 * $State: Exp $        $Locker: CRT $
 * $Date: 97/08/31 13:07:46 $
 */
/* HPUX_ID: @(#)ki_calls.h      54.1            88/11/13 */

#ifndef _SYS_KI_CALLS_INCLUDED /* allows multiple inclusion */
#define _SYS_KI_CALLS_INCLUDED

#ifdef  _KERNEL_BUILD
#include "../machine/mp.h"
#include "../h/reg.h"
#include "../h/time.h"
#include "../h/vfs.h"
#include "../h/ki_defs.h"
#else  /* ! _KERNEL_BUILD */
#include <machine/mp.h>
#include <sys/time.h>
#include <sys/vfs.h>
#include <sys/ki_defs.h>
#endif /* _KERNEL_BUILD */

/************************************************************************/
/* KI_MAX_PROCS is an arbitrary number.  It provides a user space       */
/* process a handle to read the ki_config structure before knowing how  */
/* many processors are configured and active.  It's set to 32 based on  */
/* the assumption that we won't ever support more than 32-way on PA     */
/* boxes.                                                               */
/************************************************************************/
#define KI_MAX_PROCS            32

/************************************************************************/
/*      number of parts to the trace buffer each CPU has                */
/*              (must be power of 2 in size )                           */
/************************************************************************/
#define NUM_TBUFS       4

/************************************************************************/
/*                      KI function call values                         */
/************************************************************************/

/* leave room for old Kernel GPROF parameters */
#define KI_ALLOC_TRACEMEM       10      /* Allocate sys memory for KI */
#define KI_CONFIG_READ          11      /* Read config table of KI    */
#define KI_TRACE_GET            12      /* Get a KI trace buffer      */
#define KI_USER_TRACE           13      /* Put User record in trace   */
#define KI_ALLOCATE_CT          14      /* Get CT memory from kernel  */
#define KI_READ_CT              15      /* Read some CT memory        */
#define KI_WRITE_CT             16      /* Write some CT memory       */
#define KI_TIMEOUT_SET          17      /* Set max wait time for trace*/
#define KI_PROC_TRACE           18      /* Put any record in trace buf*/
#define KI_KMEM_GET             19      /* Read kernel memory         */
#define KI_KMEM_GETV            20      /* Read kernel memory         */

#define KI_SET_SYSCALLTRACE     21      /* ON  one System call trace  */
#define KI_CLR_SYSCALLTRACE     22      /* OFF one System call trace  */
#define KI_SET_ALL_SYSCALTRACES 23      /* ON  all System call traces */
#define KI_CLR_ALL_SYSCALTRACES 24      /* OFF all System call traces */
#define KI_SET_SYSCALLTRACE_BYPID 25    /* ON  per-process syscall trace */
#define KI_CLR_SYSCALLTRACE_BYPID 26    /* OFF per-process syscall trace */

#define KI_SET_KERNELTRACE      27      /* ON  one Kernel stub trace  */
#define KI_CLR_KERNELTRACE      28      /* OFF one Kernel stub trace  */
#define KI_SET_ALL_KERNELTRACES 29      /* ON  all Kernel stub traces */
#define KI_CLR_ALL_KERNELTRACES 30      /* OFF all Kernel stub traces */

#define KI_FREE_TRACEMEM        31      /* De-alloc sys memory for KI */
#define KI_CONFIG_CLEAR         32      /* Clear KI counters          */
#define KI_TRACE_SWITCH         33      /* Switch to a new trace buf  */
#define KI_TIMEOUT_GET          34      /* Return ki_timeout value    */

/************************************************************************/
/* enable flags in the  kc_kernelenable[KI_MAXKERNCALLS]; buffer        */
/************************************************************************/
#define KI_GETPRECTIME          0       /* enable on KI precise timer */
#define KI_SYSCALLS             1       /* enable on system call trace*/
#undef  KI_SERVESTRAT        /* 2 obsolete */
#undef  KI_SERVSYNCIO        /* 3 obsolete */
#define KI_ENQUEUE              4
#define KI_QUEUESTART           5
#define KI_QUEUEDONE            6
#define KI_HARDCLOCK            7
#define KI_CLOSEF               8
#define KI_BRELSE               9
#define KI_GETNEWBUF            10
#define KI_MELCO_TRACE          11      /* Trace reserved for MELCO driver */
#define KI_SWTCH                12
#undef  KI_RESUME_NCS        /* 13 obsolete */
#define KI_RESUME               14      /* in locore.s */
#define KI_HARDCLOCK_IDLE    /* 15 obsolete */
#define KI_SUIDPROC             16      /* from measurement process */
#undef  KI_DM1_SEND          /* 17 obsolete */
#undef  KI_DM1_RECV          /* 18 obsolete */
#define KI_RFSCALL              19      /* NFS server requeste */
#define KI_RFS_DISPATCH         20      /* NFS client requests */
#define KI_SETRQ                21      /* in locore.s */
#define KI_DO_BIO               22      /* NFS driver to server */
#define KI_USERPROC             23      /* from user process */
#define KI_MEMFREE              24      /* free page from region */
#define KI_ASYNCPAGEIO          25      /* pageing and swapping activity */
#define KI_LOCALLOOKUPPN        26      /* pathname locallookuppn */
#define KI_FAULT                27      /* Virtual page / Protection faults */
#define KI_MISS_ALPHA           28      /* Missed an alpha semaphore */
#define KI_PREGION              29      /* Process pregion structures */
#define KI_REGION               30      /* Process region structures */
#define KI_ACTIVE               31      /* Process deactivation/reactivation */
#define KI_SMOUNT               32      /* mount file system */
#define KI_UMOUNT_VFS           33      /* unmount file system */
#define KI_THREAD_CREATE        34      /* kthread creation */
#define KI_THREAD_EXIT          35      /* kthread exit */

/************************************************************************/
/*      Size of the KI kernel call table, only the above 33 are used    */
/************************************************************************/
#define KI_MAXKERNCALLS 50

/************************************************************************/
/* size of the KI system call table; this constant has to be reviewed   */
/* on every release to validate the total system call numbers           */
/************************************************************************/

#define KI_MAXSYSCALLS  600 /* HPUX REVISIT:
				 The ki system call table should be
				 determined by the BE at compile-time
				 This constant wastes space and uses
				 too much mantainance */

/************************************************************************/
/*      The following are fake system call numbers assigned to the      */
/*      kernel daemon processes that will never do a system call.       */
/*      These system daemon use the unused slots at the end of the      */
/*      kt_syscall_times table as their time buckets.                   */
/************************************************************************/
#define KI_SWAPPER              (KI_MAXSYSCALLS - 2)
#define KI_VHAND                (KI_MAXSYSCALLS - 3)
#define KI_STATDAEMON           (KI_MAXSYSCALLS - 4)
#define KI_SOCKREGD             (KI_MAXSYSCALLS - 5)
#define KI_NETISR               (KI_MAXSYSCALLS - 6)
#define KI_SYNCDAEMON           (KI_MAXSYSCALLS - 7)
#define KI_STRSCHED_KXTI_S      (KI_MAXSYSCALLS - 8)
#define KI_STRSCHED_KXTI_R      (KI_MAXSYSCALLS - 9)
#define KI_SPINUPD              (KI_MAXSYSCALLS -10)
#define KI_XPORTD               (KI_MAXSYSCALLS -11)
#define KI_UNHASHDAEMON         (KI_MAXSYSCALLS -12)
#define KI_VDMAD                (KI_MAXSYSCALLS -13)
#define KI_LVMKD                (KI_MAXSYSCALLS -14)
#define KI_VXD                  (KI_MAXSYSCALLS -15)
#define KI_NVSISR               (KI_MAXSYSCALLS -16)
#define KI_STRWELD              (KI_MAXSYSCALLS -17)
#define KI_STRSCHED_UP          (KI_MAXSYSCALLS -18)
#define KI_STRSCHED_MP          (KI_MAXSYSCALLS -19)
#define KI_STRSCHED_BLK         (KI_MAXSYSCALLS -20)
#define KI_STRMEM               (KI_MAXSYSCALLS -21)
#define KI_TTISR                (KI_MAXSYSCALLS -22)
#define KI_DCEPTHREAD           (KI_MAXSYSCALLS -23)
#define KI_NFSKD                (KI_MAXSYSCALLS -24)
#define KI_KTEST                (KI_MAXSYSCALLS -25)
#define KI_COMMITDAEMON         (KI_MAXSYSCALLS -26)
#define KI_ASYNC_DAEMON         (KI_MAXSYSCALLS -27)

/************************************************************************/
/*              reasons for pfault, 0 is a KI error                     */
/************************************************************************/
#define KI_INVALID         1    /* invalidated due to vhand or sched         */
#define KI_MPROTECT        2    /* protection fault due to writing to        */
                                /* mprotect'ed shared mmap region            */
#define KI_MMAP_DBD_HOLE   3    /* write fault to a DBD_HOLE in a mmap file  */
#define KI_COPYONWRITE     4    /* copy on write fault                       */
#define KI_INTRANSIT       5    /* page is in transit (paging in or out)     */
#define KI_ACCESSRIGHT     6    /* access right fault                        */
#define KI_PROTID          7    /* protection id fault                       */
#define KI_ALIGNMENT       8    /* alignment fault                           */
#define KI_HDL_OTHERFAULTS 9    /* process in zombie state, null pointer     */
                                /* dereferencing detected, ..., etc.         */

/************************************************************************/
/*              values used by ki_fault()                               */
/************************************************************************/
#define KI_PFAULTN      1
#define KI_VFAULTN      2

/************************************************************************/
/*              values used by ki_region()                              */
/************************************************************************/
#define KI_ALLOCREG     1
#define KI_FREEREG      2
#define KI_DUPREG       3
#define KI_GROWREG      4

#define KI_FREEPREG     1
#define KI_DUPPREGION   2
#define KI_ATTACHREG    3
#define KI_GROWPREG     4

/************************************************************************/
/*                      Originators of thread create                    */
/************************************************************************/

typedef enum _kthread_create_caller {
   KI_LWP_CALLER = 1,   /* non    daemon thread */ 
   KI_SKTHREAD_D = 2,   /* System daemon thread */
   KI_UKTHREAD_D = 3    /* User   daemon thread */
} kthread_create_caller_t;

/************************************************************************/
/*                      Possible KI clock states                        */      
/************************************************************************/
/* accumulated KERNEL (systemcall) time in behalf of process    */
#define KT_SYS_CLOCK            0
/* accumulated USER time in behalf of process                   */
#define KT_USR_CLOCK            1
/* accumulated CSW time (context switch)                        */
#define KT_CSW_CLOCK            2
/* accumulated IDLE time                                        */
#define KT_IDLE_CLOCK           3
/* accumulated INT/USR interrupt time robbed from process       */
#define KT_INTUSR_CLOCK         4
/* accumulated INT/IDLE interrupt time robbed from IDLE         */
#define KT_INTIDLE_CLOCK        5
/* accumulated INT/SYS interrupt time robbed from process       */
#define KT_INTSYS_CLOCK         6
/* accumulated virtual page fault time from process             */
#define KT_VFAULT_CLOCK         7
/* accumulated protection fault time from process               */
#define KT_UPFAULT_CLOCK        8
/* accumulated protection fault time in general                 */
#define KT_PFAULT_CLOCK         9
/* accumulated trap time from process                           */
#define KT_TRAP_CLOCK           10
/* accumulated spare 0 time from process                        */
/*#define KT_SPARE_CLOCK          11*/

/*
 * KT_SPARE_CLOCK was removed to save space in the kd_swtch struct.  New
 * clock types can be introduced, but remember to increase KT_NUMB_CLOCKS
 */

#define KT_NUMB_CLOCKS          11

/************************************************************************/
/*              per-clock information                                   */
/************************************************************************/

/* 
/* ki_runtimes: Define a structure for KTC clock used by per processor*/
*/

struct  ki_runtimes  {
   kt_t        kp_accumtm;  /* 8:             */
   kclockcnt_t kp_clockcnt; /* 4: Maybe we should go to muint64_t for 
							kclockcnt_t */
   uint32_t    kp_data;      /* 4: This carries various data for
				high volume traces.  see kd_swtch  */
#if defined(KTC_CPI)
/* Enhancement.
/*    The new field in the KTC structure can be used to store the processor*/
/*    recovery counter deltas. This counter is used to calculate the per */
/*    processor mode (or KTC) cycles per instruction counter (or CPI).*/
*/
   uint64_t    kp_accumrc;
#endif /* KTC_CPI */
};

#if !defined(_KR_T)
#define _KR_T
typedef struct ki_runtimes  kr_t; 
#endif /* _KR_T */

#define KP_accumtm kp_accumtm


/************************************************************************/
/*              per-processor information                               */
/************************************************************************/

struct ki_proc_info {   
   /* Relative rate of the CR_IT's from the monarch processor             */
   double      kpi_freq_ratio;          /*  8: CR_IT ratio from monarch    */
   kt_t        kpi_time;                /*  8: This cpu's view of time     */
   kptr_t      kpi_next[NUM_TBUFS];     /* 32: Next avail byte of buf N    */
   kcaddr_t    kpi_tbuf[NUM_TBUFS];     /* 32: Allocated trace buffers     */
   kregister_t kpi_incmclk;             /*  8: CR_IT for last KTC clock    */
   kregister_t kpi_inckrc;              /*  8: CR_RC for last KTC clock    */
   kregister_t kpi_offset_correction;   /*  8: add my_CR_IT=monarch_CR_IT  */
   kregister_t kpi_inckclk;             /*  8: CR_IT for ki_getprectime()  */
   /*           
   /*                  The current sum is: 112. */
   /* */
   */

   /* per processor KTC clock                                            */
   kr_t        kpi_timestruct[KT_NUMB_CLOCKS];

   kbufno_t    kpi_curbuf;              /*  4: current buffer number      */

   /* counts of kernel routine call when enabled (above)                 */
   uint32_t    kpi_kernelcounts[KI_MAXKERNCALLS];

   /* counts of system calls when enabled (above) (see signal.h)         */
   uint32_t    kpi_syscallcounts[KI_MAXSYSCALLS];

   /* array to store accumulated system call times                       */
   kt_t        kpi_syscall_times[KI_MAXSYSCALLS];       

};

#if !defined(_KPI_T)
#define _KPI_T
typedef struct ki_proc_info     kpi_t;
#endif /* _KPI_T */

/*
//
/* Access macros for the KI Proc Info register typed fields:*/
//
*/

/*      RawCrItCount_t                                                     */
#define                KPI_incmclk           kpi_incmclk.kreg_it   

/*      RawCrRcCount_t                                                     */
#define                KPI_inckrc            kpi_inckrc.kreg_rc

/*      RawCrItDelta_t                                                     */
#define                KPI_offset_correction kpi_offset_correction.kreg_itdelta

/*      RawCrItCount_t                                                     */
#define                KPI_inckclk           kpi_inckclk.kreg_it

/*
//
/* Access macros for the KI Proc Info kt_t typed fields:*/
//
*/

#define KPI_time kpi_time

/*
//
/* Access macros for the KI Proc Info Buffer field:*/
//
/* Note: _Bufno is modulo NUM_TBUFS value.*/
//
*/

#define KPI_tbuf(_Bufno) kpi_tbuf[(_Bufno)].kca_val


/************************************************************************/
/*               KI configuration specific information                  */
/*       A convenient way to access a ki_config structure: ki_cf        */
/************************************************************************/

#define ki_version              (ki_cf.kc_version)
#define ki_runningprocs         (ki_cf.kc_runningprocs)
#define ki_trace_sz             (ki_cf.kc_trace_sz)     
#define ki_cbuf                 (ki_cf.kc_cbuf.kca_val)
#define ki_count_sz             (ki_cf.kc_count_sz)
#define ki_timeout              (ki_cf.kc_timeout)
#define ki_timeoutct            (ki_cf.kc_timeoutct)
#define ki_nunit_per_sec        (ki_cf.kc_nunit_per_sec)
#define ki_tracing_info         (ki_cf.kc_tracing_info)
#define ki_syscallenable        (ki_cf.kc_syscallenable)
#define ki_kernelenable         (ki_cf.kc_kernelenable)
#define ki_sequence_count       (ki_cf.kc_sequence_count)

#define kc_proc(I)              (ki_cf.kc_proc_info_addr[I])

/************************************************************************/
/*      Kernel usage of this struct must align to cacheline             */ 
/*                                                                      */
/* Note :                                                               */
/*    Because of the KI API, we tried to define this structure binary   */
/*    compatible between __ILP32__ and __LP64__.                        */
/*                                                                      */
/* Warning:                                                             */
/*    The layout of this structure may change at every new OS release.  */
/*    Its definitions should be defined with accesses performance in    */
/*    mind. The KI module being itself very release dependent.          */
/************************************************************************/

struct ki_config     
{
   char      kc_version[32];         /* 32: version string                  */
   int       kc_runningprocs;        /*  4: number of active processors     */
   uint32_t  kc_trace_sz;            /*  4: trace buffer size               */
   kcaddr_t  kc_cbuf;                /*  8: counter pool                    */
   uint32_t  kc_timeout;             /*  4: timeout in 1/HZ seconds         */
   uint32_t  kc_timeoutct;           /*  4: timeout clock KI_TRACE_GET      */
   uint32_t  kc_nunit_per_sec;       /*  4: native units per second         */
   kseqcnt_t kc_sequence_count;      /*  4: tracing sequence count          */
   /*                                   --
   /* The current sum is a power of 2 : 64. Mostly for MP Performance issues.*/
   */

   /* 
   /* array of booleans to handle kernel stubs traces: */
   /*    - TRUE  if this kernel stub is being traced.*/
   /*    - FALSE if this kernel stub is not being traced.*/
   */

   kboolean_t  kc_kernelenable[KI_MAXKERNCALLS];

   /*
   /* ki_proc_info_addr[] is an array of pointers to the ki_proc_info */
   /* structures in the per-processor mpproc_info[] structure. */
   /* Beginning of this structure should be aligned to the type */
   /* of the 1st element in the ki_proc_info_addr structure.   */
   */

   kgaddr_t  kc_proc_info_addr[KI_MAX_PROCS];

   /* 
   /* array of booleans to handle system call traces:*/
   /*    - TRUE  if this system call being traced.*/
   /*    - FALSE if this system call is not being traced.*/
   */

   kboolean_t  kc_syscallenable[KI_MAXSYSCALLS];

   /*
   /* ki_config - not often accessed members.*/
   /* As a start (11/96):*/
   */

   uint32_t  kc_count_sz;            /*  4: counter pool size               */
   uint32_t  ki_max_tbsz;            /* Max size of kernel trace buffers */

};

#if !defined(_KC_T)
#define _KC_T
typedef struct ki_config        kc_t;
#endif /* _KC_T */

#define KI_CF   sizeof(struct ki_config)

/*
/*/////////////////////////////////////////////////////////////////////////////*/
//
/* KI Time Related Conversion Macros:*/
//
/*/////////////////////////////////////////////////////////////////////////////*/
*/

#if !defined(_KERNEL)

/* 
/* ki_timeval conversions:*/
*/

#define double_ki_timeval_to_sec(X) \
        (double_nunit_to_sec((unsigned int)(X)))

#define double_ki_timeval_to_msec(X) \
        (double_nunit_to_msec((unsigned int)(X)))

#define double_ki_timeval_to_usec(X) \
        (double_nunit_to_usec((unsigned int)(X)))

/* 
/* timeval conversions:*/
*/

#define double_timeval_to_sec(X) \
        (((double) (X).tv_sec) + ((double) (X).tv_usec)/1000000.0)
#define double_timeval_to_msec(X) \
        ((((double) (X).tv_sec)*1000.0) + (((double)(X).tv_usec)/1000.0))
#define double_timeval_to_usec(X) \
        (((double) (X).tv_sec)*1000000.0 + (X).tv_usec)

/* 
/* nunit conversions:*/
//
/*   Note that, for performance, ki_nunit_per_sec is casted to a double.*/
*/

#define double_nunit_to_sec(A) ( ((double)(A)) / ((double)ki_nunit_per_sec) )

#ifdef  FAST_NUNIT_CONVERSION

double ki_nunit_per_msec;
double ki_nunit_per_usec;

/* 
/* Must first call fast_nunit_conversion_init() at appropriate place*/
/* (e.g. shortly after a call to ki_call(KI_CONFIG_READ))            */
*/
   
#define fast_nunit_conversion_init() \
        ki_nunit_per_msec = ((double)ki_nunit_per_sec)/1000.0; \
        ki_nunit_per_usec = ((double)ki_nunit_per_sec)/1000000.0

#define double_nunit_to_msec(A) ( ((double)(A)) / ki_nunit_per_msec )

#define double_nunit_to_usec(A) ( ((double)(A)) / ki_nunit_per_usec )

#else /* !FAST_NUNIT_CONVERSION */

#define double_nunit_to_msec(A) \
        ( ( ((double)(A)) / ((double)ki_nunit_per_sec) ) * 1000.0 )
#define double_nunit_to_usec(A) \
        ( ( ((double)(A)) / ((double)ki_nunit_per_sec) ) * 1000000.0 )

#endif  /* FAST_NUNIT_CONVERSION */

#endif  /* !_KERNEL */

#endif /* _SYS_KI_CALLS_INCLUDED */
