/*
 * @(#)ki_defs.h: $Revision: 1.2.106.2 $ $Date: 97/07/12 22:12:56 $
 * $Locker: CRT $
 */

/*
/* Kernel Instrumentation Definitions:*/
//
/* This file provides kernel instrumentation definitions exposed to */
/* the other kernel components and to the user-space programs.*/
//
*/

#ifndef _SYS_KI_DEFS_INCLUDED /* allows multiple inclusion */
#define _SYS_KI_DEFS_INCLUDED

/*
/* Include inttypes.h to get basic fixed-sized integer types definitions*/
*/

#ifndef _KERNEL_BUILD
#include <sys/stdsyms.h>
#include <sys/_inttypes.h>
#else /* KERNEL_BUILD */
#include "../h/stdsyms.h"
#include "../h/_inttypes.h" 
#endif /* _KERNEL_BUILD */

/*
/* KI flag for 64bit specific interface:*/
//
/* This will allow 32bit applications to get KI structures 64bit format.*/
*/

#define KI64_INTF (int32_t)0x80000000

/*
/*/////////////////////////////////////////////////////////////////////////////*/
//
/* KI specific types:*/
//
/*/////////////////////////////////////////////////////////////////////////////*/
*/

/* This data type is to reserve space in other header files for 64bit 
/*	integers.  *ANYONE* using KI should not use this.  It is strictly */
/*	to maintain binary compatibility with other BE headers.*/
//
/* There is a fundamental assumption that 4 byte integers are not padded*/
/* to 8 bytes.  If this changes, this structure must be changed to */
/* accomadate that.*/
*/

struct ki_space64 {
	uint32_t pad1;
	uint32_t pad2;
}; 

/*
/* KI kernel generic pointer type:*/
//
/*   - mostly used for KI traces definitions:*/
//
*/

#if !defined(_KPTR_T)
#define _KPTR_T
#if !defined(__STDC_32_MODE__)
typedef uint64_t kptr_t;
#else
#if !defined(_KERNEL)
typedef struct ki_space64 kptr_t;
#endif /* !_KERNEL */
#endif /*narrow mode space reservation*/
#endif /* !_KPTR_T */

/*
/* KI struct buf type:*/
*/

#if !defined(_KBUF_T)
#define _KBUF_T
typedef struct buf kbuf_t;
#endif /* !_KBUF_T */

/*
/* Kernel Instrumentation Time types definitions:*/
//
/* Notes :*/
/*    - kt_t defines the ILP32 or LP64 Kernel Instrumentation Time type.*/
/*      Its format is binary compatible between ILP32 and LP64. */
*/

/*
/* KI Time type:*/
/* */
/*   This definition is binary compatible between ILP32 and LP64.*/
*/

#if !defined(_KCLOCKCNT_T)
#define _KCLOCKCNT_T
typedef uint32_t kclockcnt_t;
#endif /* _KCLOCKCNT_T */

#if !defined(_BPTYPE_T)
#define _BPTYPE_T
typedef uint16_t bptype_t;
#endif /* !_BPTYPE_T */

#if !defined(_BMCNT_T)
#define _BMCNT_T
typedef uint16_t bmcnt_t;
#endif /* !_BMCNT_T */

#if !defined(_BQLEN_T)
#define _BQLEN_T
typedef uint16_t bqlen_t;
#endif /* !_BQLEN_T */

#if !defined(_KSITE_T)
#define _KSITE_T
typedef uint32_t ksite_t;
#endif /* !_KSITE_T */

#if !defined(_KBOOLEAN_T)
#define _KBOOLEAN_T
typedef unsigned char kboolean_t;
#define KTRUE  ((kboolean_t)TRUE)
#define KFALSE ((kboolean_t)FALSE)
#endif /* !_KBOOLEAN_T */

#if !defined(_KPGCNT_T)
#define _KPGCNT_T
typedef uint32_t kpgcnt_t;
#endif /* !_KPGCNT_T */

#if !defined(_GADDR_T)
#define _GADDR_T
typedef void * gaddr_t;
#endif /* !_GADDR_T */

/*
/* KI Kernel Global Address Type*/
//
/* Description of kgaddr union fields references:*/
/*    - kga_val:   ILP32/LP64 Kernels - referenced*/
/*                 LP64       User    - referenced*/
/*                 ILP32      User    - referenced with ILP32 Kernel*/
/*    - kga_val64: ILP32/LP64 Kernels - no access*/
/*                 ILP32      User    - referenced with LP64  Kernel*/
*/

#if !defined(_KGADDR_T)
#define _KGADDR_T
typedef union _kgaddr {
   gaddr_t   kga_val; 
#if !defined(__STDC_32_MODE__)
   uint64_t  kga_val64;
#endif /*narrow mode space reservation*/
   struct kgaddr_split {
    	uint32_t high;     /*upper half of kgaddr_t*/
   	uint32_t low;   /*lower half of kgaddr_t*/ 
   } split;
} kgaddr_t;
#endif /* !_KGADDR_T */

/*
/* KI Kernel Char / Byte Address Type:*/
//
/* Description of kgaddr union fields references:*/
/*    Same restrictions as kgaddr_t.*/
*/

#if !defined(_KCADDR_T)
#define _KCADDR_T
typedef union _kcaddr {
   caddr_t   kca_val; 
#if !defined(__STDC_32_MODE__)
   uint64_t  kca_val64;
#endif
   struct kcaddr_split {
        uint32_t high;     /*upper half of kgaddr_t*/
        uint32_t low;   /*lower half of kgaddr_t*/
   } split;
} kcaddr_t;
#endif /* !_KCADDR_T */

/*
/* KI Clock Type:*/
*/

#if !defined(_KCLOCKTYPE_T)
#define _KCLOCKTYPE_T
typedef uint8_t kclocktype_t;
#endif /* !_KCLOCKTYPE_T */

/*
/* KI Buffer Number:*/
*/

#if !defined(_KBUFNO_T)
#define _KBUFNO_T
typedef uint32_t kbufno_t;
#endif /* !_KBUFNO_T */

#if !defined(_KCPUSTATE_T)
#define _KCPUSTATE_T
typedef unsigned char kcpustate_t;
#endif /* _KCPUSTATE_T */

#if !defined(_KRECSZ_T)
#define _KRECSZ_T
typedef unsigned short krecsz_t;
#endif /* _KRECSZ_T */

#if !defined(_KRECID_T)
#define _KRECID_T
typedef unsigned short krecid_t;
#endif /* _KRECID_T */

#if !defined(_KSEQCNT_T)
#define _KSEQCNT_T
typedef uint32_t  kseqcnt_t;
#endif /* _KSEQCNT_T */

#if !defined(_KSPU_T)
#define _KSPU_T
typedef unsigned short kspu_t;
#endif /* _KSPU_T */

#if !defined(_KFLAG_T)
#define _KFLAG_T
typedef uint32_t kflag_t;
#endif /* _KFLAG_T */

#if !defined(_KSTATE_T)
/* 
/*    sorry for the naming convention but kstat_t is already defined in */
/*    onc+sys/kstat.h.*/
*/
#define _KSTATE_T
typedef uint8_t kstate_t;
#endif /* _KSTATE_T */

#if !defined(_KNICE_T)
#define _KNICE_T
typedef uint8_t knice_t;
#endif /* _KNICE_T */

#if !defined(_KCPU_T)
#define _KCPU_T
typedef uint8_t kcpu_t;
#endif /* _KCPU_T */

#if !defined(_KPRI_T)
#define _KPRI_T
typedef uint16_t kpri_t;
#endif /* _KPRI_T */

#if !defined(_KSCHEDPOLICY_T)
#define _KSCHEDPOLICY_T
typedef int32_t kschedpolicy_t;
#endif /* _KSCHEDPOLICY_T */

/*
/* KI Generic Identifier Type:*/
*/

#if !defined(_KID_T)
#define _KID_T
typedef int32_t kid_t;
#endif /* _KID_T */

/*
/* KI Generic Type:*/
*/

#if !defined(_KTYPE_T)
#define _KTYPE_T
typedef int8_t ktype_t;
#endif /* _KTYPE_T */

/*
/* KI File Type:*/
*/

#if !defined(_KFTYPE_T)
#define _KFTYPE_T
typedef int16_t kftype_t;
#endif /* _KTYPE_T */

/*
/* KI Mount Type:*/
*/

#if !defined(_KMTYPE_T)
#define _KMTYPE_T
typedef int16_t kmtype_t;
#endif /* _KMTYPE_T */

/*
/* KI Syscall Number Type:*/
*/

#if !defined(_KSYSCALLNO_T)
#define _KSYSCALLNO_T
typedef uint16_t ksyscallno_t;
#endif /* _KSYSCALLNO_T */

/*
/* KI User Type:*/
*/

#if !defined(_UTYPE_T)
#define _UTYPE_T
typedef uint32_t utrace_type_t;
#endif /* !_UTYPE_T */

#if !defined(_KPC_T)
/* 
/* Warning: The following definition works only for ILP32 and LP64.*/
*/
#define _KPC_T
typedef union _kpc {
   ulong_t  kpc_val;
#if !defined(__STDC_32_MODE__)
   uint64_t kpc_val64;
#endif
   struct kpc_t_split {
        uint32_t high;     /*upper half of kgaddr_t*/
        uint32_t low;   /*lower half of kgaddr_t*/
   } split;
} kpc_t;
#endif /* !_KPC_T */

/*
/* The following types definitions are related to registers and their size is */
/* platform dependant. */
/* The naming convention used is from the similar 'mach.800/clock.h' types.*/
*/

#if !defined(_RAWCRITCOUNT_T)
#define _RAWCRITCOUNT_T
typedef unsigned long RawCrItCount_t;
#endif /* _RAWCRITCOUNT_T */

#if !defined(_RAWCRRCCOUNT_T)
#define _RAWCRRCCOUNT_T
typedef unsigned long RawCrRcCount_t;
#endif /* _RAWCRRCCOUNT_T */

#if !defined(_RAWCRITDELTA_T)
#define _RAWCRITDELTA_T
typedef long RawCrItDelta_t;
#endif /* _RAWCRITDELTA_T */

#if !defined(_RAWCRRCDELTA_T)
#define _RAWCRRCDELTA_T
typedef long RawCrRcDelta_t;
#endif /* _RAWCRRCDELTA_T */

#if !defined(_KREGISTER_T)
#define _KREGISTER_T
typedef union _kregister {
   unsigned long   kreg_val;
   RawCrItCount_t  kreg_it;
   RawCrItDelta_t  kreg_itdelta;
   RawCrRcCount_t  kreg_rc;
#if !defined(__STDC_32_MODE__)
   uint64_t        kreg_val64;
#endif
   struct kregister_split {
        uint32_t high;     /*upper half of kgaddr_t*/
        uint32_t low;   /*lower half of kgaddr_t*/
   } split;
   uint32_t        kreg_val32;
} kregister_t;
#endif /* _KREGISTER_T */

#endif /* _SYS_KI_DEFS_INCLUDED */
