/* @(#) netdiag1.h $Revision: 1.13.106.2 $ $Date: 97/07/12 22:16:08 $ */

#ifndef _SYS_NETDIAG1_INCLUDED
#define _SYS_NETDIAG1_INCLUDED

/* This include file is for the netdiag1 tracing/logging psuedo driver. */

#ifdef _KERNEL
#include "../h/time.h"
#include "../h/subsys_id.h"
#include "../h/spinlock.h"
#include "../h/net_diag.h"
#else  /* _KERNEL  */
#include <sys/time.h>
#include <subsys_id.h>
#include <net_diag.h>
#endif /* _KERNEL */

#ifdef _KERNEL
#include "../h/types.h"
#else
#include <sys/types.h>  /* uid_t is defined in this file */
#endif /* _KERNEL */

#ifndef TRUE
#define TRUE		1 
#endif

#ifndef FALSE
#define FALSE		0 
#endif 

#ifndef NULL
		#undef NULL
#define NULL		0
#endif


/* 
 * Defines for the ioctl commands  
 */
#define NETD_GET_STATUS		_IOWR('D',1, dm_mask_t_test)
#define NETD_SET_MASK		_IOW('D',2, dm_mask_t)
#define NETD_RESET_MASK		_IOW('D',3, rs_masks)
#define NETD_SET_BUFSIZE	_IOWR('D',4, int)
#define NETD_NEW_BUF		_IOW('D',5, int)
#define NETD_STOP		_IOW('D',6, int)
#define NETD_START		_IOW('D',7, int)
#define NETD_GET_DEV_SIZE	_IOWR('D',8, dm_mask_t) 
#define NETD_GET_DEV_NAMES	_IOWR('D',9, dm_mask_t) 
#define NETD_SET_DEVICE		_IOWR('D',10, dm_mask_t)

#if defined(__LP64__) && defined(_KERNEL)

#define _NETD_GET_STATUS_32     _IOWR('D',1, _dm_mask_t_test_32)
#define _NETD_SET_MASK_32       _IOW('D',2, _dm_mask_t_32)
#define _NETD_RESET_MASK_32     _IOW('D',3, _rs_masks_32)
#define _NETD_GET_DEV_SIZE_32   _IOWR('D',8, _dm_mask_t_32)
#define _NETD_GET_DEV_NAMES_32  _IOWR('D',9, _dm_mask_t_32)
#define _NETD_SET_DEVICE_32     _IOWR('D',10, _dm_mask_t_32)

/* dm_masks_t */
#define dm_masks_t_to_dm_masks_t_32(dm_masks, dm_masks32) { \
	bcopy((caddr_t)(&dm_masks), (caddr_t)(dm_masks32), sizeof(_dm_mask_t_32)); \
	(dm_masks32)->device_name	= (ptr32_t)(dm_masks.device_name); }

#define dm_masks_t_32_to_dm_masks_t(dm_masks32, dm_masks) { \
	bcopy((caddr_t)(dm_masks32), (caddr_t)(&dm_masks), sizeof(_dm_mask_t_32)); \
	dm_masks.device_name 		= (char *)((dm_masks32)->device_name); }

/* rs_masks */
#define rs_masks_to_rs_masks_32(rs_masks, rs_masks32) { \
	(rs_masks32)->subsys_id 	= rs_masks.subsys_id; \
	(rs_masks32)->rs_filters 	= rs_masks.rs_filters; \
	(rs_masks32)->device_id 	= rs_masks.device_id; \
	(rs_masks32)->device_name 	= (ptr32_t)(rs_masks.device_name); \
	(rs_masks32)->size 			= rs_masks.size; }

#define rs_masks_32_to_rs_masks(rs_masks32, rs_masks) { \
	rs_masks.subsys_id 			= (rs_masks32)->subsys_id; \
	rs_masks.rs_filters 		= (rs_masks32)->rs_filters; \
	rs_masks.device_id 			= (rs_masks32)->device_id; \
	rs_masks.device_name 		= (char *)((rs_masks32)->device_name); \
	rs_masks.size 				= (rs_masks32)->size; }


#endif

/* 
 * Miscellaneous constants
 */
#define NETDIAG_MAX_IOVECS	20	/* Max allowable vectors    */
#define	NETDIAG_Q_DF		200	/* default for que pointers */
#define	NETDIAG_MAX_Q		1000	/* maximum for que pointers */

#define	NETDIAG_EMPTY		0
#define	NETDIAG_OK		1
#define	NETDIAG_STOP		-1

#define NETDIAG_ON		1
#define NETDIAG_OFF		0

#define NO_QUES			2	/* 0-1, Number of diag queues	*/
#define NET_TRACE		0	/* minor number for Tracing	*/
#define NET_LOG			1	/* minor number for Logging	*/


#define QUE_FULL		2
#define QUE_EMPTY		0
#define QUE_OK			1

#define MAX_T_LEN		2000	/* max number for the -m option */


#define DIAG_OPEN_STATE		3 /* opened by the reader (nktl daemon) */
#define DIAG_ACTIVE_STATE	4 /* opened and ready to read */
#define DIAG_CLOSED_STATE	5 /* no reader has the queue opened */
#define DIAG_CLOSING_STATE	6 /* in the process of closing a device */

/* Open Flags for Netdiag device files */
#define DIAG_IOCTL		O_WRONLY  /* flag for a process not reading */
#define DIAG_READ		O_RDONLY  /* flag for a process doing exclusive reads  */
#define DIAG_CHECK		(O_WRONLY | O_APPEND) /* check if t/l daemon exists */
#define DIAG_STATUS		(O_RDONLY | O_NDELAY) /* allow non_super user to get status
						       * from the kernel.		 */

/*
 * Data Structures
 */

/* This defines what the message looks like when it is stored in the	*/
/* queue. It has a length and a caddr_t base.				*/
typedef struct {
     int     len;
     caddr_t base;
} ktl_buf_t;

/*-------------------  queues for tracing/logging -----------------------*/
/* Tracing and logging each have their own queues.  Each is indexed by	 */
/* the device minor number which is 0 for tracing and 1 for logging.	 */
/*-----------------------------------------------------------------------*/
typedef struct {
  int	  que_head;		/* msg queue head			 */
  int	  que_tail;		/* msg queue tail			 */
  int	  que_state;		/* diag device opened			 */
  int	  que_msgs_qued;	/* number of msgs qued ctr		 */
  int	  que_size;		/* the que size requested (in caddr_t's) */
  int	  que_high_water_mark;	/* high water mark. (actual queue size)	 */
  u_int	  que_msgs_dropped;	/* number of msgs dropped		 */
  u_int	  que_msgs_captured;	/* total number of messages captured	 */
  int	  que_kill;		/* determines if que should be dealloc'd */
  int	  que_sleep;		/* determines if any proc is sleeping	 */
  caddr_t que_save_msg;		/* saved msg from an incomplete read	 */
  caddr_t *que_start;		/* ptr to queue of ptrs to messages	 */
} que_elements_t;


/*		Trace and log messages, header and packet	*/
/* the pseudo driver uses this struct to send to the ktl daemon */
/* with a netdiag_read()					*/
typedef struct {
	short		subsys_id;
	int		kind;
	int		class;
	unsigned short	log_instance;	
	uid_t		uid;
	int		dev_id;
#ifdef _KERNEL
	int		tid;	
#else  /* _KERNEL */
	int		pid;		
#endif /* _KERNEL */
	int		rpr;		/* <- obsolete? */
	int		cm_path_id;
	int		packetlen;
	int		tracedlen;
#if defined(__LP64__)
	struct timeval_new time;
#else 
	struct timeval time;
#endif
	unsigned int	dropped_packets;
	char		pkt;
} ktl_msg_hdr_type;

/*		Trace and log filters				 */
/* Used by the daemon and controller for sending filters to the pseudo driver*/
/* This structure is also used by the driver to return card name list to the */
/* controller								     */
typedef struct {
	short		subsys_id;
	int		capture;
	int		kind;
	int		class;
	int		device_id;
	int		t_len;
	int		size;
	uid_t		uid;
	short		number_of_cards;  /* kernel will return to the ioctl
					   * caller how many cards are being
					   * traced.
					  */
	int		firmlog_level_or_mask;	/* used by NS_LS_X25 only 
						 * When ntl_control is storing
						 * the name in the kernel,
						 * it carries the firmlog level.
						 * When kernel is returning
						 * the card name list on a
						 * querry, such as status
						 * report, it contains a mask
						 * as follows:
						 * Bit 0 represents the level of 1st
						 * card name, bit 1 is for the 2nd
						 * card.  A 1 means level 2, 0 is level
						 * 1.
						*/
	char		*device_name;	  /* ptr to a card name */
} dm_mask_t;

#if defined(__LP64__) && defined(_KERNEL)

/*		Trace and log filters				 */
/* Used by the daemon and controller for sending filters to the pseudo driver*/
/* This structure is also used by the driver to return card name list to the */
/* controller								     */
typedef struct {
	short		subsys_id;
	int		capture;
	int		kind;
	int		class;
	int		device_id;
	int		t_len;
	int		size;
	uid_t		uid;
	short		number_of_cards;  /* kernel will return to the ioctl
					   * caller how many cards are being
					   * traced.
					  */
	int		firmlog_level_or_mask;	/* used by NS_LS_X25 only 
						 * When ntl_control is storing
						 * the name in the kernel,
						 * it carries the firmlog level.
						 * When kernel is returning
						 * the card name list on a
						 * querry, such as status
						 * report, it contains a mask
						 * as follows:
						 * Bit 0 represents the level of 1st
						 * card name, bit 1 is for the 2nd
						 * card.  A 1 means level 2, 0 is level
						 * 1.
						*/
	ptr32_t		device_name;	  /* ptr to a card name */
} _dm_mask_t_32;

#endif


/*  Used by the controller, pseudo driver and daemon to reset masks */
typedef struct {
	short		subsys_id;
	int		rs_filters;
	int		device_id;	/* device Id to be deleted */
	char		*device_name;	/* ptr to card name to be deleted */
	int		size;		/* len of card name to be deleted */
} rs_masks;

#if defined(__LP64__) && defined(_KERNEL)

/*  Used by the controller, pseudo driver and daemon to reset masks */
typedef struct {
	short		subsys_id;
	int		rs_filters;
	int		device_id;	/* device Id to be deleted */
	ptr32_t		device_name;	/* ptr to card name to be deleted */
	int		size;		/* len of card name to be deleted */
} _rs_masks_32;

#endif

/* Used by the test utilties, such as ktl_stat.c and testk.c for sending filters to the pseudo driver	 */
typedef struct {
	short		subsys_id;
	int		capture;
	int		kind;
	int		class;
	int		device_id_mask[8];  /* card instances mask */
	int		t_len;
	int		size;
	uid_t		uid;
	short		number_of_cards;  /* kernel will return to the ioctl
					   * caller how cards are being 
					   * traced.
					  */
	int		firmlog_level_or_mask;	/* used by NS_LS_X25 only 
						 * When ntl_control is storing
						 * the name in the kernel,
						 * it carries the firmlog level.
						 * When kernel is returning
						 * the card name list on a
						 * querry, such as status
						 * report, it contains a mask
						 * as follows:
						 * Bit 0 represents the level of 1st
						 * card name, bit 1 is for the 2nd
						 * card.  A 1 means level 2, 0 is level
						 * 1.
						*/
	char		*device_name;	  /* ptr to a card name */
} dm_mask_t_test;

#if defined(__LP64__) && defined(_KERNEL)

/* Used by the test utilties, such as ktl_stat.c and testk.c for sending filters to the pseudo driver	 */
typedef struct {
	short		subsys_id;
	int		capture;
	int		kind;
	int		class;
	int		device_id_mask[8];  /* card instances mask */
	int		t_len;
	int		size;
	uid_t		uid;
	short		number_of_cards;  /* kernel will return to the ioctl
					   * caller how cards are being 
					   * traced.
					  */
	int		firmlog_level_or_mask;	/* used by NS_LS_X25 only 
						 * When ntl_control is storing
						 * the name in the kernel,
						 * it carries the firmlog level.
						 * When kernel is returning
						 * the card name list on a
						 * querry, such as status
						 * report, it contains a mask
						 * as follows:
						 * Bit 0 represents the level of 1st
						 * card name, bit 1 is for the 2nd
						 * card.  A 1 means level 2, 0 is level
						 * 1.
						*/
	ptr32_t		device_name;	  /* ptr to a card name */
} _dm_mask_t_test_32;

#endif

/* definitions for default values */
#define	 DF_CLASS		0
#define	 DF_KIND		0
#define	 DF_UID			-1
#define	 DF_DEVICE_ID		0
#define	 DF_TRACE_LEN		-2
#define	 DF_NAME_LIST		NULL
#define	 DF_DEVICE_ID_MASK	{0, 0, 0, 0, 0, 0, 0, 0}
#define	 NETDIAG_SET_MASK		1      /* set card instance mask */
#define	 NETDIAG_RESET_MASK		2      /* reset card instance mask bit */
#define	 NETDIAG_RESET_ALL_MASK		3      /* reset all bits in the mask */
#define	 NETDIAG_SET_TO_MINUS_1		4

/* Possible duplicated definition with nui_osi.h in nktl_start.c */
#ifndef	 MAX_CARD_INSTANCE
#define	 MAX_CARD_INSTANCE	255  /* max card instance, 8 bits in minor number */	   
#endif

/* definition for bit positions for the NETD_RESET_MASK call (rs_filters) */
#define	 RS_CLASS		0x1 
#define	 RS_KIND		0x2
#define	 RS_UID			0x4
#define	 RS_DEVICE_ID		0x8
#define	 RS_TRACE_LEN		0x10
#define	 RS_SUBSYS		0x20
#define	 RS_DEVICE_NAME		0x40

#define	 RS_ALL_NET_TR		(RS_KIND|RS_DEVICE_ID|RS_TRACE_LEN|RS_SUBSYS)
#define	 RS_ALL_NET_LOG		(RS_CLASS|RS_DEVICE_ID|RS_SUBSYS)

#ifdef _KERNEL
extern que_elements_t	netdiag_ques[];

#  ifdef _PROTOTYPES
        extern void netdiag_send_msg(int, ktl_buf_t *);
#  else /* not _PROTOTYPES */
        extern void netdiag_send_msg();
#  endif /* not _PROTOTYPES */
#endif /* _KERNEL */

/*
 * Convenience macros
 *	Note (_size is not used yet)
 */
#ifdef _KERNEL
#define LOG_MSG_DROPPED(_size) \
{ LOG_QUE_M_SPINLOCK; \
  netdiag_ques[NET_LOG].que_msgs_dropped++; \
  LOG_QUE_M_SPINUNLOCK; }

#define TRACE_MSG_DROPPED(_size) \
{ TRACE_QUE_M_SPINLOCK; \
  netdiag_ques[NET_TRACE].que_msgs_dropped++; \
  TRACE_QUE_M_SPINUNLOCK; }
#endif /* _KERNEL */

/*
 * Spinlock Macros
 *                      "_R_" == reference the resource
 *                      "_M_" == modify the resource
 *			(These are the same for first release)
 *
 * There are 4 sinlocks used to protect resources:
 *	log_mask_lock		NETDIAG_MASK_ORDER
 *	trace_mask_lock		NETDIAG_MASK_ORDER
 *	log_que_lock		NETDIAG_QUE_ORDER
 *	trace_que_lock		NETDIAG_QUE_ORDER
 *
 */

#ifdef _KERNEL
extern lock_t *log_mask_lock;
#define LOG_MASK_R_SPINLOCK(_ss,_ctxt)	 { SPINLOCK_USAV(log_mask_lock, _ctxt); }
#define LOG_MASK_R_SPINUNLOCK(_ss,_ctxt) { SPINUNLOCK_USAV(log_mask_lock, _ctxt); }
#else  /* _KERNEL */
#define LOG_MASK_R_SPINLOCK(_ss,_ctxt)	 { ; } /* No spinlocks in user-space */
#define LOG_MASK_R_SPINUNLOCK(_ss,_ctxt) { ; } /* No spinlocks in user-space */
#endif /* _KERNEL */
#define LOG_MASK_M_SPINLOCK(_ss,_ctxt)	 { LOG_MASK_R_SPINLOCK(_ss,_ctxt); }
#define LOG_MASK_M_SPINUNLOCK(_ss,_ctxt) { LOG_MASK_R_SPINUNLOCK(_ss,_ctxt); }

#ifdef _KERNEL
#define LOG_INSTANCE_SPINLOCK(_ss,_ctxt)	{ LOG_MASK_M_SPINLOCK(_ss,_ctxt); }
#define LOG_INSTANCE_SPINUNLOCK(_ss,_ctxt)	{ LOG_MASK_M_SPINUNLOCK(_ss,_ctxt); }
#else  /* _KERNEL */
#define LOG_INSTANCE_SPINLOCK(_ss,_ctxt)	{ ; } /* No spinlocks in user-space */
#define LOG_INSTANCE_SPINUNLOCK(_ss,_ctxt)	{ ; } /* No spinlocks in user-space */
#endif /* _KERNEL */

#ifdef _KERNEL
extern lock_t *trace_mask_lock;
#define TRACE_MASK_R_SPINLOCK(_ss,_ctxt)   { SPINLOCK_USAV(trace_mask_lock, _ctxt); }
#define TRACE_MASK_R_SPINUNLOCK(_ss,_ctxt) { SPINUNLOCK_USAV(trace_mask_lock, _ctxt); }
#else  /* _KERNEL */
#define TRACE_MASK_R_SPINLOCK(_ss,_ctxt)   { ; } /* No spinlocks in user-space */
#define TRACE_MASK_R_SPINUNLOCK(_ss,_ctxt) { ; } /* No spinlocks in user-space */
#endif /* _KERNEL */
#define TRACE_MASK_M_SPINLOCK(_ss,_ctxt)   { TRACE_MASK_R_SPINLOCK(_ss,_ctxt); }
#define TRACE_MASK_M_SPINUNLOCK(_ss,_ctxt) { TRACE_MASK_R_SPINUNLOCK(_ss,_ctxt); }

#define MASK_R_SPINLOCK(_queue,_ss,_ctxt)  { if (_queue == NET_LOG) \
						 LOG_MASK_R_SPINLOCK(_ss,_ctxt) \
					     else TRACE_MASK_R_SPINLOCK(_ss,_ctxt) }
#define MASK_R_SPINUNLOCK(_queue,_ss,_ctxt) { if (_queue == NET_LOG) \
						  LOG_MASK_R_SPINUNLOCK(_ss,_ctxt) \
					      else TRACE_MASK_R_SPINUNLOCK(_ss,_ctxt) }

#define MASK_M_SPINLOCK(_queue,_ss,_ctxt)   { MASK_R_SPINLOCK(_queue,_ss,_ctxt) }
#define MASK_M_SPINUNLOCK(_queue,_ss,_ctxt) { MASK_R_SPINUNLOCK(_queue,_ss,_ctxt) }

#ifdef _KERNEL
extern lock_t *log_que_lock;
#define LOG_QUE_R_SPINLOCK		{ spinlock(log_que_lock); }
#define LOG_QUE_R_SPINUNLOCK		{ spinunlock(log_que_lock); }
#else  /* _KERNEL */
#define LOG_QUE_R_SPINLOCK		{ ; } /* No spinlocks in user-space */
#define LOG_QUE_R_SPINUNLOCK		{ ; } /* No spinlocks in user-space */
#endif /* _KERNEL */
#define LOG_QUE_M_SPINLOCK		{ LOG_QUE_R_SPINLOCK; }
#define LOG_QUE_M_SPINUNLOCK		{ LOG_QUE_R_SPINUNLOCK; }

#ifdef _KERNEL
extern lock_t *trace_que_lock;
#define TRACE_QUE_R_SPINLOCK		{ spinlock(trace_que_lock); }
#define TRACE_QUE_R_SPINUNLOCK		{ spinunlock(trace_que_lock); }
#else  /* _KERNEL */
#define TRACE_QUE_R_SPINLOCK		{ ; } /* No spinlocks in user-space */
#define TRACE_QUE_R_SPINUNLOCK		{ ; } /* No spinlocks in user-space */
#endif /* _KERNEL */
#define TRACE_QUE_M_SPINLOCK		{ TRACE_QUE_R_SPINLOCK;	}
#define TRACE_QUE_M_SPINUNLOCK		{ TRACE_QUE_R_SPINUNLOCK; }

#define QUE_R_SPINLOCK(_queue)		{ if (_queue == NET_LOG) \
					      LOG_QUE_R_SPINLOCK \
					 else TRACE_QUE_R_SPINLOCK }
#define QUE_R_SPINUNLOCK(_queue)	{ if (_queue == NET_LOG) \
					      LOG_QUE_R_SPINUNLOCK \
					 else TRACE_QUE_R_SPINUNLOCK }

#define QUE_M_SPINLOCK(_queue)		{ QUE_R_SPINLOCK(_queue) }
#define QUE_M_SPINUNLOCK(_queue)	{ QUE_R_SPINUNLOCK(_queue) }

#ifdef _KERNEL
#define QUE_R_SLEEP(_chan, _pri, _queue) { if (_queue == NET_LOG) \
                                               sleep_spinunlock(_chan, _pri, log_que_lock); \
                                           else sleep_spinunlock(_chan, _pri, trace_que_lock); }
#define QUE_M_SLEEP(_chan, _pri, _queue) { QUE_R_SLEEP(_chan, _pri, _queue); }
#else  /* _KERNEL */
#define QUE_R_SLEEP(_chan, _pri, _queue) { ; } /* No spinlocks in user-space */
#define QUE_M_SLEEP(_chan, _pri, _queue) { ; } /* No spinlocks in user-space */
#endif /* _KERNEL */

#ifdef _lint
extern struct user u;       /* required for k_tid() in net_diag.c. 
			     * Part of 64bit clean 
			     *  - shobha, 23th Sep 1996.
			     */
#endif

#  ifdef _PROTOTYPES
extern struct timeval ms_gettimeofday(void);
#  else
extern struct timeval ms_gettimeofday();
#  endif

#endif /* _SYS_NETDIAG1_INCLUDED */
