/*
 * @(#)sio_drv.h: $Revision: 1.10.106.2 $ $Date: 97/07/12 23:44:56 $
 * $Locker: CRT $
 */

/****************************************************************************
** Allow multiple inclusion of this header file
****************************************************************************/

#ifndef _SIO_HDR_INCLUDED
#define _SIO_HDR_INCLUDED

/****************************************************************************
** Include files
****************************************************************************/

#ifdef _KERNEL_BUILD
#include "../h/kern_svcs.h"	/* for prototypes */
#include "../h/types.h"		/* for dev_t */
#include "../h/conf.h"		/* for drv_info_t & drv_ops_t */
#include "../h/mp.h"		/* for uniprocessor */
#include "../h/buf.h"		/* for struct buf */
#include "../io/dma.h"		/* for io_map_cntl_t */
#include "../sio/iotree.h"	/* for port_num_type */
#include "../sio/llio.h"	/* for probe_req_type */
#else   /* ! _KERNEL_BUILD */
#include <sys/kern_svcs.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/mp.h>
#include <sys/buf.h>
#include <sys/dma.h>
#include <sio/iotree.h>
#include <sio/llio.h>
#endif /* _KERNEL_BUILD */

#ifndef NULL
		#undef NULL
#define NULL 0L
#endif

#ifdef _KERNEL

/****************************************************************************
** The driver header is used in sio_install_driver(SIO2) to describe
** an SIO device driver.  Configuration information previously contained in
** /etc/master is included in these structures.  The driver header must be
** included in every SIO driver.  The driver header is defined by the
** following structures:
******************************************************************************/

typedef struct add_child {          /* add child information */
    char            *name;          /* name of add child driver ("disc3") */
    int             (*install) __((void));
} add_child_t;

typedef struct overlay {            /* overlay information */
    char            *top;           /* top name of overlay top ("disc2") */
    char            *base;          /* base name of overlay base ("hpfl0") */
} overlay_t;

typedef int (*sio_idy_t) __((ent_id_type ent_id));

typedef void (*sio_server_entry_t) __((void *pda, void *msg, int msg_length,
	io_subq_type subq_num, set_of_32 *subq_mask, boolean *return_frame));

typedef int (*sio_attach_t) __((void));

typedef int (*sio_init_t) __((void));

typedef int (*sio_get_minor_t) __((void *pda, char *option, int *minor_num));

typedef int (*sio_get_port_t) __((dev_t dev, port_num_type *port_num));

typedef struct drv_data {           /* SIO specific information */
    char            *module_name;   /* module name ("disc3") */
    unsigned int    type;           /* type of module: device, interface, bus */
    unsigned int    flag;           /* flag for SIO information */
    unsigned int    mask;           /* mask addressing portions of dev_t */
    char            **parents;      /* ptr to array of parent names */
    add_child_t     *add_child;     /* ptr to array of add child structures */
    overlay_t       *overlay;       /* ptr to the overlay structure */
    sio_idy_t       idy;            /* ptr to identify func ("disc3_idy") */
    sio_server_entry_t entry;       /* ptr to port server entry point */
    sio_attach_t    attach;         /* ptr to the driver attach func */
    sio_init_t      init;           /* ptr to the driver initialization func */
    sio_get_minor_t get_minor;      /* ptr to the driver build minor func */
    sio_get_port_t  get_port;       /* ptr to the driver get port num func */
} drv_data_t;

typedef struct sio_info {           /* SIO driver information */
    drv_ops_t       *sio_drv_ops;   /* ptr to struct of driver entry points */
    drv_info_t      *sio_drv_info;  /* ptr to struct of driver information */
    drv_data_t      *sio_drv_data;  /* ptr to struct of SIO specific info */
} sio_info_t;

/****************************************************************************
** SIO flag constants
******************************************************************************/

#define SIO_NO_MGR            0x1   /* hardware without software (target) */
#define SIO_BOUND             0x2   /* at least one bound manager */
#define SIO_INIT              0x4   /* call init routine even if not bound */
#define SIO_LDM               0x8   /* the driver is a LDM */
#define SIO_INITED            0x10  /* driver init routine already called */
#define SIO_INSTR             0x20  /* the driver is instr0 */

/****************************************************************************
** Mask constants
******************************************************************************/

#define IO_AUTOX_MASK         0x00fffe00
#define IO_CENT_MASK          0x00ff0000
#define IO_DISK_MASK          0x00ffff00
#define IO_GPIO_MASK          0x00ff0000
#define IO_INSTR_MASK         0x00ff0000
#define IO_LAN_MASK           0x00ff0000
#define IO_PRINT_MASK         0x00ffff00
#define IO_SCC_MASK           0x00ff0000
#define IO_SERIAL_MASK        0x00ff0000
#define IO_TAPE_MASK          0x00ffff00
#define IO_TTY_MASK           0x00fff000
#define IO_NO_MASK            0x00000000

/****************************************************************************
** Macro gets a token that the kernel instrumentation routines use to 
** generate a handle.  For now (and probably forever) just use a pointer
** to the I/O tree node.
****************************************************************************/
#define sio_get_id( port_num) sio_port_to_node( port_num)

/****************************************************************************
** Macro to reassign EIM 
****************************************************************************/
#define sio_get_new_eim(Port_num,EIM_control_var,Default_eim) \
        (*EIM_control_var ? 0 : \
                (*EIM_control_var=sio_reassign_eim_mask(Port_num,Default_eim)))

/****************************************************************************
** active request pointer array macros
****************************************************************************/

/****************************************************************************
** Interface:
**     SIO_RESIZE_REQ_ARRAY( align, ptr_t, Array, Max)
**
** Parameters:
**     align  (Input)        io_get_mem alignment of request pointer array
**     ptr_t  (Input)        request pointer type
**     Array  (Input/Output) start of request pointer array (request **)
**     Max    (Input/Output) max num elements in request pointer array
**
** Assumptions:
**     The request pointer array has already been allocated using io_get_mem
**     with NONEQUIV_MEM_TYPE.
**
** Description:
**     The maximum number of active request pointers have been stored in the
**     array.  Allocate a new request pointer array, copy the old one into it
**     and deallocatte the old request pointer array.
****************************************************************************/

#define SIO_REQ_CLUMP_SIZE 64

#define SIO_RESIZE_REQ_ARRAY( align, ptr_t, Array, Max)                    \
{                                                                          \
	ptr_t    *sio_req_old;                                             \
	ptr_t    *sio_req_new;                                             \
	size_t   sio_req_old_size;                                         \
	size_t   sio_req_new_size;                                         \
	mem_type sio_req_mem;                                              \
                                                                           \
	sio_req_old_size = Max * sizeof(ptr_t);                            \
	sio_req_new_size =                                                 \
		((size_t) (unsigned int) (Max + SIO_REQ_CLUMP_SIZE)) *     \
		sizeof(ptr_t);                                             \
	sio_req_mem.num_bytes   = sio_req_new_size;                        \
	sio_req_mem.alignment   = align;                                   \
	sio_req_mem.type_object = NONEQUIV_MEM_TYPE;                       \
	sio_req_new = (ptr_t *) io_get_mem( sio_req_mem);                  \
	bzero( (caddr_t) sio_req_new, sio_req_new_size);                   \
                                                                           \
	sio_req_old = Array;                                               \
	(void) bcopy(                                                      \
		(caddr_t) sio_req_old,                                     \
		(caddr_t) sio_req_new,                                     \
		sio_req_old_size                                           \
	);                                                                 \
	Array = sio_req_new;                                               \
	Max += SIO_REQ_CLUMP_SIZE;                                         \
                                                                           \
	sio_req_mem.num_bytes   = sio_req_old_size;                        \
	sio_req_mem.alignment   = align;                                   \
	sio_req_mem.type_object = NONEQUIV_MEM_TYPE;                       \
	io_rel_mem( (void *) sio_req_old, sio_req_mem);                    \
}

/****************************************************************************
** Interface:
**     SIO_GET_REQ_INDEX( index, req_ptr, align, ptr_t, Array, Max, Last)
**
** Parameters:
**     index    (Output)       index into the request pointer array
**     req_ptr  (Input)        pointer to the request (request *)
**     align    (Input)        io_get_mem alignment of request pointer array
**     ptr_t    (Input)        request pointer type
**     Array    (Input/Output) start of request pointer array (request **)
**     Max      (Input/Output) max num elements in request pointer array
**     Last     (Input/Output) last used element in request pointer array
**
**     The globals (Array, Max and Last) are input parameters, but they may
**     change and become output paramters if an array resize is necessary.
**     These parameters should be in the port data area.
**
** Assumptions:
**     1. The request pointer array has been allocated using io_get_mem
**        with NONEQUIV_MEM_TYPE.
**     2. The Last parameter has been initialized to 0
**     3. The Max parameter has been initialized to the maximum number of
**        elements in the array.
**
** Description:
**     Save a pointer to an active request in an array of pointers.
**     The main idea is the user sends in a pointer to an active request and
**     gets back an index into an array.  If necessary, the request pointer
**     array will be resized.  This macro is designed to be used like this:
**
**        1. Invoke SIO_GET_REQ_INDEX when the active requests are allocated.
**        2. Save the index in the active request structure.
**        3. When an IO request message is sent, get the index from the 
**           active request and store it in the message ID.
**        4. When the IO reply message comes back, use the message ID
**           as an index into the request pointer array in order to get
**           the active request pointer.
**
**     Only steps 3 and 4 should be in the performance path.
**     The following diagram attempts to show the relationship between
**     the active request structure and the active request pointer array.
**        
**          active req                           active req ptr array
**         +-----------+                           +-------------+
**         |           |                       [0] |             |
**         |           |                           |             |
**         |           |                           |             |
**         +-----------+                           +-------------+
**         |   index   |                   [index] | act req ptr |
**         +-----------+                           +-------------+
**         |           |                           |             |
**         |           |                           |             |
**         |           |  [n * SIO_REQ_CLUMP_SIZE] |             |
**         +-----------+                           +-------------+
**
** Example:
**
**     typedef struct {
**         ...
**         int index;
**         ...
**     } my_active_req_type;
**
**     typedef struct {
**         ...
**         my_active_req_type **req_array;
**         int                req_array_max;
**         int                req_array_last;
**         ...
**     } my_pda_type;
**
**     my_do_bind_routine( pda, msg)
**         ...
**         mem.type_object     = NONEQUIV_MEM_TYPE;
**         mem.alignment       = 5;
**         mem.num_bytes       = sizeof(my_active_req_type *) * SIO_REQ_CLUMP_SIZE;
**         pda->req_array      = (my_active_req_type **) io_get_mem(mem);
**         pda->req_array_max  = SIO_REQ_CLUMP_SIZE;
**         pda->req_array_last = 0;
**         bzero( (caddr_t) pda->req_array, (size_t) mem.num_bytes);
**
**     my_create_active_req_routine( pda)
**         ...
**         mem.alignment   = 5;
**         mem.type_object = NONEQUIV_MEM_TYPE;
**         mem.num_bytes   = sizeof(my_active_req_type);
**         active_req = (my_active_req_type *) io_get_mem(mem);
**         SIO_GET_REQ_INDEX(
**             active_req->index,
**             active_req,
**             5,
**             my_active_req_type *,
**             pda->req_array,
**             pda->req_array_max,
**             pda->req_array_last
**         );
**
**     my_send_io_request( pda, msg_out, active_req)
**         ...
**         io_get_trn( & (msg_out->TNUM));
**         msg_out->MID = active_req->index;
**         ...
**         io_send( pda->dam_port, pda->dam_low_subq, msg_out, MSG_SIZE);
**
**     my_get_io_reply( pda, msg_in)
**         active_req = pda->req_array[msg_in->MID];
**         ...
****************************************************************************/
 
#define SIO_GET_REQ_INDEX( index, req_ptr, align, ptr_t, Array, Max, Last) \
{                                                                          \
	int xn;                                                            \
	for (xn = Last ; ((xn < Max) && (Array[xn] != NULL)) ; xn++) ;     \
	index = Last = xn;                                                 \
	if (xn == Max) {                                                   \
		SIO_RESIZE_REQ_ARRAY( align, ptr_t, Array, Max)            \
	}                                                                  \
	Array[xn] = req_ptr;                                               \
}

/****************************************************************************
** Interface:
**     SIO_FREE_REQ_INDEX( index, Array, Last)
**
** Parameters:
**     index  (Input)  index into the request pointer array
**     Array  (Input)  start of request pointer array (request **)
**     Last   (Input)  last used element in request pointer array
**
** Description:
**     Give back the active request pointer index by storing NULL into
**     the request pointer array.  If necessary change the last element used.
**     Invoke SIO_FREE_REQ_INDEX when the active requests are deallocated.
****************************************************************************/
 
#define SIO_FREE_REQ_INDEX( index, Array, Last)                            \
	Array[index] = NULL;                                               \
	if (index < Last) {                                                \
		Last = index;                                              \
	}

/****************************************************************************
** SIO versions of mapping services
******************************************************************************/

#define SIO_MAP_FUNC(x) ((io_map_cntl_t *)sio_get_map_funcs(x))

#define sio_dma_alloc(Port_num,Iova) \
	(*SIO_MAP_FUNC(Port_num)->dma_alloc)(SIO_MAP_FUNC(Port_num)->arg,Iova)

#define sio_f_dma_alloc(Fn,Iova) \
	(*Fn->dma_alloc)(Fn->arg,Iova)

#define sio_dma_free(Port_num,Iova,Addr) \
	(*SIO_MAP_FUNC(Port_num)->dma_free) \
		(SIO_MAP_FUNC(Port_num)->arg,Iova,Addr)

#define sio_f_dma_free(Fn,Iova,Addr)	\
	(*Fn->dma_free)(Fn->arg,Iova,Addr)

#define sio_fastmap(Port_num,Range_type,Host_range,Io_range) \
	(*SIO_MAP_FUNC(Port_num)->fastmap) \
		(SIO_MAP_FUNC(Port_num)->arg,Range_type,Host_range,Io_range)

#define sio_f_fastmap(Fn,Range_type,Host_range,Io_range)	\
	(*Fn->fastmap)(Fn->arg,Range_type,Host_range,Io_range)

#define sio_get_pva(Port_num,Iova) \
	(*SIO_MAP_FUNC(Port_num)->get_pva)(SIO_MAP_FUNC(Port_num)->arg,Iova)

#define sio_f_get_pva(Fn,Iova)	\
	(*Fn->get_pva)(Fn->arg,Iova)

#define sio_map(Port_num,Map_cb,Hints,Range_type,Host_range,Io_range) \
	((Hints & IO_CONTIGUOUS) == 0 ? \
		(*SIO_MAP_FUNC(Port_num)->map)(SIO_MAP_FUNC(Port_num)->arg, \
			Map_cb,Hints,Range_type,Host_range,Io_range) :      \
		(*SIO_MAP_FUNC(Port_num)->map_contiguous) \
			(SIO_MAP_FUNC(Port_num)->arg, \
			Map_cb,Hints,Range_type,Host_range,Io_range) )
#define sio_f_map(Fn,Map_cb,Hints,Range_type,Host_range,Io_range) \
	((Hints & IO_CONTIGUOUS) == 0 ? \
	  (*Fn->map)(Fn->arg,Map_cb,Hints,Range_type,Host_range,Io_range): \
	  (*Fn->map_contiguous)(Fn->arg,Map_cb,Hints,Range_type, \
	  Host_range, Io_range) )

#define sio_remap(Port_num,Range_type,Host_range,Io_range) \
	(*SIO_MAP_FUNC(Port_num)->remap) \
		(SIO_MAP_FUNC(Port_num)->arg,Range_type,Host_range,Io_range)

#define sio_f_remap(Fn,Range_type,Host_range,Io_range) \
	(*Fn->remap)(Fn->arg,Range_type,Host_range,Io_range) 

#define SIO_MAP_FUNC_PTR(x) ((io_map_cntl_t **)sio_get_map_funcs_ptr(x))
#define sio_set_attributes(Port_num,Attributes) { \
	(*SIO_MAP_FUNC(Port_num)->set_attributes) \
		(SIO_MAP_FUNC_PTR(Port_num), \
		 SIO_MAP_FUNC(Port_num)->arg,Attributes); }

#define sio_shared_mem_lock(Port_num) \
	(*SIO_MAP_FUNC(Port_num)->shmem_lock)(SIO_MAP_FUNC(Port_num)->shmem_arg)

#define sio_shared_mem_unlock(Port_num) \
	(*SIO_MAP_FUNC(Port_num)->shmem_unlock) \
		(SIO_MAP_FUNC(Port_num)->shmem_arg)

#define sio_unmap(Port_num,Io_range) \
	(*SIO_MAP_FUNC(Port_num)->unmap)(SIO_MAP_FUNC(Port_num)->arg,Io_range)

#define sio_f_unmap(Fn,Io_range)  \
	(*Fn->unmap)(Fn->arg,Io_range)

/****************************************************************************
** External function prototype declarations
******************************************************************************/

extern int sio_install_driver __((
	sio_info_t *header));
extern int sio_get_hw_path_str __((
	port_num_type port_number, char *hw_path_str));
extern int sio_put_desc __((
	port_num_type port_number, char *desc_str));
extern int sio_scsi_put_desc __((
	port_num_type port_num, char *vendor_id, char *product_id, char *s));
extern int sio_put_pda __((
	port_num_type port_number, void *pda, sio_info_t *header));
extern void *sio_get_pda __((
	dev_t dev, sio_info_t *header));
extern void *sio_port_to_node __((
	port_num_type port_num));
extern int sio_get_iodc __((
	int dev_inst, sio_info_t *header, void *iodc_ptr));
extern int sio_card_inst __((
	port_num_type port_num, int *card_inst));
extern void sio_inq_desc __((
	char *desc, char *inquiry_data));
extern void sio_model_desc __((
	char *desc, bit8 *model_num));
extern void sio_amigoid_desc __((
	char *desc, int amigoid));
extern int sio_replace_desc __((
	port_num_type port_number, char *desc_str));
extern int sio_identify __((
	port_num_type port_number, void **id_ptr, void **desc_ptr));
extern int sio_probe_hw_path __((
	port_num_type port_number, probe_req_type *req));
extern uint32_t sio_reassign_eim_mask __((
	port_num_type port_number, uint32_t default_eim));
extern void sio_reassign_ioforw __((
	port_num_type port_number, uint32_t eim));
extern io_map_cntl_t **sio_get_map_funcs_ptr __((
	port_num_type port_num));
extern io_map_cntl_t *sio_get_map_funcs __((
	port_num_type port_num));
extern int hwpath_to_dev __((
	int npaths, hw_path_t hwpath[], int blockdev, dev_t devno[]));

/* External function prototypes for hardware identify entry points */
extern int autox0_idy __((ent_id_type ent_id));
extern int disc1_idy __((ent_id_type ent_id));
extern int disc3_idy __((ent_id_type ent_id));
extern int disc4_idy __((ent_id_type ent_id));
extern int hpfl1_idy __((ent_id_type ent_id));
extern int hpib1_idy __((ent_id_type ent_id));
extern int lan1_idy __((ent_id_type ent_id));
extern int lan3_idy __((ent_id_type ent_id));
extern int lanmux0_idy __((ent_id_type ent_id));
extern int lantty0_idy __((ent_id_type ent_id));
extern int lpr2_idy __((ent_id_type ent_id));
extern int lpr3_idy __((ent_id_type ent_id));
extern int mux2_idy __((ent_id_type ent_id));
extern int scsi1_idy __((ent_id_type ent_id));
extern int scsi3_idy __((ent_id_type ent_id));
extern int tape1_idy __((ent_id_type ent_id));
extern int tape2_idy __((ent_id_type ent_id));
extern int target_idy __((ent_id_type ent_id));

#endif /* _KERNEL */

#endif /* _SIO_HDR_INCLUDED */
