/* $Header: wait.h,v 1.25.106.2 97/08/11 23:18:57 rven Exp $ */

#ifndef _SYS_WAIT_INCLUDED
#define _SYS_WAIT_INCLUDED

/*
 * This file holds definitions relevent to the four system calls
 * wait, waitpid, waitid, and wait3.
 */

#ifdef _KERNEL_BUILD
#include "../h/stdsyms.h"
#include "../h/types.h"
#else  /* ! _KERNEL_BUILD */
#include <sys/stdsyms.h>
#include <sys/types.h>
#endif /* _KERNEL_BUILD */

#ifdef _INCLUDE_POSIX_SOURCE

    /*
     * Option bits for the waitpid and wait3 calls.  WNOHANG causes the
     * wait to not hang if there are no stopped or terminated processes, rather
     * returning an error indication in this case (pid==0).  WUNTRACED
     * indicates that the caller should receive status about untraced children
     * which stop due to signals.  If children are stopped and a wait without
     * this option is done, it is as though they were still running... nothing
     * about them is returned. WNOWAIT causes the wait to not be registered.
     * This means that the process that has been waited on, can be waited on
     * again with identical results, provided that the status of the child
     * does not change in the meantime.
     */
#  define WNOHANG	1	/* don't hang in wait */
#  define WUNTRACED	2	/* tell about stopped, untraced children */
#  define _WNOWAIT	4	/* don't register the wait */

   /* waitpid and wait valid options */
#  ifndef _INCLUDE_XOPEN_SOURCE_EXTENDED
#    define _WAITMASK	(WNOHANG | WUNTRACED | _WNOWAIT) /* all legal flags */
#   ifdef _XPG4_EXTENDED
#    define WIFSIGNALED(_X)	((((_X)&0377)!=0)&&(((_X)&0377)!=0177))
#   else /* _XPG4_EXTENDED */
#    define WIFSIGNALED(_X)	((((_X)&0377)!=0)&&(((_X)&0377)!=0177))
#   endif /* _XPG4_EXTENDED */
#  endif /* !_INCLUDE_XOPEN_SOURCE_EXTENDED */

#  define WIFEXITED(_X)		(((_X)&0377)==0)
#  define WIFSTOPPED(_X)	(((_X)&0377)==0177)
#  define WEXITSTATUS(_X)	(((_X)>>8)&0377)
#  define WTERMSIG(_X)		((_X)&0177)
#  define WSTOPSIG(_X)		(((_X)>>8)&0377)

#  ifndef _KERNEL
#  ifdef __cplusplus
    extern "C" {
#  endif /* __cplusplus */

#  ifdef _CLASSIC_POSIX_TYPES
     extern int wait();
     extern int waitpid();
#  else /* not _CLASSIC_POSIX_TYPES */
#    ifdef _PROTOTYPES
       extern pid_t wait(int *);
       extern pid_t waitpid(pid_t, int *, int);
#    else /* not _PROTOTYPES */
       extern pid_t wait();
       extern pid_t waitpid();
#    endif /* not _PROTOTYPES */
#  endif /* not _CLASSIC_POSIX_TYPES */

#  ifdef __cplusplus
    }
#  endif /* __cplusplus */
#  endif /* not _KERNEL */

#endif /* _INCLUDE_POSIX_SOURCE */

#if defined (_INCLUDE_XOPEN_SOURCE_EXTENDED) || defined (_BSD)
#  ifdef _XPG4_EXTENDED
#    ifdef _KERNEL_BUILD
#include "../h/resource.h"
#    else  /* ! _KERNEL_BUILD */
#include <sys/resource.h> 	/* needed for extended wait3() */
#    endif /* _KERNEL_BUILD */
#  endif /* _XPG4_EXTENDED */


/* for waitpid, wait3, and waitid */
#define WNOWAIT	_WNOWAIT  /* required by COSE, but not base XPG4 */

#  define WSTOPFLG	0177	/* value of s.stopval if process is stopped */

#  ifndef _KERNEL
#    ifdef __cplusplus
      extern "C" {
#    endif /* __cplusplus */
#    ifdef _CLASSIC_ID_TYPES
 	extern int wait3();
#    else /* not _CLASSIC_ID_TYPES */
#      ifdef _PROTOTYPES
#         ifdef _XPG4_EXTENDED
           extern pid_t wait3(int *, int, struct rusage *);
#         else /* ! _XPG4_EXTENDED */
           extern pid_t wait3(int *, int, int *);
#        endif /* _XPG4_EXTENDED */
#      else /* not _PROTOTYPES */
         extern pid_t wait3();
#      endif /* not _PROTOTYPES */
#    endif /* not _CLASSIC_ID_TYPES */

#    ifdef __cplusplus
      }
#    endif /* __cplusplus */
#  endif /* not _KERNEL */
#endif	/* _INCLUDE_XOPEN_SOURCE_EXTENDED || _BSD */

#ifdef _INCLUDE_XOPEN_SOURCE_EXTENDED
#  ifdef _KERNEL_BUILD
#include "../h/types.h"
#include "../h/signal.h"
#  else  /* ! _KERNEL_BUILD */
#include <sys/types.h>		/* for waitid() */
#include <sys/signal.h>		/* for waitid() */
#  endif /* _KERNEL_BUILD */

/* additional options for waitid() */
#define WSTOPPED	0x0008
#define WCONTINUED	0x0010
#define WEXITED		0x0020

/* wait1() can now return this status's if the WCONTINUED option
 * were selected in waitid(), respectively.
 */
#define WCONTFLG  0177777  /* process was stopped and has continued */

/* waitid() valid options */
#define _WAITIDMASK	(WEXITED|WSTOPPED|WCONTINUED|WNOHANG|WNOWAIT) 
#define _WAITMASK	(WNOHANG | WUNTRACED | WNOWAIT |WCONTINUED) 

#define WLOW2BYTES 0177777

#define WIFSIGNALED(_X)	((((_X)&0377)!=0)&&(((_X)&0377)!=WSTOPFLG)\
                                              &&(((_X)&WLOW2BYTES)!=WCONTFLG))

/* macros to test return status from waitid() for continued and traced
 * conditions.
 */
#define WIFCONTINUED(_X)	(( (_X) & WLOW2BYTES ) == WCONTFLG )

/* type used by 1st parameter in waitid() */
#  ifndef _IDTYPE_T
#    define _IDTYPE_T
/*
 *  idtype_t is defined as enum so that the debugger can see the symbols.
 *
 *  idtype_t contains the set of processes that should receive the
 *  signal.
*/
typedef enum {
	P_PID,		/* process id */
	P_PGID,		/* process group id */
	P_SID,		/* session id */
	P_UID,		/* effective user id */
	P_GID,		/* effective group id */
	P_CID,		/* class id; We don't have this in HP-UX. */
	P_ALL		/* all processes */
} idtype_t;
#  endif /* _IDTYPE_T */

/* waitid() declaration */
#  ifndef _KERNEL
#    ifdef __cplusplus
      extern "C" {
#    endif /* __cplusplus */

#  ifdef _PROTOTYPES
	extern int waitid(idtype_t, id_t, siginfo_t *, int);
#  else /* !_PROTOTYPES */
	extern int waitid();
#  endif /* _PROTOTYPES */

#    ifdef __cplusplus
      }
#    endif /* __cplusplus */
#  endif

#endif /* _INCLUDE_XOPEN_SOURCE_EXTENDED */

#if defined (_INCLUDE_HPUX_SOURCE) || defined (_BSD)
/*
 * Structure of the information in the first word returned by both
 * wait and wait3.  If w_stopval==WSTOPFLG, then the second structure
 * describes the information returned, else the first.  See WUNTRACED below.
 */
   union wait	{
	int	w_status;		/* used in syscall */
	/*
	 * Terminated process status.
	 */
	struct {
		unsigned short	w_pad;		/* pad to low order 16 bits */
		unsigned int	w_Retcode:8;	/* exit code if w_termsig==0 */
		unsigned int	w_Coredump:1;	/* core dump indicator */
		unsigned int	w_Termsig:7;	/* termination signal */
	} w_T;
	/*
	 * Stopped process status.  Returned
	 * only for traced children unless requested
	 * with the WUNTRACED option bit.
	 */
	struct {
		unsigned short	w_pad;		/* pad to low order 16 bits */
		unsigned int	w_Stopsig:8;	/* signal that stopped us */
		unsigned int	w_Stopval:8;	/* == W_STOPPED if stopped */
	} w_S;
   };
#  define w_termsig	w_T.w_Termsig
#  define w_coredump	w_T.w_Coredump
#  define w_retcode	w_T.w_Retcode
#  define w_stopval	w_S.w_Stopval
#  define w_stopsig	w_S.w_Stopsig

#  define WCOREDUMP(_X)	((_X)&0200)
#endif	/* _INCLUDE_HPUX_SOURCE || _BSD */

#ifdef _BSD
#  undef  WIFEXITED
#  define WIFEXITED(_X)		((_X).w_stopval != WSTOPFLG && (_X).w_termsig == 0)
#  undef  WIFSTOPPED
#  define WIFSTOPPED(_X)	((_X).w_stopval == WSTOPFLG)
#  undef  WIFSIGNALED
#  define WIFSIGNALED(_X)	((_X).w_stopval != WSTOPFLG && (_X).w_termsig != 0)
#  undef  WEXITSTATUS
#  define WEXITSTATUS(_X) 	((_X).w_retcode)
#  undef  WTERMSIG
#  define WTERMSIG(_X)    	((_X).w_termsig)
#  undef  WCOREDUMP
#  define WCOREDUMP(_X)   	((_X).w_coredump)
#  undef  WSTOPSIG
#  define WSTOPSIG(_X)    	((_X).w_stopsig)
#endif  /* _BSD */

#endif	/* _SYS_WAIT_INCLUDED */
